/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.eepic;

import jpicedt.format.input.latex.*;

import jpicedt.format.input.util.*;

import jpicedt.graphic.model.*;
import jpicedt.graphic.PicPoint;

/**
 * Eepic "fast" (solid) polygon : \\path(x1,y1)...(xN,yN)
 * <p>
 * @author Sylvain Reynal
 * @since jpicedt 1.1
 * @version $Id: EepicPathCommand.java,v 1.5.2.1 2007/09/02 11:56:00 reynal Exp $
 * <p>
 */
public class EepicPathCommand extends SequenceExpression  implements ExpressionConstants {

    private Pool pool;

    public EepicPathCommand(Pool pl){

		super(true); // throw IncompleteSequence Exception
		this.pool = pl;
		add(new LaTeXInstanciationExpression("\\path", new PicMultiCurve(),pool)); 
		add(new RepeatExpression(
			new SequenceExpression(
					WHITE_SPACES_OR_EOL,
					new PicPointExpression("(",",",")"){
						public void action(ParserEvent e){
						    if (DEBUG) System.out.println(e);
						    PicPoint pt = (PicPoint)e.getValue();
						    ((PicMultiCurve)(pool.currentObj)).addPoint(pt.toMm(pool.getDouble(LaTeXParser.KEY_UNIT_LENGTH)));
							}
						}),
			2, AT_LEAST)
		);
    }
}
