/*
 LaTeXCircle.java - November 13, 2003 - jPicEdt, a picture editor for LaTeX.
 Copyright (C)  Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.format.input.latex;

import jpicedt.format.input.pstricks.PsRPutExpression;
import jpicedt.format.input.util.*;
import jpicedt.graphic.*;
import jpicedt.graphic.model.*;

/**
 * Parses LaTeX circles, i.e. \\circle or \\circle*. This is to be used as a child expression of either
 * {@link #LaTeXPutExpression LaTeXPutExpression} or 
 * {@link #jpicedt.format.input.pstricks.PsRPutExpression jpicedt.format.input.pstricks.PsRPutExpression}.
 * <p>
 * The parsing code uses either PsRPutExpression.KEY_RPUT_POINT or LaTeXPutExpression.KEY_PUT_POINT, depending
 * on the first non-null value found.
 */
public class LaTeXCircle extends SequenceExpression implements PicObjectConstants,ExpressionConstants {

	private Pool pool;

	public LaTeXCircle(Pool pl) {

		super(true);// throw IncompleteExpression
		this.pool = pl;

		this.add(new AlternateExpression(
				new LaTeXInstanciationExpression("\\circle*{", new PicEllipse(), pool){
			             public void action(ParserEvent e) {
				             super.action(e);
				             pool.currentObj.setAttribute(FILL_STYLE,SOLID);
			             }},
				new LaTeXInstanciationExpression("\\circle{", new PicEllipse(), pool)
			)
		);

			
		this.add(WHITE_SPACES_OR_EOL);
		
		this.add(
		    new NumericalExpression(DOUBLE, POSITIVE, "}", true) { // parse diameter
			    public void action(ParserEvent e) {
				    if (DEBUG) System.out.println(e);
				    PicEllipse ell = (PicEllipse)(pool.currentObj);
				    double diam = getValue().doubleValue() * pool.getDouble(LaTeXParser.KEY_UNIT_LENGTH);

					PicPoint putPoint = (PicPoint)pool.get(PsRPutExpression.KEY_RPUT_POINT);
					if (putPoint==null) putPoint = (PicPoint)pool.get(LaTeXPutExpression.KEY_PUT_POINT);
					if (putPoint==null) putPoint = new PicPoint();
					
				    // compute BL and TR corners :
				    PicPoint pBL = new PicPoint(putPoint); // center
				    pBL.translate(-diam/2.0, -diam/2.0);
				    PicPoint pTR = new PicPoint(putPoint); // center
				    pTR.translate(diam/2.0, diam/2.0);
				    ell.setPoint(PicEllipse.P_BL, pBL,null);
				    ell.setPoint(PicEllipse.P_TR, pTR,null);
			    }
		    }
		);
	}
}

