/*
 LaTeXInstanciationExpression.java - July 23, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C)  Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://trashx.ensea.fr/jpicedt/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.format.input.latex;

import jpicedt.graphic.model.*;
import jpicedt.format.input.util.*;

/**
 * instanciates a new Element by cloning the given object, when it finds the given tag,
 * then adds it to the current PicGroup ; makes use of the PicAttributeSet associated with
 * the ATTRIBUTES key in LaTeXParser.
 * @author Sylvain Reynal
 * @since jpicedt 1.3
 * @version $Id: LaTeXInstanciationExpression.java,v 1.6.2.1 2007/09/02 11:56:01 reynal Exp $
 *
 */
public class LaTeXInstanciationExpression extends LiteralExpression  implements ExpressionConstants {

	private Element prototype;
	private Pool pool;
	private String tag;

	/**
	 * @param literal tag expression to match
	 * @param prototype element to be instanciated by cloning
	 * @param pl pool where to add the instanciated element ; also used to fetch the current attribute set for PsTricks
	 */
	public LaTeXInstanciationExpression(String tag, Element prototype,Pool pl){

		super(tag);
		this.tag = tag;
		this.prototype = prototype;
		this.pool = pl;
	}

	public void action(ParserEvent e){

		if (DEBUG) System.out.println(e);
		pool.currentObj = (Element)prototype.clone();
		pool.currentObj.getAttributeSet().setAttributes(pool.getAttributeSet(LaTeXParser.KEY_ATTRIBUTES)); // overrides only non-default values
		pool.currentGroup.addChild(pool.currentObj);
	}

	public String toString(){

		return "[LaTeXInstanciationExpression:tag="+tag+" prototype="+prototype+"]";
	}
}

