/*
 LaTeXOval.java - November 13, 2003 - jPicEdt, a picture editor for LaTeX.
 Copyright (C)  Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.format.input.latex;

import jpicedt.format.input.pstricks.PsRPutExpression;
import jpicedt.format.input.util.*;
import jpicedt.graphic.*;
import jpicedt.graphic.model.*;

/**
 * Parses LaTeX \\oval command. This is to be used as a child expression of either
 * {@link #LaTeXPutExpression LaTeXPutExpression} or 
 * {@link #jpicedt.format.input.pstricks.PsRPutExpression jpicedt.format.input.pstricks.PsRPutExpression}.
 * <p>
 * \\oval(w,h) or \oval(w,h)[align] (for half- or quarter- ovals).
 * <p>
 * [pending] add "rounded corner" capacity to PicRectangle
 * @author Sylvain Reynal
 * @since jpicedt 1.4pre2
 * @version $Id: LaTeXOval.java,v 1.3.2.1 2007/09/02 11:56:01 reynal Exp $
 */
public class LaTeXOval extends SequenceExpression implements PicObjectConstants,ExpressionConstants {

	private Pool pool;
	
	public LaTeXOval(Pool pl) {

		super(true);// throw IncompleteExpression
		this.pool = pl;

		this.add(new LaTeXInstanciationExpression("\\oval", new PicParallelogram(), pool)); // instanciate a rectangle
		this.add(WHITE_SPACES_OR_EOL);
		this.add(
		    new PicPointExpression("(", ",", ")"){ // parse "(width,height)"
			    public void action(ParserEvent e) {
				    if (DEBUG) System.out.println(e);
				    PicPoint pt = (PicPoint)e.getValue();
				    PicParallelogram oval = (PicParallelogram)pool.currentObj;
				    double width = pt.x * pool.getDouble(LaTeXParser.KEY_UNIT_LENGTH);
				    double height = pt.y * pool.getDouble(LaTeXParser.KEY_UNIT_LENGTH);
					// set put point:
					PicPoint putPoint = (PicPoint)pool.get(PsRPutExpression.KEY_RPUT_POINT);
					if (putPoint==null) putPoint = (PicPoint)pool.get(LaTeXPutExpression.KEY_PUT_POINT);
					if (putPoint==null) putPoint = new PicPoint();
				    // compute BL and TR corners :
				    PicPoint pBL = new PicPoint(putPoint); // center
				    pBL.translate(-width/2.0, -height/2.0);
				    PicPoint pTR = new PicPoint(putPoint); // center
				    pTR.translate(width/2.0, height/2.0);
				    oval.setPoint(PicParallelogram.P_BL, pBL,null);
				    oval.setPoint(PicParallelogram.P_TR, pTR,null);
			    }
		    }
		);
		this.add(WHITE_SPACES_OR_EOL);
		// [pending] not used so far !!!
		SequenceExpression alignmentExp = new SequenceExpression(true);// throw IncompleteSequence Exception
		alignmentExp.add(new LiteralExpression("["));
		alignmentExp.add(new WordExpression("]",true)); // swallows chars till "]" is found
		this.add(new OptionalExpression(alignmentExp));
	}
}

