/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.latex;

import jpicedt.format.input.util.*;
import jpicedt.graphic.PicPoint;
import jpicedt.graphic.model.EditPointConstraint;

/**
 * an expression that parses "(x,y)" string (LaTeX-like PicPoint)
 * and gives the corresponding numerical values to the point number "ptNumber" of the current Element
 * using its <code>setPoint()</code>, using the given constraint. 
 * @see jpicedt.graphic.model.Element#setPoint
 * @author Sylvain Reynal
 * @since jpicedt 1.3
 * @version $Id: LaTeXPicPointExpression.java,v 1.5.2.1 2007/09/02 11:56:02 reynal Exp $
 *
 */
public class LaTeXPicPointExpression extends PicPointExpression implements ExpressionConstants {

	private int ptNumber;
	private EditPointConstraint constraint;
	private Pool pool;

	public LaTeXPicPointExpression(int ptNumber,Pool pl){
		this(ptNumber, null, pl);
	}

	public LaTeXPicPointExpression(int ptNumber,EditPointConstraint constraint, Pool pl){
		super("(",",",")");
		this.ptNumber=ptNumber;
		this.pool = pl;
		this.constraint=constraint;
	}

	public void action(ParserEvent e){
		if (DEBUG) System.out.println(e);
		PicPoint pt = getPicPoint();
		double unitLength = pool.getDouble(LaTeXParser.KEY_UNIT_LENGTH);
		pool.currentObj.setPoint(ptNumber,pt.toMm(unitLength),constraint);
	}
}
