/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.latex;

import jpicedt.format.input.util.*;
import jpicedt.graphic.model.*;

/**
 *	 Legal syntax :
 * <ul>
 * <li>	      \\linethickness{0.4pt}
 * <li>       \\linethickness{0.4mm}
 * <li>       \\linethickness{0.4cm}
 * <li>       \\linethickness{0.4} // default to mm
 * </ul>
 * note : 1pt = 1/72.27 inch = 0.3515 mm    cf. LaTeX Book (Leslie Lamport) p.192
 * @author Sylvain Reynal
 * @since jpicedt 1.3
 * @version $Id: LineThicknessExpression.java,v 1.5.2.1 2007/09/02 11:56:02 reynal Exp $
 *
 */
public class LineThicknessExpression extends SequenceExpression  implements ExpressionConstants,PicObjectConstants {

    private Pool pool;
    private double lineWidth;

    public LineThicknessExpression(Pool pl){
		
		super(true); // throw IncompleteSequence Exception
		pool = pl;

		// \\linethickness{val}
		add(new LiteralExpression("\\linethickness{"));
		add(WHITE_SPACES_OR_EOL);
		add(new NumericalExpression(DOUBLE, POSITIVE, null, false){ 
			public void action(ParserEvent e){
				if (DEBUG) System.out.println(e);
				lineWidth = ((Number)e.getValue()).doubleValue();
				pool.getAttributeSet(LaTeXParser.KEY_ATTRIBUTES).setAttribute(LINE_WIDTH,e.getValue());
		}});
		// no white-space here !
		add(new WordExpression("}", true){ // postfix="}", swallow
			public void action(ParserEvent e){
				if (DEBUG) System.out.println(e);	
				String s= (String)e.getValue();
				if (s.equals("pt")) {
					lineWidth *= PS_POINT; 
					pool.getAttributeSet(LaTeXParser.KEY_ATTRIBUTES).setAttribute(LINE_WIDTH,new Double(lineWidth)); 
					return;
				}
				if (s.equals("cm")) {
					lineWidth *= 10; 
					pool.getAttributeSet(LaTeXParser.KEY_ATTRIBUTES).setAttribute(LINE_WIDTH,new Double(lineWidth)); 
					return;
				}
				// "mm" ? -> unchanged
		}});
    }
	
	public String toString(){
		return "[LineThicknessExpression]";
	}
	
} // LineThicknessExpression
