/*
 PicArrowTypeExpression.java - July 23, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C)  Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.format.input.latex;

import jpicedt.format.input.util.*;
import jpicedt.graphic.model.*;

/**
 * "0/1 0/1" arrow type (we take for granted that this string is followed by a LaTeX-picPoint, as in
 * %Line 0 1 (2.3,4.5)..., possibly with leading whitespaces)
 * @author Sylvain Reynal
 * @since jpicedt 1.3
 * @version $Id: PicArrowTypeExpression.java,v 1.5.2.1 2007/09/02 11:56:02 reynal Exp $
 *
 */
public class PicArrowTypeExpression extends WordExpression  implements ExpressionConstants, PicObjectConstants {

    private Pool pool;

    public PicArrowTypeExpression(Pool pl){
		super("(", false); // postfix="(" (start of PicPoint), not swallowed
		pool = pl;
    }

    public void action(ParserEvent e){
	
		if (DEBUG) System.out.println(e);
		String s = (String)e.getValue();
		if (s.startsWith("0 1")) {
			pool.currentObj.setAttribute(LEFT_ARROW, Arrow.NONE);
			pool.currentObj.setAttribute(RIGHT_ARROW, Arrow.ARROW_HEAD);
			return;
		}
		else if (s.startsWith("1 0")) {
			pool.currentObj.setAttribute(LEFT_ARROW, Arrow.ARROW_HEAD);
			pool.currentObj.setAttribute(RIGHT_ARROW, Arrow.NONE);
			return;
		}
		else if (s.startsWith("1 1")) {
			pool.currentObj.setAttribute(LEFT_ARROW, Arrow.ARROW_HEAD);
			pool.currentObj.setAttribute(RIGHT_ARROW, Arrow.ARROW_HEAD);
			return;
		}
		else {
			pool.currentObj.setAttribute(LEFT_ARROW, Arrow.NONE);
			pool.currentObj.setAttribute(RIGHT_ARROW, Arrow.NONE);
		}
    }
}
