/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.latex;

import jpicedt.format.input.util.*;
import jpicedt.graphic.model.*;
import jpicedt.graphic.PicPoint;

/**
 * PicEllipse :
 * %Ellipse(x0,y0)(width)(height)
 * %Ellipse(x0,y0)(width,height) filled|whiten|blacken|shade arcStart= arcExtent=
 * Any string (multiput...)
 * %End Ellipse
 */
public class PicEllipseExpression extends SequenceExpression  implements ExpressionConstants {

    private Pool pool;
	private PicPoint pCenter=new PicPoint();
	private double width;

    public PicEllipseExpression(Pool pl){

		super(true); // throw IncompleteSequence Exception
		pool = pl;

		add(new LaTeXInstanciationExpression("%Ellipse", new PicEllipse(),pool));
		add(WHITE_SPACES);
		add(new OptionalExpression(new PicArrowTypeExpression(pool)));
		add(new CenterPicPointExpression()); // save (x0,y0) in pCenter
		add(WHITE_SPACES);

		// width :
		add(new LiteralExpression("(")); 
		add(new NumericalExpression(DOUBLE,POSITIVE,")",true){
			public void action(ParserEvent e){
				if (DEBUG) System.out.println(e);
				width = ((Double)e.getValue()).doubleValue() * pool.getDouble(LaTeXParser.KEY_UNIT_LENGTH);
		}});
		add(WHITE_SPACES);
	
		// height :
		add(new LiteralExpression("(")); 
		add(new NumericalExpression(DOUBLE,POSITIVE,")",true){ 
			public void action(ParserEvent e){
				if (DEBUG) System.out.println(e);
				double height = ((Double)e.getValue()).doubleValue() * pool.getDouble(LaTeXParser.KEY_UNIT_LENGTH);
				PicPoint pBL = new PicPoint(pCenter).translate(-width/2., -height/2.);
				PicPoint pTR = new PicPoint(pCenter).translate(width/2., height/2.);
				((PicEllipse)(pool.currentObj)).setPoint(PicEllipse.P_BL,pBL,null); // no constraint
				((PicEllipse)(pool.currentObj)).setPoint(PicEllipse.P_TR,pTR,null);
		}});
		add(WHITE_SPACES);

		AlternateExpression paramAlt = new AlternateExpression();
		// dash :
		paramAlt.add(new PicDashStatement(pool)); 
		// arcStart = ?
		paramAlt.add(new StatementExpression("arcStart","=",null, DOUBLE, ANY_SIGN){ 
			public void action(ParserEvent e){
				if (DEBUG) System.out.println(e);
				((PicEllipse)(pool.currentObj)).setAngleStart(((Double)e.getValue()).doubleValue());
		}});
		// arcExtent = ?
		paramAlt.add(new StatementExpression("arcExtent","=",null, DOUBLE, ANY_SIGN){ 
			public void action(ParserEvent e){
				if (DEBUG) System.out.println(e);
				((PicEllipse)(pool.currentObj)).setAngleExtent(((Double)e.getValue()).doubleValue());
		}});
		// fillcolor (WHITEN,...)
		paramAlt.add(new PicColorExpression(pool)); 
		OptionalExpression paramOpt = new OptionalExpression(paramAlt);
		add(paramOpt);
		add(WHITE_SPACES);
		add(paramOpt);
		add(WHITE_SPACES);
		add(paramOpt);
		add(WHITE_SPACES);
		add(paramOpt);
		add(new PicEndExpression("%End Ellipse"));
    }
	
	public String toString(){
		return "[PicEllipseExpression]";
	}
	
	/** store the ellipse center in a private class field for later processing */
	class CenterPicPointExpression extends PicPointExpression {
		public CenterPicPointExpression(){
			super("(",",",")");
		}
		
		public void action(ParserEvent e){
			if (DEBUG) System.out.println(e);
			PicPoint pt = getPicPoint();
			double unitLength = pool.getDouble(LaTeXParser.KEY_UNIT_LENGTH);
			pCenter.setCoordinates(pt.toMm(unitLength));
		}
	}
	
} // PicEllipseExpression
