/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.latex;

import jpicedt.format.input.util.*;
import java.util.*;
import java.util.regex.*;

/**
 * Match "%End XXXX"-like expressions at the beginning of a line (or preceeded by whitespaces), 
 * after skipping as many lines as necessary.
 */
public class PicEndExpression extends AbstractRegularExpression {

	private String literal;
	private Pattern pattern;

	/**
	 * @param literal e.g. "%End Line"
	 */
	public PicEndExpression(String literal){
		try {
			this.pattern = Pattern.compile("\n *" + literal);
		}
		catch (PatternSyntaxException e){
			e.printStackTrace();
		}
		this.literal = literal;
	}

	/**
	 * @return TRUE in any case, else throw an exception (@EOF) ; (this means that exception throwing is
	 * the only way to get out of the this expression...)
	 * in case of success, call action() with value=leading String
	 * @throws REParserException.NotFoundInFile if EOF was reached before this expression could be found
	 */
	public boolean interpret(Context context) throws REParserException {

		context.mark();
		String skipped = context.find(pattern); // move caret past matched string if found
		if (skipped != null){
			action(new ParserEvent(this, context, true, skipped));
			return true;
		}
		throw new REParserException.NotFoundInFile(context, this); //EOF=>stop here
	}

	public String toString(){
		return "[PicEndExpression:" + literal + "]";
	}
}
