/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.latex;

import jpicedt.format.input.util.*;
import jpicedt.graphic.model.*;

/**
 * PicGroup:<br>
 * %Begin group<br>
 * ...<br>
 * %End group<br>
 * @author Sylvain Reynal
 * @since jpicedt 1.3
 * @version $Id: PicGroupExpression.java,v 1.5.2.1 2007/09/02 11:56:08 reynal Exp $
 *
 */
public class PicGroupExpression extends AlternateExpression  implements ExpressionConstants {

    private Pool pool;

    public PicGroupExpression(Pool pl){

		pool = pl;
	
		add(new LiteralExpression("%Begin group"){
			public void action(ParserEvent e){
				if (DEBUG) System.out.println(e);
				pool.picGroupStack.push(pool.currentGroup);
				pool.currentGroup = new PicGroup();
			}
	    });
		add(new LiteralExpression("%End group"){
			public void action(ParserEvent e) throws REParserException {
				if (pool.picGroupStack.empty()) throw new REParserException.EndGroupMismatch(e.getContext(), e.getSource());
				// save current PicGroup
				PicGroup obj = pool.currentGroup;
				// recall old (i.e. parent) PicGroup
				pool.currentGroup = (PicGroup)pool.picGroupStack.pop();
				// add current PicGroup to parent
				pool.currentGroup.addChild(obj);
			}
	    });
    }
}
