/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.latex;

import jpicedt.format.input.util.*;
import jpicedt.graphic.model.PicObjectConstants;

/**
 * \\unitlength value<br>
 * \\setlength{\\unitlength}{value}<br>		
 * <br>
 * where value's permitted syntaxs are : 0.11, 0.11mm, 0.11cm, 0.11pt with possible leading whitespaces before the "unit".<br>
 * @author Sylvain Reynal
 * @since jpicedt 1.3
 * @version $Id: UnitLengthExpression.java,v 1.5.2.1 2007/09/02 11:56:09 reynal Exp $
 *
 */
public class UnitLengthExpression extends SequenceExpression  implements ExpressionConstants,PicObjectConstants {

	private Pool pool;
	private double unitLength;

	public UnitLengthExpression(Pool pl){

		super(true); // throw IncompleteSequence Exception

		this.pool = pl;

		// "\\unitlength" or "\\setlength{\\unitlength}{":
		add(new AlternateExpression(
		        new LiteralExpression("\\unitlength "),
		        new LiteralExpression("\\setlength{\\unitlength}{")));
		// numerical value:
		add(new NumericalExpression(DOUBLE, POSITIVE, null, false){ // no postfix, but possibly whitespaces before
			    public void action(ParserEvent e){
				    if (DEBUG) System.out.println(e);
					unitLength = ((Number)e.getValue()).doubleValue();
				    pool.put(LaTeXParser.KEY_UNIT_LENGTH,unitLength);
		}});
		add(WHITE_SPACES);
		// unit: "cm" or "pt" or "mm" or nothing (which is the same as "mm")
		add(new OptionalExpression(new AlternateExpression(
		                               new LiteralExpression("mm"), // do nothing ! this is the default
		                               new LiteralExpression("cm"){
			                               public void action(ParserEvent e){
				                               if (DEBUG) System.out.println(e);
				                               unitLength *= 10;
											   pool.put(LaTeXParser.KEY_UNIT_LENGTH,unitLength);
									   }},
		                               new LiteralExpression("pt"){
			                               public void action(ParserEvent e){
				                               if (DEBUG) System.out.println(e);
				                               unitLength *= PS_POINT;
											   pool.put(LaTeXParser.KEY_UNIT_LENGTH,unitLength);
									   }}))
		   );
		// optional trailing "}" in case \\setlength was found
		add(new OptionalExpression(new LiteralExpression("}")));
	}
	
	public String toString(){
		return "[UnitLengthExpression]";
	}
	
} // UnitLengthExpression
