/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.pstricks;

import jpicedt.format.input.util.*;
import jpicedt.graphic.io.formatter.*;
import jpicedt.graphic.model.*;

/**
 * Used by PSTParametersExpression to parse statements involving angles, for instance
 * "hatchangle=45" or "shadowangle=90".
 * @author Sylvain Reynal
 * @version $Id: PSTAngleParameter.java,v 1.4.2.1 2007/09/02 11:56:11 reynal Exp $ 
 */
public class PSTAngleParameter extends StatementExpression implements PicObjectConstants {

	private Pool pool;
	private PicAttributeName attribute;
	private Pool.Key attributeSetKey;
	private double val; // temporary stores parsed value

	/**
	 * @param pl parser's pool
	 * @param tag LHS tag (e.g. "hatchangle" or "shadowangle") for the StatementExpression
	 * @param attribute name of attribute to modify (must be a predefined PicAttributeName of type "Double" or at least "Number")
	 * @param attributeSetKey used to fetch the attribute set in which parsed parameters are stored. 
	 */
	public PSTAngleParameter(Pool pl, String tag, PicAttributeName attributeName, Pool.Key attributeSetKey){

		super(tag,"=",null,DOUBLE,ANY_SIGN);
		pool = pl;
		attribute = attributeName;
		this.attributeSetKey = attributeSetKey;

	}

	public void action(ParserEvent e){
		if(DEBUG) System.out.println(e);
		val = ((Number)e.getValue()).doubleValue(); // in degrees
		pool.setAttribute(attributeSetKey,attribute, new Double(val));
	}
} // class
