/*
 *  jPicEdt, a picture editor for LaTeX.
 *  Copyright (C) 1999-2007  Sylvain Reynal
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  Sylvain Reynal
 *  Département de Physique
 *  Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 *  6, avenue du Ponceau
 *  95014 CERGY CEDEX
 *  FRANCE
 *  Tel : 00 +33 130 736 245
 *  Fax : 00 +33 130 736 667
 *  e-mail : reynal@ensea.fr
 *  jPicEdt web page : http://www.jpicedt.org
 */
package jpicedt.format.input.pstricks;

import jpicedt.format.input.util.*;
import jpicedt.graphic.io.formatter.*;
import jpicedt.graphic.model.*;

/**
 * Parses statements similar to "dash=1pt 3mm" (first opaque, then transparent).
 * @author Sylvain Reynal
 * @version $Id: PSTDashExpression.java,v 1.4.2.1 2007/09/02 11:56:12 reynal Exp $ 
 */
public class PSTDashExpression extends SequenceExpression implements PicObjectConstants {

	private Pool pool;
	private Pool.Key setKey;
	private double val;// temporary stores parsed value


	/**
	 * @param pl parser's pool
	 * @param attributeSetKey used to fetch the attribute set in which parsed parameters are stored. 
	 */
	public PSTDashExpression(Pool pl, Pool.Key attributeSetKey) {

		super(false);
		pool = pl;
		this.setKey = attributeSetKey;

		// "dash=3pt" ... (opaque dash)
		add(
			new StatementExpression("dash", "=", null, DOUBLE, POSITIVE) {// no postfix, but possibly whitespaces before
				public void action(ParserEvent e) {
					if (DEBUG) System.out.println(e);
					val = pool.getDouble(PstricksParser.KEY_R_UNIT) * ((Number)e.getValue()).doubleValue();// suppose there's no unit
					pool.setAttribute(setKey,DASH_OPAQUE, new Double(val));
				}
			});
		// now, if there's a specified unit, we rescale to parsed value (i.e. dividing by pstRunit), then we compute the actual length value depending on the unit found
		add(new OptionalExpression(
			new AlternateExpression(
				new LiteralExpression("cm") {
					public void action(ParserEvent e) {
						if (DEBUG)
							System.out.println(e);
							val = val * 10.0 / pool.getDouble(PstricksParser.KEY_R_UNIT);
							pool.setAttribute(setKey,DASH_OPAQUE, new Double(val));
					}
				},
				new LiteralExpression("mm") {
					public void action(ParserEvent e) {
						if (DEBUG)
							System.out.println(e);
							val = val / pool.getDouble(PstricksParser.KEY_R_UNIT);
							pool.setAttribute(setKey,DASH_OPAQUE, new Double(val));
					}
				},
				new LiteralExpression("pt") {
					public void action(ParserEvent e) {
						if (DEBUG)
							System.out.println(e);
							val = val * PS_POINT / pool.getDouble(PstricksParser.KEY_R_UNIT);
							pool.setAttribute(setKey,DASH_OPAQUE, new Double(val));
					}
			})));
		// ... " 5mm" (transparent dash)
		add(WHITE_SPACES_OR_EOL);
		add(
			new NumericalExpression(DOUBLE, POSITIVE, null, false) {
				public void action(ParserEvent e) {
					if (DEBUG)
						System.out.println(e);
					val = pool.getDouble(PstricksParser.KEY_R_UNIT) * ((Number)e.getValue()).doubleValue();// suppose there's no unit
					pool.setAttribute(setKey,DASH_TRANSPARENT, new Double(val));
				}
			});
		add(new OptionalExpression(new AlternateExpression(
			new LiteralExpression("cm") {
				public void action(ParserEvent e) {
					if (DEBUG)
						System.out.println(e);
					val = val * 10.0 / pool.getDouble(PstricksParser.KEY_R_UNIT);
					pool.setAttribute(setKey,DASH_TRANSPARENT, new Double(val));
				}
			},
			new LiteralExpression("mm") {
				public void action(ParserEvent e) {
					if (DEBUG)
						System.out.println(e);
					val = val / pool.getDouble(PstricksParser.KEY_R_UNIT);
					pool.setAttribute(setKey,DASH_TRANSPARENT, new Double(val));
				}
			},
			new LiteralExpression("pt") {
				public void action(ParserEvent e) {
					if (DEBUG)
						System.out.println(e);
					val = val * PS_POINT / pool.getDouble(PstricksParser.KEY_R_UNIT);
					pool.setAttribute(setKey,DASH_TRANSPARENT, new Double(val));
				}
			})));

	}
}

