/*
 *  jPicEdt, a picture editor for LaTeX.
 *  Copyright (C) 1999-2007  Sylvain Reynal
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  Sylvain Reynal
 *  Département de Physique
 *  Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 *  6, avenue du Ponceau
 *  95014 CERGY CEDEX
 *  FRANCE
 *  Tel : 00 +33 130 736 245
 *  Fax : 00 +33 130 736 667
 *  e-mail : reynal@ensea.fr
 *  jPicEdt web page : http://www.jpicedt.org
 */
package jpicedt.format.input.pstricks;

import jpicedt.format.input.util.*;
import jpicedt.graphic.io.formatter.*;
import jpicedt.graphic.model.*;
import java.awt.*;

/**
 * Parses statements similar to "fillstyle=style" where style is one of PicObjectConstants predefined fill styles.
 * @author Sylvain Reynal
 * @version $Id: PSTFillStyleExpression.java,v 1.4.2.1 2007/09/02 11:56:12 reynal Exp $ 
 */
public class PSTFillStyleExpression extends SequenceExpression implements PicObjectConstants {

	private Pool pool;
	private Pool.Key attributeSetKey;


	/**
	 * @param pl parser's pool
	 * @param attributeSetKey used to fetch the attribute set in which parsed parameters are stored. 
	 */
	public PSTFillStyleExpression(Pool pl, Pool.Key attributeSetKey) {

		super(false);// doesn't throw IncompleteSequenceException

		pool = pl;
		this.attributeSetKey = attributeSetKey;
	}
	
	public boolean interpret(Context context) throws REParserException { 
		
		if (!context.matchAndMove("fillstyle=")) return false;
		WHITE_SPACES_OR_EOL.interpret(context);
		if (context.matchAndMove("none")) pool.setAttribute(attributeSetKey, FILL_STYLE,NONE);
		else if (context.matchAndMove("solid")) pool.setAttribute(attributeSetKey, FILL_STYLE,SOLID);
		else if (context.matchAndMove("vlines*")) pool.setAttribute(attributeSetKey, FILL_STYLE,VLINES_FILLED);
		else if (context.matchAndMove("vlines")) pool.setAttribute(attributeSetKey, FILL_STYLE,VLINES);
		else if (context.matchAndMove("hlines*")) pool.setAttribute(attributeSetKey, FILL_STYLE,HLINES_FILLED);
		else if (context.matchAndMove("hlines")) pool.setAttribute(attributeSetKey, FILL_STYLE,HLINES);
		else if (context.matchAndMove("crosshatch*")) pool.setAttribute(attributeSetKey, FILL_STYLE,CROSSHATCH_FILLED);
		else if (context.matchAndMove("crosshatch")) pool.setAttribute(attributeSetKey, FILL_STYLE,CROSSHATCH);
		else throw new REParserException.SyntaxError(context,this);
		return true;
	}
}

