/*
 *  jPicEdt, a picture editor for LaTeX.
 *  Copyright (C) 1999-2007  Sylvain Reynal
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  Sylvain Reynal
 *  Département de Physique
 *  Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 *  6, avenue du Ponceau
 *  95014 CERGY CEDEX
 *  FRANCE
 *  Tel : 00 +33 130 736 245
 *  Fax : 00 +33 130 736 667
 *  e-mail : reynal@ensea.fr
 *  jPicEdt web page : http://www.jpicedt.org
 */
package jpicedt.format.input.pstricks;

import jpicedt.format.input.util.*;
import jpicedt.graphic.io.formatter.*;
import jpicedt.graphic.model.*;
import java.awt.*;

/**
 * Parses statements similar to "linestyle=style" where style is one of PicObjectConstants predefined line styles.
 * @author Sylvain Reynal
 * @version $Id: PSTLineStyleExpression.java,v 1.4.2.1 2007/09/02 11:56:12 reynal Exp $ 
 */
public class PSTLineStyleExpression extends SequenceExpression implements PicObjectConstants {

	private Pool pool;
	private Pool.Key setKey;


	/**
	 * @param attributeSetKey used to fetch the attribute set in which parsed parameters are stored. 
	 */
	public PSTLineStyleExpression(Pool pl, Pool.Key attributeSetKey) {

		super(false);// doesn't throw IncompleteSequenceException (which means that there might be nothing b/w [ and ])

		pool = pl;
		this.setKey = attributeSetKey;

		add(new LiteralExpression("linestyle="));
		add(WHITE_SPACES_OR_EOL);
		add(
			new WordExpression(null, false) {// no postfix, swallows as many letters as possible, no digit
				public void action(ParserEvent e) {
					if (DEBUG) System.out.println(e);
					String style = (String)(e.getValue());
					if (style.equals("none"))
						pool.setAttribute(setKey,LINE_STYLE, NONE);
					else if (style.equals("dotted"))
						pool.setAttribute(setKey,LINE_STYLE, DOTTED);
					else if (style.equals("dashed"))
						pool.setAttribute(setKey,LINE_STYLE, DASHED);
					else
						pool.setAttribute(setKey,LINE_STYLE, SOLID);// "solid"
				}
			});
	}
}

