/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.pstricks;

import jpicedt.format.input.util.*;
import jpicedt.graphic.model.*;
import jpicedt.graphic.PicPoint;

/**
 * Quick disk : (streamlines version of \\pscircle*),
 * <b>\\qdisk(x0,y0)(rad)</B>
 * @author Sylvain Reynal
 * @version $Id: PsQDiskExpression.java,v 1.5.2.1 2007/09/02 11:56:12 reynal Exp $ 
 */
public class PsQDiskExpression extends SequenceExpression implements PicObjectConstants {

	private Pool pool;

	/**
	 * Uses default tag.
	 */
	public PsQDiskExpression(Pool pl){
		this(pl,null);
	}
	
	/**
	 * @param tag if null, default to \\qdisk
	 */
	public PsQDiskExpression(Pool pl, String tag){

		super(true); // throw IncompleteSequence Exception
		pool = pl;
		if (tag == null) tag = "\\qdisk";

		add(new PSTInstanciationExpression(tag, new PicEllipse(),pool));
		add(WHITE_SPACES_OR_EOL);
		// circle's center
		add(new PSTPicPointExpression(PicEllipse.P_CENTER,pool));
		add(WHITE_SPACES_OR_EOL);
		// radius :
		add(new LiteralExpression("{"));
		add(WHITE_SPACES_OR_EOL);
		add(new NumericalExpression(DOUBLE,POSITIVE,"}",true){
			    public void action(ParserEvent e){
				    if (DEBUG) System.out.println(e);
					double radius = ((Double)e.getValue()).doubleValue() * pool.getDouble(PstricksParser.KEY_R_UNIT);
				    // recall center in order to build BL and TR corners :
					PicPoint pBL = pool.currentObj.getPoint(PicEllipse.P_CENTER,null);
					PicPoint pTR = new PicPoint(pBL);
					pBL.translate(-radius, -radius);
					pTR.translate(radius, radius);
				    ((PicEllipse)(pool.currentObj)).setPoint(PicEllipse.P_BL, pBL,null);
					((PicEllipse)(pool.currentObj)).setPoint(PicEllipse.P_TR, pTR,null);
			    }});
	}
	public String toString(){
		return "[PsQDiskExpression]";
	}
}
