/*  AbstractRegularExpression.java - jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.util;


/** 
 * This is the abstract superclass for all regular expressions that may help building a RegExp-based parser.
 * It mimics the RegExp scheme, while at the same time allowing expressions to send ParserEvent's when
 * a piece of input text was successfully parsed (that is, instead of delegating to a separate ParserHandler through
 * a <i>callback</i> mechanism,  events are handled in the core of the <code>action</code> method). 
 * A <code>Context</code> is then used to feed successive pieces of text to the set of reg-exp's that build up 
 * the grammar tree,
 * and a <code>Pool</code> allow reg-exp's to
 * share data across the whole grammar tree.
 * @since jpicedt 1.3
 * @author Sylvain Reynal
 * @version $Id: AbstractRegularExpression.java,v 1.4.2.1 2007/09/02 11:56:13 reynal Exp $
 *
 */
public abstract class AbstractRegularExpression implements ExpressionConstants {

    /**
     * Parses this expression, possibly using the given Context to fetch the String to interpret
	 * if this Expression is a leaf expression.
     * @return TRUE if parsing was successful
     * @throws REParserException if an error occur during parsing
     */
    public abstract boolean interpret(Context c) throws REParserException;

    /**
     * Called in the course of the <i>interpret</i> operation at the end of a SUCCESSFUL interpret operation. 
     * Should be overriden by daughter classes to process proper action, e.g. set Element's attributes...
     * Current implementation does nothing, but sending a debugging string.
     */
    public void action(ParserEvent e) throws REParserException {if (DEBUG) System.out.println(e);}

} 
