/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.util;

import java.util.*;

/** 
 * An expression specified by a String to be exactly matched at the current cursor position.
 * @since jpicedt 1.3
 * @author Sylvain Reynal
 * @version $Id: LiteralExpression.java,v 1.4.2.1 2007/09/02 11:56:13 reynal Exp $
 *
 */
public class LiteralExpression extends AbstractRegularExpression {

    private String expr;

    /**
     * @param expr The string to be matched
     */
    public LiteralExpression(String expr){
		this.expr=expr;
    }

    /**
     * @return TRUE if expr has been found
     * Calls "action" with value=given string
     */
    public boolean interpret(Context c) throws REParserException {

		if (c.matchAndMove(expr)) {     //  <-- current substring contains expr
			action(new ParserEvent(this, c, true,  expr));
			return true;
		}
		//action(new ParserEvent(this, c, false, expr));
		return false;
    }

    /**
     *
     */
    public String toString(){
		return "[Literal:\"" + expr + "\"]";
    }
}
