/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.util;

/**
 * An event that gets sent as an argument of the "action" method during an interpret operation
 * @since jpicedt 1.3
 * @author Sylvain Reynal
 * @version $Id: ParserEvent.java,v 1.5.2.1 2007/09/02 11:56:14 reynal Exp $
 *
 */
public class ParserEvent {

	private AbstractRegularExpression source;
	private Context context;
	private boolean parsingSuccessful;
	private Object value;

	public ParserEvent(AbstractRegularExpression source,
	                   Context context,
	                   boolean parsingSuccessful,
	                   Object value){

		this.source=source;
		this.context=context;
		this.parsingSuccessful=parsingSuccessful;
		this.value=value;
	}

	/**
	 *
	 */
	public String toString(){

		String vv = value.toString().replace('\n','|');
		return "{ParserEvent:class=" + source.getClass().getName()
		       + "\n             context=" + context
		       + "\n             value=\"" + vv + "\""
		       	   + "\n             source=" + source
		       + "\n}";
	}

	/**
	 * @return the Expression that sourced this event 
	 */
	public AbstractRegularExpression getSource(){
		return source;
	}

	/**
	 * @return the Context that is attached to current the parsing operation
	 */
	public Context getContext(){
		return context;
	}

	/**
	 * @return TRUE if this event was emitted as a result of a successful operation
	 */
	public boolean isSuccessful(){
		return parsingSuccessful;
	}

	/**
	 * @return a value that generally identifies the result of the parsing operation, such as : number, word, ...
	 */
	public Object getValue(){ 
		return value;
	}
}
