/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.util;

/**
 * a RegExp that represents a single occurence of a wild-char, i.e. 
 * ANY_CHAR, EOL or ANY_CHAR_EOL (as defined in <a href="ExpressionConstants.html">ExpressionConstants</a>
 * or one of the predefinite fields in java.lang.Character.
 * @since jpicedt 1.3
 * @author Sylvain Reynal
 * @version $Id: WildCharExpression.java,v 1.5.2.1 2007/09/02 11:56:15 reynal Exp $
 *
 */
public class WildCharExpression extends AbstractRegularExpression {

    private int type;
	private Character C;

    /**
     * @param type one of predefinite types available in class java.lang.Character. 
	 *        May also be ANY_CHAR (any character), EOL (end of line), or ANY_CHAR_EOL (a combination of both).
     */
    public WildCharExpression(int type){
		this.type=type;
    }

    public int getType(){
		return type;
	}

    /**
     * @return TRUE if the first character of the remaining substring is a wild-char of the permitted type
     * If TRUE, send a ParserEvent with value=char found ('\n' in case of an EOL)
     */
    public boolean interpret(Context context) throws REParserException{

		C = context.read();
		if (C == null) return false; // at EOF or at end-of-block
		char c = C.charValue();
		// EOL ?
		if (c=='\n'){
			if (type==EOL || type==ANY_CHAR_EOL) {
				action(new ParserEvent(this, context, true, new Character(c)));
				return true;
			}
			else {
				context.pushBack();
				return false;
			}
		}
			
		// otherwise, we compare the current char with the given wildchar type, and proceed further
		// by sending the appropriate ParserEvent :
		if (type==ANY_CHAR || type==ANY_CHAR_EOL || Character.getType(c) == this.type) {
			action(new ParserEvent(this, context, true, C));
			return true;
		}
		// no match:
		context.pushBack();
		return false;
    }

    /**
     * Return the last character parsed.
     */
	public Character getCharacter(){
		return C;
	}
	 
	 
    public String toString(){

		String charType;
		switch(type){
			case ANY_CHAR:
				charType="ANY_CHAR";
				break;
			case EOL:
				charType="EOL";
				break;
			case ANY_CHAR_EOL:
				charType="ANY_CHAR_EOL";
				break;
			default:
				charType=new Integer(type).toString();
		}
		return "[WildChar:" +  charType + "]";
    }
}

