/*
 PicParallelogramFormatter.java - August 29, 2003 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.format.output.eepic;

import jpicedt.graphic.*;
import jpicedt.graphic.model.*;
import jpicedt.graphic.io.formatter.*;
import jpicedt.graphic.PicPoint;

import java.awt.*;

/**
 * Formats a PicParallelogram to the eepic package, using \\path macros, with appropriate filling if any,
 * or \\dashline macros if dash is positive.
 * @author Sylvain Reynal
 * @since jpicedt 1.4
 * @version $Id: PicParallelogramFormatter.java,v 1.3.2.1 2007/09/02 11:55:13 reynal Exp $
 * <p>
 */
public class PicParallelogramFormatter implements Formatter, PicObjectConstants,EepicConstants {

	/** the Element this formater acts upon */
	protected PicParallelogram element;
	/** the producing factory */
	protected EepicFormatter factory;

	public PicParallelogramFormatter(PicParallelogram element, EepicFormatter factory){
		this.element = element;
		this.factory=factory;
	}

	public String format(){

		StringBuffer buf = new StringBuffer(100);
		PicAttributeSet set = element.getAttributeSet();
		
		PicPoint ptTR = element.getPoint(PicParallelogram.P_TR,null);
		PicPoint ptBL = element.getPoint(PicParallelogram.P_BL,null);
		PicPoint ptTL = element.getPoint(PicParallelogram.P_TL,null);
		PicPoint ptBR = element.getPoint(PicParallelogram.P_BR,null);

		factory.appendThicknessString(buf,element);

		if (StyleConstants.getFillStyle(set)==SOLID){
			Color fillColor = StyleConstants.getFillColor(set);
			if (fillColor.equals(Color.WHITE)) buf.append("\\whiten");
			else if (fillColor.equals(Color.BLACK)) buf.append("\\blacken");
			else buf.append("\\shade"); // SHADE
			buf.append("\\path"); 
			buf.append(ptBL);
			buf.append(ptBR);
			buf.append(ptTR);
			buf.append(ptTL);
			buf.append(ptBL); 
			buf.append(CR_LF);
		}
		
		// else non-filled, and we handle the non-zero dash case as well :
		else {
			double dash=(StyleConstants.getDashOpaque(set)+StyleConstants.getDashTransparent(set))/2.0;
			if (StyleConstants.getLineStyle(set)!=DASHED) dash=0;// [pending] add support for black/white dash + dotted
			if (dash <= 0){
				buf.append("\\path");
				buf.append(ptBL);
				buf.append(ptBR);
				buf.append(ptTR);
				buf.append(ptTL);
				buf.append(ptBL);
				buf.append(CR_LF);
			}
			else {
				factory.appendDashLine(buf, ptBL, ptBR, dash);
				factory.appendDashLine(buf, ptBR, ptTR, dash);
				factory.appendDashLine(buf, ptTR, ptTL, dash);
				factory.appendDashLine(buf, ptTL, ptBL, dash);
			}
		}
		buf.append(CR_LF);
		return buf.toString();
	}

} 
