/*
 LatexConstants.java - March 1, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
package jpicedt.format.output.latex;

/** 
 * "emulated latex" parameters : these parameters get used by LatexFormatter, LatexParser
 *  and LatexViewFactory.
 * <ul>
 * <li>lineThickness : (mm) ; used when parsing LaTeX file, when no \\lineThickness command is found
 * <li>emLineLength : (mm) when in "em. latex" mode, a line whose slope doesn't match an existing
 * <li>LaTeX slope (cf. picture environment) is emulated by tiny horizontal and/or
 * vertical segments ; this parameter determines the length of these segments
 * <li>maxLatexCircleDiameter (mm) : circles whose diameter is greater than this value are emulated ;
 *                                        this ususally depends on available "lasym" fonts
 * <li>maxLatexDiskDiameter (mm) : same thing, but for disks now.
 * <li>maxEmLineSlope : line with an absolute slope greater than this value are considered to be vertical, i.e. "line(0,1){etc...}"
 * <li>minEmLineSlope : same thing, but for horizontal lines.
 * <li>fileWrapperProlog and fileWrapperEpilog : things to prepend and append before and after
 *     "begin{picture}" and "end{picture}" to create a content able to be compiled by latex. May include
 *     things like "include{package}",... 
 */
public interface LatexConstants {
	
	/**
	 * Line separator for the current platform (i.e. "\n" on Unix, "\n\r" on Windows, "\r" on Mac,..)
	 */
	public static final String CR_LF = System.getProperty("line.separator");

	//////////////////////////////
	// KEY for PROPERTIES 
	/////////////////////////////
	public static final String KEY_EM_LINE_LENGTH = "latex.emulated-line-length";
	public static final String KEY_MAX_CIRCLE_DIAMETER = "latex.max-circle-diameter";
	public static final String KEY_MAX_DISK_DIAMETER ="latex.max-disk-diameter";
	public static final String KEY_MAX_EM_CIRCLE_SEGMENT_LENGTH = "latex.max-emulated-circle-segment-length";
	public static final String KEY_MAX_EM_LINE_SLOPE = "latex.max-emulated-line-slope";
	public static final String KEY_FILE_WRAPPER_PROLOG = "latex.file-wrapper-prolog";
	public static final String KEY_FILE_WRAPPER_EPILOG = "latex.file-wrapper-epilog";

	////////////////////////////////
	//// DEFAULT VALUES
	////////////////////////////////	
    public static final double DEFAULT_EM_LINE_LENGTH = 0.12;
    public static final double DEFAULT_MAX_CIRCLE_DIAMETER = 14 ; //mm
    public static final double DEFAULT_MAX_DISK_DIAMETER = 5.4; //mm
    public static final double DEFAULT_MAX_EM_CIRCLE_SEGMENT_LENGTH = 1;
    public static final double DEFAULT_MAX_EM_LINE_SLOPE = 1000.0 ;
	public static final String DEFAULT_FILE_WRAPPER_PROLOG = 
		"\\documentclass{article} "+CR_LF+ "\\thispagestyle{empty}" + CR_LF+"\\begin{document} "+CR_LF;
	public static final String DEFAULT_FILE_WRAPPER_EPILOG = "\\end{document}";

	public static final String RESCALING_TEX_FUNCTION = "\\JPicScale";

	
}
