/*
 PicParallelogramFormatter.java - August 29, 2003 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.format.output.latex;

import jpicedt.graphic.*;
import jpicedt.graphic.model.*;
import jpicedt.graphic.io.formatter.*;
import jpicedt.graphic.PicPoint;

import java.awt.*;

/**
 * Formats a PicParallelogram to the LaTeX-picture format.
 * @author Sylvain Reynal
 * @since jpicedt 1.4
 * @version $Id: PicParallelogramFormatter.java,v 1.3.2.1 2007/09/02 11:55:13 reynal Exp $
 * <p>
 *
 */
public class PicParallelogramFormatter implements Formatter, PicObjectConstants,LatexConstants {

	/** the Element this formater acts upon */
	protected PicParallelogram element;
	/** the producing factory */
	protected LatexFormatter factory;

	public PicParallelogramFormatter(PicParallelogram element, LatexFormatter factory){
		this.element = element;
		this.factory=factory;
	}

	/**
	  * Depending on the parallelogram being rectangle or not, either use LaTeX's \\line command,
	  * or the \\rule command if the shape is to be filled.
	  * @return The LaTeX string corresponding to this object
	  */
	public String format(){

		StringBuffer buf = new StringBuffer(100);

		PicPoint ptTR = element.getPoint(PicParallelogram.P_TR,null);
		PicPoint ptBL = element.getPoint(PicParallelogram.P_BL,null);
		PicPoint ptTL = element.getPoint(PicParallelogram.P_TL,null);
		PicPoint ptBR = element.getPoint(PicParallelogram.P_BR,null);

		factory.appendThicknessString(buf,element);

		// LaTeX commands
		// we use a LaTeX \\rule if it's a filled rectangle
		if(element.isRectangle() && element.getAttribute(FILL_STYLE)==SOLID){

			buf.append("\\put");
			buf.append(element.getPoint(PicParallelogram.P_BL,null));
			buf.append("{\\rule{");
			buf.append(PEToolKit.doubleToString(ptBL.distance(ptBR))); // horizontal length
			buf.append("\\unitlength}{");
			buf.append(PEToolKit.doubleToString(ptBL.distance(ptTL))); // vertical length
			buf.append("\\unitlength}}");
		}
		// else we handle the non-zero dash case as well :
		else {
			double dash = element.getAttribute(LINE_STYLE)==DASHED ? ((Double)element.getAttribute(DASH_OPAQUE)).doubleValue() : 0; // [pending] support black/white dash
			buf.append(factory.lineToLatexString(ptBL.x, ptBL.y, ptBR.x, ptBR.y, Arrow.NONE,Arrow.NONE, dash));
			buf.append(CR_LF);
			buf.append(factory.lineToLatexString(ptBL.x, ptBL.y, ptTL.x, ptTL.y, Arrow.NONE,Arrow.NONE, dash));
			buf.append(CR_LF);
			buf.append(factory.lineToLatexString(ptBR.x, ptBR.y, ptTR.x, ptTR.y, Arrow.NONE,Arrow.NONE, dash));
			buf.append(CR_LF);
			buf.append(factory.lineToLatexString(ptTL.x, ptTL.y, ptTR.x, ptTR.y, Arrow.NONE,Arrow.NONE, dash));
		}
		buf.append(CR_LF);
		return buf.toString();
	}

} 
