/*
 PicEllipseFormatter.java - August 30, 2003 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.format.output.pstricks;

import jpicedt.graphic.io.formatter.*;
import jpicedt.graphic.*;
import jpicedt.graphic.model.*;

import java.awt.*;
import java.awt.geom.*;

/**
 * Formats a PicEllipse using PsTricks macros, including pstricks-add's \\psellipticarc if applicable.
 * [SR:pending] arrows for flat ellipse
 * @author Vincent Guirardel, Sylvain Reynal
 * @since jpicedt 1.4
 * @version $Id: PicEllipseFormatter.java,v 1.11.2.1 2007/09/02 11:55:13 reynal Exp $
 * 
 */
public class PicEllipseFormatter implements Formatter, PicObjectConstants, PstricksConstants {

	private PicEllipse ellipse;
	private PstricksFormatter factory;

	public PicEllipseFormatter(PicEllipse ellipse,PstricksFormatter factory){
		this.ellipse = ellipse;
		this.factory=factory;
	}

	/**
	 * \\rput{rotation}(centerX,centerY){\\psellipse[param](0,0)(greatAxis/2,smallAxis/2)} => plain ellipse<br>
	 * \\rput{rotationAngle}(centerX,centerY){\qline(min,0)(max,0)} => flat ellipse
	 */
	public String format(){

		StringBuffer buf = new StringBuffer(100); // 100 as initial capacity seems to be a good guess
		PicAttributeSet set = ellipse.getAttributeSet();
		
		// first handle possibly user-defined colours
		PstricksFormatter.ParameterString paramStr = factory.createParameterString(ellipse);
		if (paramStr.isDefinedColourString()) buf.append(paramStr.getUserDefinedColourBuffer());

		// -------------- plain ellipse -----------------
		if (ellipse.isPlain() && ellipse.getArcType()!=Arc2D.CHORD){
			// [SR:pending] why do we exclude CHORD here ???
			//  \\rput{rotationAngle}(centerX,centerY){ \\psellipse[param](0,0)(greatAxis/2,smallAxis/2)

			buf.append("\\rput{");
			buf.append(PEToolKit.doubleToString(Math.toDegrees(ellipse.getRotationAngle())));
			buf.append("}");

			// center
			buf.append(ellipse.getPoint(PicEllipse.P_CENTER,null));

			buf.append("{");

			buf.append("\\psellipse");
			// parameters
			buf.append("[");
			buf.append(paramStr.getParameterBuffer());
			buf.append("]");
			buf.append("(0,0)");
			// greatAxis
			buf.append("(");
			buf.append(PEToolKit.doubleToString(ellipse.getGreatAxisLength()/2));
			buf.append(",");
			// smallAxis
			buf.append(PEToolKit.doubleToString(ellipse.getSmallAxisLength()/2));
			buf.append(")");

			buf.append("}");
			buf.append(CR_LF);
		} // plain ellipse
		
		// ------------------ flat ellipse -----------------------
		else if (ellipse.isFlat()){
			// draw a line:  \\rput{rotationAngle}(centerX,centerY){\qline(min,0)(max,0)}
			double min,max;
			double start=ellipse.getRotatedAngleStart();//degrees
			double end=ellipse.getRotatedAngleEnd();//degrees
			min=Math.min(Math.cos(Math.toRadians(start)),Math.cos(Math.toRadians(end)));
			max=Math.max(Math.cos(Math.toRadians(start)),Math.cos(Math.toRadians(end)));
			if ( ( start<0 && end >0) || (end >360 )) max=1;
			if ( end >180) min=-1;
			if (ellipse.getArcType()==Arc2D.PIE){//take center into account in this case
				if (min>0) min=0;
				if (max<0) max=0;
			}
			min *= ellipse.getGreatAxisLength();
			max *= ellipse.getGreatAxisLength();

			buf.append("\\rput{");
			buf.append(PEToolKit.doubleToString(Math.toDegrees(ellipse.getRotationAngle())));
			buf.append("}");

			// center
			buf.append(ellipse.getPoint(PicEllipse.P_CENTER,null));

			//the line
			buf.append("{");
			buf.append("\\qline(");
			buf.append(PEToolKit.doubleToString(min));
			buf.append(",0)(");
			buf.append(PEToolKit.doubleToString(max));
			buf.append(",0)");
			buf.append("}");
			//[pending]: arrows !
		} // flat ellpise

		// ------------------- arc (flat case excluded) --------------------
		else {//case of an arc (chord/open/pie). Flat case excluded.
			// OPEN:  \\rput{rotationAngle}(centerX,centerY){\psellipticarc{<->}[param](0,0)(greatAxis,smallAxis){angle1}{angle2}}
			// PIE:  \\rput{rotationAngle}(centerX,centerY){\pscustom[param]{\psellipticarc(0,0)(greatAxis,smallAxis){angle1}{angle2}\lineto(0,0)\closepath}}
			// CHORD:  \\rput{rotationAngle}(centerX,centerY){\pscustom[param]{\psellipticarc(0,0)(greatAxis,smallAxis){angle1}{angle2}\closepath}}
			// depending on the value of usePsEllipticarcCommand, replace the \psellipticarc command by a parametricplot

			buf.append("\\rput{");
			buf.append(PEToolKit.doubleToString(Math.toDegrees(ellipse.getRotationAngle())));
			buf.append("}");

			// center
			buf.append(ellipse.getPoint(PicEllipse.P_CENTER,null));

			buf.append("{");



			// --------------------- use pstricks-add.sty ------------------------
			// Docs: see http://www.perce.de/LaTeX/pstricks-add/
			
			//if (PstricksFormatter.usePsEllipticArcCommand) { // [syd] now useless starting from PSTricks 1.11
				// CHORD => use pscustom so as to handle filling, since chord=arc+close_path !
				if (ellipse.getArcType()==Arc2D.CHORD) { 
					buf.append("\\pscustom");
					//[parameters]
					buf.append("[");
					buf.append(paramStr.getParameterBuffer());
					buf.append("]");
					buf.append("{\\psellipticarc");
				}
				// else for open arcs and pies, [parameters] will be included as part of the corresponding command, i.e.
				// * OPEN:
				// \\psellipticarc[<param>]{<arrows>}(<center>)(a,b){start-angle}{end-angle}
				// * PIE:
				// \\psellipticwedge[<param>](<center>)(a,b){start-angle}{end-angle}
				else {
					if (ellipse.getArcType()==Arc2D.PIE) buf.append("\\psellipticwedge");
					else buf.append("\\psellipticarc"); // Arc2D.OPEN

					//[parameters]+arrows
					buf.append("[");
					buf.append(paramStr.getParameterBuffer());
					buf.append("]");
					if (ellipse.getArcType()==Arc2D.OPEN) buf.append(PstricksUtilities.createPstricksStringFromArrows(ellipse));
				}

				buf.append("(0,0)");
				buf.append("(");
				buf.append(PEToolKit.doubleToString(0.5*ellipse.getGreatAxisLength()));
				buf.append(",");
				buf.append(PEToolKit.doubleToString(0.5*ellipse.getSmallAxisLength()));
				buf.append(")");
				buf.append("{");
				buf.append(PEToolKit.doubleToString(ellipse.getRotatedAngleStart()));
				buf.append("}");
				buf.append("{");
				buf.append(PEToolKit.doubleToString(ellipse.getRotatedAngleEnd()));
				buf.append("}");
				if (ellipse.getArcType()==Arc2D.CHORD) buf.append("\\closepath}");
				//else if (ellipse.getArcType()==Arc2D.PIE) buf.append("\\lineto(0,0)\\closepath}");
				// PIE now handled by psellipticwedge
			//}// end of psellipticarc command
			
			
			
			// --------------------- use pst-plot.sty ------------------------
			/*
			else { //parametricplot command
				if (ellipse.getArcType()!=Arc2D.OPEN) { // CHORD|PIE => prepend a pscustom command so as to handle filling, 
					// since chord=arc+close_path and pie=arc+line_to_center+close_path
					buf.append("\\pscustom");
					//[parameters]
					buf.append("[");
					buf.append(paramStr.getParameterBuffer());
					buf.append("]");
					buf.append("{\\parametricplot");
				}
				else { // OPEN => add arrows !
					// else for open arcs, [parameters] will be included as part of the corresponding command
					buf.append("\\parametricplot");

					//[parameters]+arrows : arrows need to be included as parameters
					buf.append("[");
					buf.append(paramStr.getParameterBuffer());
					if (paramStr.getParameterBuffer().length()>0) buf.append(',');
					buf.append("arrows=");
					Arrow leftArrow = (Arrow)ellipse.getAttribute(LEFT_ARROW);
					Arrow rightArrow = (Arrow)ellipse.getAttribute(RIGHT_ARROW);
					buf.append(PstricksUtilities.toPstricksString(leftArrow,false)); // e.g. "<" for ARROW_HEAD
					buf.append("-");
					buf.append(PstricksUtilities.toPstricksString(rightArrow,true)); // e.g. ">" for ARROW_HEAD
					buf.append("]");
				}

				// build aka "t cos a mul t sin b mul" i.e. x=a cost and y=b sin t, where t runs from t_start
				// to t_end:
				buf.append("{");
				buf.append(PEToolKit.doubleToString(ellipse.getRotatedAngleStart())); // i.e. t_start
				buf.append("}");
				buf.append("{");
				buf.append(PEToolKit.doubleToString(ellipse.getRotatedAngleEnd())); // i.e. t_end
				buf.append("}");
				buf.append("{ t cos ");
				buf.append(PEToolKit.doubleToString(0.5*ellipse.getGreatAxisLength()));
				buf.append(" mul t sin ");
				buf.append(PEToolKit.doubleToString(0.5*ellipse.getSmallAxisLength()));
				buf.append(" mul }");

				if (ellipse.getArcType()==Arc2D.CHORD) buf.append("\\closepath}");
				else if (ellipse.getArcType()==Arc2D.PIE) buf.append("\\lineto(0,0)\\closepath}");
			} // end of parametricplot command
			*/



			buf.append("}");

			buf.append(CR_LF);
		}

		return buf.toString();

	}


} 
