/*
 PstricksCustomizer.java - March 4, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Departement de Physique
 Ecole Nationale Superieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

 package jpicedt.format.output.pstricks;

import jpicedt.Localizer;
import jpicedt.graphic.toolkit.AbstractCustomizer;
import jpicedt.graphic.PEToolKit;
import jpicedt.widgets.DecimalNumberField;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;

/**
 * a panel for configuring the PstricksFormatter.
 * @author Sylvain Reynal
 * @since jPicEdt 1.3
 * @version $Id: PstricksCustomizer.java,v 1.9.2.1 2007/09/02 11:55:14 reynal Exp $ 
 */
public class PstricksCustomizer extends AbstractCustomizer implements PstricksConstants {

	private DecimalNumberField thinLinesTF, thickLinesTF;
	private JTextArea prologTA, epilogTA;
	//private JRadioButton ellipticArcRadio,parametricPlotRadio;
	//private ButtonGroup ellipticArcGroup;
	private Properties preferences;

	/**
	 * construct a new panel for pstricks preferences editing.
	 * @param preferences Properties used to init the widgets fields, and to store the values
	 * when "storePreferences" is called.
	 */
	public PstricksCustomizer(Properties preferences){

		this.preferences = preferences;
		
		// init main box
		Box box = new Box(BoxLayout.Y_AXIS);
		// temp. buffers
		JPanel p;
		JLabel l;

		/* Sub-Panel 1 :generic parameters for Pstricks */
		// nada

		/* Sub-Panel 2 : prolog and epilog */

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		c.gridwidth = GridBagConstraints.REMAINDER; 
		
		p = new JPanel(gbl);
		p.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
		            Localizer.currentLocalizer().get("format.PrologEpilog")));

		l = new JLabel(" "+Localizer.currentLocalizer().get("format.Prolog"));
		l.setToolTipText(Localizer.currentLocalizer().get("format.Prolog.tooltip"));
		gbl.setConstraints(l,c);
		p.add(l);
		prologTA = new JTextArea(5,50);
		gbl.setConstraints(prologTA,c);
		p.add(prologTA);

		l = new JLabel(" "+Localizer.currentLocalizer().get("format.Epilog"));
		l.setToolTipText(Localizer.currentLocalizer().get("format.Epilog.tooltip"));
		gbl.setConstraints(l,c);
		p.add(l);
		epilogTA = new JTextArea(5,50);
		gbl.setConstraints(epilogTA,c);
		p.add(epilogTA);

		box.add(p);

		/* Sub-Panel 3 : other things: ellipticArc. */

		/* [syd: now useless as recent releases of PSTricks (>=1.11) officially include psellipticarc and psellipticarcn]
		//GridBagLayout gbl = new GridBagLayout();
		//GridBagConstraints c = new GridBagConstraints();
		//c.gridwidth = GridBagConstraints.REMAINDER; 
		c.anchor=GridBagConstraints.WEST;
		p = new JPanel(gbl);
		p.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
		            Localizer.currentLocalizer().get("format.pstricks.PstEllipticArc")));

		ellipticArcRadio = new JRadioButton(" "+Localizer.currentLocalizer().get("format.pstricks.UsePstEllipticArc"));
		ellipticArcRadio.setToolTipText(Localizer.currentLocalizer().get("format.pstricks.UsePstEllipticArc.tooltip"));
		ActionHandler ah = new ActionHandler();
		ellipticArcRadio.addActionListener(ah);
		parametricPlotRadio = new JRadioButton(" "+Localizer.currentLocalizer().get("format.pstricks.UseParametricPlot"));
		parametricPlotRadio.setToolTipText(Localizer.currentLocalizer().get("format.pstricks.UseParametricPlot.tooltip"));
		parametricPlotRadio.addActionListener(ah);
		
		ButtonGroup ellipticArcGroup = new ButtonGroup();
		ellipticArcGroup.add(ellipticArcRadio);
		ellipticArcGroup.add(parametricPlotRadio);
		
		gbl.setConstraints(ellipticArcRadio,c);
		p.add(ellipticArcRadio);
		
		gbl.setConstraints(parametricPlotRadio,c);
		p.add(parametricPlotRadio);

		box.add(p);
		*/
		
		// completed :
		add(box, BorderLayout.NORTH);
	}

	/**
	 * @return the panel title, used e.g. for Border or Tabpane title. 
	 */
	public String getTitle(){ 
		return "PSTricks";	
	}


	/**
	 * @return the Icon associated with this panel, used e.g. for TabbedPane decoration 
	 */
	public Icon getIcon(){ 
		return null;	
	}


	/**
	 * @return the tooltip string associated with this panel 
	 */
	public String getTooltip(){ 
		return Localizer.currentLocalizer().get("format.pstricks.tooltip");	
	}

	/**
	 * Load widgets display content with a default value retrieved from the LatexConstants interface.
	 */
	public void loadDefault() {
		epilogTA.setText(DEFAULT_FILE_WRAPPER_EPILOG);
		prologTA.setText(DEFAULT_FILE_WRAPPER_PROLOG);
		
		/*
		if ((Boolean.valueOf(DEFAULT_USE_PS_ELLIPTIC_ARC_COMMAND)).booleanValue()){
		    ellipticArcRadio.setSelected(true);
			prologTA.setText(DEFAULT_FILE_WRAPPER_PROLOG_ELLIPTICSTY);
		}
		else {
		    parametricPlotRadio.setSelected(true);
			prologTA.setText(DEFAULT_FILE_WRAPPER_PROLOG_PSTPLOTSTY);
		}*/
		

	}

	/**
	 * Load widgets value from the Properties object given in the constructor.
	 */
	public void load() {

		epilogTA.setText(preferences.getProperty(KEY_FILE_WRAPPER_EPILOG,DEFAULT_FILE_WRAPPER_EPILOG));
		prologTA.setText(preferences.getProperty(KEY_FILE_WRAPPER_PROLOG,DEFAULT_FILE_WRAPPER_PROLOG));
		
		/*
		if (Boolean.valueOf(preferences.getProperty(KEY_USE_PS_ELLIPTIC_ARC_COMMAND)).booleanValue()){
		    ellipticArcRadio.setSelected(true);
			prologTA.setText(preferences.getProperty(KEY_FILE_WRAPPER_PROLOG,DEFAULT_FILE_WRAPPER_PROLOG_ELLIPTICSTY));
		}
		else {
		    parametricPlotRadio.setSelected(true);
			prologTA.setText(preferences.getProperty(KEY_FILE_WRAPPER_PROLOG,DEFAULT_FILE_WRAPPER_PROLOG_PSTPLOTSTY));
		}*/

	}	

	/**
	 * Store current widgets value to the Properties object given in the constructor,
	 * then update PstricksFormatter accordingly.
	 */
	 public void store(){
		preferences.setProperty(KEY_FILE_WRAPPER_PROLOG,prologTA.getText());
		preferences.setProperty(KEY_FILE_WRAPPER_EPILOG,epilogTA.getText());
		//preferences.setProperty(KEY_USE_PS_ELLIPTIC_ARC_COMMAND,(new Boolean(ellipticArcRadio.isSelected())).toString());
		PstricksFormatter.configure(preferences); // update
	}
	
	/** actionListener for radio-buttons related to parametric/elliptic choice ; allows to check
	 * proper inclusion of needed package in \\usepackage{} line */
	 /*
	class ActionHandler implements ActionListener {
		public void actionPerformed(ActionEvent e){
			if (e.getSource()==ellipticArcRadio || e.getSource()==parametricPlotRadio){
				if (ellipticArcRadio.isSelected()) prologTA.setText(DEFAULT_FILE_WRAPPER_PROLOG_ELLIPTICSTY);
				else prologTA.setText(DEFAULT_FILE_WRAPPER_PROLOG_PSTPLOTSTY);
			}
		}
	}*/
	

} 
