/*
 PstricksFormatter.java - 2001 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Departement de Physique
 Ecole Nationale Superieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package jpicedt.format.output.pstricks;

import jpicedt.format.output.util.PicGroupFormatter;

import jpicedt.graphic.PEToolKit;
import jpicedt.graphic.model.AbstractCurve;
import jpicedt.graphic.model.Drawing;
import jpicedt.graphic.model.Element;
import jpicedt.graphic.model.Arrow;
import jpicedt.graphic.model.PicAttributeSet;
import jpicedt.graphic.model.PicEllipse;
import jpicedt.graphic.model.PicGroup;
import jpicedt.graphic.model.PicObjectConstants;
import jpicedt.graphic.model.PicParallelogram;
import jpicedt.graphic.model.PicText;
import jpicedt.graphic.model.StyleConstants;

import jpicedt.graphic.io.formatter.Formatter;
import jpicedt.graphic.io.formatter.FormatterFactory;

import java.awt.Color;
import java.awt.geom.Rectangle2D;

import java.util.Properties;
import java.util.Iterator;

// debug (see main())
import jpicedt.*;
import jpicedt.graphic.*;
import jpicedt.graphic.grid.*;
import jpicedt.graphic.io.parser.*;
import java.io.*;

/**
 * Produces formatters for the PsTricks macro package.
 * @author Sylvain Reynal
 * @since jpicedt 1.3.2
 * @version $Id: PstricksFormatter.java,v 1.21.2.1 2007/09/02 11:55:14 reynal Exp $
 *
 */
public class PstricksFormatter implements FormatterFactory, PicObjectConstants, PstricksConstants {

	//protected static boolean usePsEllipticArcCommand = Boolean.valueOf(DEFAULT_USE_PS_ELLIPTIC_ARC_COMMAND).booleanValue();
	protected static String fileWrapperEpilog = DEFAULT_FILE_WRAPPER_EPILOG;
	protected static String fileWrapperProlog = DEFAULT_FILE_WRAPPER_PROLOG;
	/*static {
		if (usePsEllipticArcCommand) fileWrapperProlog=DEFAULT_FILE_WRAPPER_PROLOG_ELLIPTICSTY;
		else fileWrapperProlog=DEFAULT_FILE_WRAPPER_PROLOG_PSTPLOTSTY;
	}*/

	private final PicAttributeSet defaultAttributes = new PicAttributeSet();

	/**
	 * @return a Formatter able to format the given Element in the PsTricks format
	 */
	public Formatter createFormatter(Element e){
		if (e instanceof AbstractCurve) return new AbstractCurveFormatter((AbstractCurve)e,this);
		if (e instanceof PicEllipse) return new PicEllipseFormatter((PicEllipse)e,this);
		if (e instanceof PicParallelogram) return new PicParallelogramFormatter((PicParallelogram)e,this);
		// [SR:en_cours] if (e instanceof PicCircleFrom3Points) return new PicEllipseFormatter(((PicCircleFrom3Points)e).toEllipse(),this);//[pending: to be improved: add comments for the 3 control points]
		if (e instanceof PicText) return new PicTextFormatter((PicText)e,this);
		if (e instanceof PicGroup) return new PicGroupFormatter((PicGroup)e,this);
		return new NonSupportedFormatter(e);
	}

	class NonSupportedFormatter implements Formatter {
		Element element;
		NonSupportedFormatter(Element e){
			this.element = e;
		}
		public String format(){
			return "% Sorry, " + element.getName() + " is not supported (yet) by this format";
		}
	}

	/**
	 * Configure static fields using the given Properties object
	 * @param properties used to read shared parameters
	 *        If null, default values are used.
	 */
	public static void configure(Properties preferences){
		//usePsEllipticArcCommand = Boolean.valueOf(preferences.getProperty(KEY_USE_PS_ELLIPTIC_ARC_COMMAND,DEFAULT_USE_PS_ELLIPTIC_ARC_COMMAND)).booleanValue();
		fileWrapperEpilog = preferences.getProperty(KEY_FILE_WRAPPER_EPILOG,DEFAULT_FILE_WRAPPER_EPILOG);
		fileWrapperProlog = preferences.getProperty(KEY_FILE_WRAPPER_PROLOG,DEFAULT_FILE_WRAPPER_PROLOG);
		/*
		if (usePsEllipticArcCommand) // adapt default value
			fileWrapperProlog = preferences.getProperty(KEY_FILE_WRAPPER_PROLOG,DEFAULT_FILE_WRAPPER_PROLOG_ELLIPTICSTY);
		else
			fileWrapperProlog=preferences.getProperty(KEY_FILE_WRAPPER_PROLOG,DEFAULT_FILE_WRAPPER_PROLOG_PSTPLOTSTY);
		*/
	}

	/**
	 * @return a Formatter able to format the given Drawing in the PsTricks format ;
	 *         this may reliy on calls to <code>createFormatter(Element e)</code> on the elements
	 *         of the drawing, plus creating auxiliary
	 * @param outputConstraint constraint used by the factory to create a specific Formatter on-the-fly
	 */
	public Formatter createFormatter(Drawing d, Object outputConstraint){
		return new DrawingFormatter(d,outputConstraint);
	}

	////////////////////////////////////////////////////////////////////
	//// Toolkit
	////////////////////////////////////////////////////////////////////

	// [SR:pending] move the following methods to PstricksUtilities
	
	/**
	 * Append "\\newrgbcolor{colourName}{cR cG cB}" to the given buffer
	 */
	private void appendUserDefinedColour(StringBuffer userDefinedColorBuffer, String colourName, float[] colourComponents){
		userDefinedColorBuffer.append("\\newrgbcolor{");
		userDefinedColorBuffer.append(colourName);
		userDefinedColorBuffer.append("}{");
		userDefinedColorBuffer.append(PEToolKit.doubleToString(colourComponents[0]));
		userDefinedColorBuffer.append(" ");
		userDefinedColorBuffer.append(PEToolKit.doubleToString(colourComponents[1]));
		userDefinedColorBuffer.append(" ");
		userDefinedColorBuffer.append(PEToolKit.doubleToString(colourComponents[2]));
		userDefinedColorBuffer.append("}");
		userDefinedColorBuffer.append(CR_LF);
	}

	/**
	 * Returns PsTricks's "standard" parameter string for the given Element:
	 * <ul>
	 * <li> linewidth=xxx, linecolor=xxxx, doubleline=true/false and rel.
	 * <li> if dash non-nul : linestyle=dashed, dash=xx yy
	 * <li> if object if filled : fillstyle=solid, fillcolor=xxxx + possibly shadow and hatch parameters.
	 * </ul>
	 * Note that leading and trailing brackets must be added by the caller ! (this allows the caller to add its own set of parameters)
	 * <p>
	 * Since jpicedt 1.3.3: border is supported (not bordercolor), custom parameters are included verbatim
	 * @return an object encapsulating the parameter string as well as one or more \\newrgbcolor commands whenever deemed necessary.
	 */
	public ParameterString createParameterString(Element obj){

		PicAttributeSet attributes = obj.getAttributeSet();

		StringBuffer paramBuf = new StringBuffer(100);
		StringBuffer userDefinedColorBuffer=null; // no user-defined colour by default

		// linewidth
		double lineWidth = StyleConstants.getLineWidth(attributes);
		if (lineWidth != StyleConstants.getLineWidth(defaultAttributes)){
			paramBuf.append("linewidth=");
			paramBuf.append(PEToolKit.doubleToString(lineWidth));
		}

		// linecolor
		Color lineColor = StyleConstants.getLineColor(attributes);
		if (!lineColor.equals(StyleConstants.getLineColor(defaultAttributes))){
			if (paramBuf.length()>0) paramBuf.append(',');
			paramBuf.append("linecolor=");
			String lineColorName = PstricksUtilities.getPsTricksColorName(lineColor);
			if (lineColorName != null) paramBuf.append(lineColorName); // ok, this is a predefined colour
			else { // define a new colour named "userLineColor" (it doesn't matter if it's not a unique name, provided the caller insert it just BEFORE the command that needs it)
				userDefinedColorBuffer = new StringBuffer(100);
				appendUserDefinedColour(userDefinedColorBuffer,"userLineColour",lineColor.getRGBColorComponents(null));
				paramBuf.append("userLineColour");
			}
		}

		// linestyle
		if (StyleConstants.getLineStyle(attributes)==DASHED){
			if (paramBuf.length()>0) paramBuf.append(',');
			paramBuf.append("linestyle=dashed,dash=");
			paramBuf.append(PEToolKit.doubleToString(StyleConstants.getDashOpaque(attributes)));
			paramBuf.append(" ");
			paramBuf.append(PEToolKit.doubleToString(StyleConstants.getDashTransparent(attributes)));
		}
		else if (StyleConstants.getLineStyle(attributes)==DOTTED){
			if (paramBuf.length()>0) paramBuf.append(',');
			paramBuf.append("linestyle=dotted");
			double dotSep = StyleConstants.getDotSep(attributes);
			if (dotSep!=StyleConstants.getDotSep(defaultAttributes)){
				paramBuf.append(",dotsep=");
				paramBuf.append(PEToolKit.doubleToString(dotSep));
			}
		}
		else if (StyleConstants.getLineStyle(attributes)==NONE){
			if (paramBuf.length()>0) paramBuf.append(',');
			paramBuf.append("linestyle=none");
		}
		// else paramBuf.append(",linestyle=solid"); default

		// overstrike (if border > 0)
		if (StyleConstants.isOverStrike(attributes)){
			if (paramBuf.length()>0) paramBuf.append(',');
			paramBuf.append("border=");
			paramBuf.append(PEToolKit.doubleToString(StyleConstants.getOverStrikeWidth(attributes)));
		}


		// fill style and fill colour
		String fillStyle = StyleConstants.getFillStyle(attributes);
		if (fillStyle==NONE){
		}
		else { // solid or hatches -> add fill colour (even if it's redundant for hatches that are not "starred"=
			Color fillColor = StyleConstants.getFillColor(attributes);
			if(!(fillColor.equals(StyleConstants.getFillColor(defaultAttributes)))){ // i.e. "black" so far
				String fillColorName = PstricksUtilities.getPsTricksColorName(fillColor);
				if (paramBuf.length()>0) paramBuf.append(',');
				paramBuf.append("fillcolor=");
				if (fillColorName != null) paramBuf.append(fillColorName); // ok, this is a predefined colour
				else { // define a new colour named "userFillColor" (it doesn't matter if it's not a unique name, provided the caller insert it just BEFORE the command that needs it)
					if (userDefinedColorBuffer==null) userDefinedColorBuffer = new StringBuffer(100); // there was no user-defined colour for linecolor
					appendUserDefinedColour(userDefinedColorBuffer,"userFillColour",fillColor.getRGBColorComponents(null));
					paramBuf.append("userFillColour");
				}
			}

			if (fillStyle==SOLID) {
				if (paramBuf.length()>0) paramBuf.append(',');
				paramBuf.append("fillstyle=solid");
			}
			else { // hatches
				if (paramBuf.length()>0) paramBuf.append(',');
				if (fillStyle==VLINES) paramBuf.append("fillstyle=vlines");
				else if (fillStyle==VLINES_FILLED) paramBuf.append("fillstyle=vlines*");
				else if (fillStyle==HLINES) paramBuf.append("fillstyle=hlines");
				else if (fillStyle==HLINES_FILLED) paramBuf.append("fillstyle=hlines*");
				else if (fillStyle==CROSSHATCH) paramBuf.append("fillstyle=crosshatch");
				else if (fillStyle==CROSSHATCH_FILLED) paramBuf.append("fillstyle=crosshatch*");
				else throw new RuntimeException("not supported: fillstyle="+fillStyle);

				double x = StyleConstants.getHatchWidth(attributes);
				if (x!=StyleConstants.getHatchWidth(defaultAttributes)){
					paramBuf.append(",hatchwidth=");
					paramBuf.append(PEToolKit.doubleToString(x));
				}
				x = StyleConstants.getHatchSep(attributes);
				if (x!=StyleConstants.getHatchSep(defaultAttributes)){
					paramBuf.append(",hatchsep=");
					paramBuf.append(PEToolKit.doubleToString(x));
				}
				x = StyleConstants.getHatchAngle(attributes);
				if (x!=StyleConstants.getHatchAngle(defaultAttributes)){
					paramBuf.append(",hatchangle=");
					paramBuf.append(PEToolKit.doubleToString(x));
				}

				Color hatchColor = StyleConstants.getHatchColor(attributes);
				if (!(hatchColor.equals(StyleConstants.getHatchColor(defaultAttributes)))){
					paramBuf.append(",hatchcolor=");
					String hatchColorName = PstricksUtilities.getPsTricksColorName(hatchColor);
					if (hatchColorName != null) paramBuf.append(hatchColorName); // ok, this is a predefined colour
					else { // define a new colour named "userHatchColor" (it doesn't matter if it's not a unique name, provided the caller insert it just BEFORE the command that needs it)
						if (userDefinedColorBuffer==null) userDefinedColorBuffer = new StringBuffer(100); // there was no user-defined colour for linecolor or fillcolor
						appendUserDefinedColour(userDefinedColorBuffer,"userHatchColour",hatchColor.getRGBColorComponents(null));
						paramBuf.append("userHatchColour");
					}
				}
			}
		}
		// shadow
		if (StyleConstants.isShadow(attributes)){
			if (paramBuf.length()>0) paramBuf.append(',');
			paramBuf.append("shadow=true");

			double x = StyleConstants.getShadowSize(attributes);
			if (x!=StyleConstants.getShadowSize(defaultAttributes)){
				paramBuf.append(",shadowsize=");
				paramBuf.append(PEToolKit.doubleToString(x));
			}
			x = StyleConstants.getShadowAngle(attributes);
			if (x!=StyleConstants.getShadowAngle(defaultAttributes)){
				paramBuf.append(",shadowangle=");
				paramBuf.append(PEToolKit.doubleToString(x));
			}
			Color shadowColor = StyleConstants.getShadowColor(attributes);
			if (!(shadowColor.equals(StyleConstants.getShadowColor(defaultAttributes)))){
				paramBuf.append(",shadowcolor=");
				String shadowColorName = PstricksUtilities.getPsTricksColorName(shadowColor);
				if (shadowColorName != null) paramBuf.append(shadowColorName); // ok, this is a predefined colour
				else { // define a new colour named "userShadowColor"
					if (userDefinedColorBuffer==null) userDefinedColorBuffer = new StringBuffer(100); // there was no user-defined colour for linecolor or fillcolor
					appendUserDefinedColour(userDefinedColorBuffer,"userShadowColour",shadowColor.getRGBColorComponents(null));
					paramBuf.append("userShadowColour");
				}
			}
		}

		// arrow parameters if at least one arrow is present
		if (StyleConstants.getLeftArrow(attributes)!=Arrow.NONE || StyleConstants.getRightArrow(attributes)!=Arrow.NONE){
			double x,y;
			x = StyleConstants.getArrowGlobalScaleLength(attributes);
			y = StyleConstants.getArrowGlobalScaleWidth(attributes);
			if (x!=StyleConstants.getArrowGlobalScaleLength(defaultAttributes) || y!=StyleConstants.getArrowGlobalScaleWidth(defaultAttributes)){
				if (paramBuf.length()>0) paramBuf.append(',');
				paramBuf.append("arrowscale=");
				paramBuf.append(PEToolKit.doubleToString(y));
				paramBuf.append(' ');
				paramBuf.append(PEToolKit.doubleToString(x));
			}
			x = StyleConstants.getArrowWidthMinimumMm(attributes);
			y = StyleConstants.getArrowWidthLineWidthScale(attributes);
			if (x!=StyleConstants.getArrowWidthMinimumMm(defaultAttributes) || y!=StyleConstants.getArrowWidthLineWidthScale(defaultAttributes)){
				if (paramBuf.length()>0) paramBuf.append(',');
				paramBuf.append("arrowsize=");
				paramBuf.append(PEToolKit.doubleToString(x));
				paramBuf.append(' ');
				paramBuf.append(PEToolKit.doubleToString(y));
			}
			x = StyleConstants.getArrowLengthScale(attributes);
			if (x!=StyleConstants.getArrowLengthScale(defaultAttributes)){
				if (paramBuf.length()>0) paramBuf.append(',');
				paramBuf.append("arrowlength=");
				paramBuf.append(PEToolKit.doubleToString(x));
			}
			x = StyleConstants.getArrowInsetScale(attributes);
			if (x!=StyleConstants.getArrowInsetScale(defaultAttributes)){
				if (paramBuf.length()>0) paramBuf.append(',');
				paramBuf.append("arrowinset=");
				paramBuf.append(PEToolKit.doubleToString(x));
			}
			x = StyleConstants.getTBarWidthMinimumMm(attributes);
			y = StyleConstants.getTBarWidthLineWidthScale(attributes);
			if (x!=StyleConstants.getTBarWidthMinimumMm(defaultAttributes) || y!=StyleConstants.getTBarWidthLineWidthScale(defaultAttributes)){
				if (paramBuf.length()>0) paramBuf.append(',');
				paramBuf.append("tbarsize=");
				paramBuf.append(PEToolKit.doubleToString(x));
				paramBuf.append(' ');
				paramBuf.append(PEToolKit.doubleToString(y));
			}
			x = StyleConstants.getBracketLengthScale(attributes);
			if (x!=StyleConstants.getBracketLengthScale(defaultAttributes)){
				if (paramBuf.length()>0) paramBuf.append(',');
				paramBuf.append("bracketlength=");
				paramBuf.append(PEToolKit.doubleToString(x));
			}
			x = StyleConstants.getRoundedBracketLengthScale(attributes);
			if (x!=StyleConstants.getRoundedBracketLengthScale(defaultAttributes)){
				if (paramBuf.length()>0) paramBuf.append(',');
				paramBuf.append("rbracketlength=");
				paramBuf.append(PEToolKit.doubleToString(x));
			}
			// [pending] check if not already set by polydots
			x = StyleConstants.getPolydotsSizeMinimumMm(attributes);
			y = StyleConstants.getPolydotsSizeLineWidthScale(attributes);
			if (x!=StyleConstants.getPolydotsSizeMinimumMm(defaultAttributes) || y!=StyleConstants.getPolydotsSizeLineWidthScale(defaultAttributes)){
				if (paramBuf.length()>0) paramBuf.append(',');
				paramBuf.append("dotsize=");
				paramBuf.append(PEToolKit.doubleToString(x));
				paramBuf.append(' ');
				paramBuf.append(PEToolKit.doubleToString(y));
			}
		}
		
		// double line
		if (StyleConstants.isDoubleLine(attributes)) {
			if (paramBuf.length()>0) paramBuf.append(',');
			paramBuf.append("doubleline=true");
			double x = StyleConstants.getDoubleLineSep(attributes);
			if (x!=StyleConstants.getDoubleLineSep(defaultAttributes)){
				paramBuf.append(",doublesep=");
				paramBuf.append(PEToolKit.doubleToString(x));
			}
			Color doubleColor = StyleConstants.getDoubleLineColor(attributes);
			if (!(doubleColor.equals(StyleConstants.getDoubleLineColor(defaultAttributes)))){
				paramBuf.append(",doublecolor=");
				String doubleColorName = PstricksUtilities.getPsTricksColorName(doubleColor);
				if (doubleColorName != null) paramBuf.append(doubleColorName); // ok, this is a predefined colour
				else { // define a new colour named "userDoubleColor"
					if (userDefinedColorBuffer==null) userDefinedColorBuffer = new StringBuffer(100);
					appendUserDefinedColour(userDefinedColorBuffer,"userDoubleColour",doubleColor.getRGBColorComponents(null));
					paramBuf.append("userDoubleColour");
				}
			}
		}

		// misc params
		if (StyleConstants.getDimen(attributes)==INNER) {
			if (paramBuf.length()>0) paramBuf.append(',');
			paramBuf.append("dimen=inner");
		}
		else if (StyleConstants.getDimen(attributes)==MIDDLE) {
			if (paramBuf.length()>0) paramBuf.append(',');
			paramBuf.append("dimen=middle");
		}
		// else default="outer"

		// custom pstricks parameters
		if (StyleConstants.getPstCustom(attributes).length()>0){
			if (paramBuf.length()>0) paramBuf.append(',');
			paramBuf.append(StyleConstants.getPstCustom(attributes));
		}

		// lasy work-around for syntax errors...
		/*
		if (paramBuf.charAt(paramBuf.length()-1)==','){ // if buffer has a trailing ',', which will confuse PsTricks...
			paramBuf.deleteCharAt(paramBuf.length()-1);
	}
		*/
		return (new ParameterString(paramBuf, userDefinedColorBuffer));
	}


	/**
	 * Returns a "\\psset{...}" buffer containing default parameters (useful if these differ from PsTricks's ones)
	 * This is very similar to <code>createParameterString()</code> except that a default attribute set is used,
	 * and dimensional parameters are taken into account, for which jPicEdt's default values slightly
	 * differ from PsTricks's default (mostly due to the use of mm instead of postscript points as the basic unit). 
	 */
	public StringBuffer createDefaultParameterString(){


		StringBuffer paramBuf = new StringBuffer(100);
		StringBuffer userDefinedColorBuffer=null; // no user-defined colour by default

		paramBuf.append("\\psset{");

		paramBuf.append("linewidth=");
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getLineWidth(defaultAttributes)));

		paramBuf.append(",dotsep=");
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getDotSep(defaultAttributes)));

		paramBuf.append(",hatchwidth=");
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getHatchWidth(defaultAttributes)));
		paramBuf.append(",hatchsep=");
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getHatchSep(defaultAttributes)));

		paramBuf.append(",shadowsize=");
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getShadowSize(defaultAttributes)));

		paramBuf.append(",dimen=middle");//same as java


		// [SR:pending] not used yet
		//paramBuf.append(",doublesep=");
		//paramBuf.append(PEToolKit.doubleToString(StyleConstants.getDoubleLineSep(defaultAttributes)));

		paramBuf.append("}");
		paramBuf.append(CR_LF);

		paramBuf.append("\\psset{");

		paramBuf.append("dotsize=");
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getPolydotsSizeMinimumMm(defaultAttributes)));
		paramBuf.append(' ');
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getPolydotsSizeLineWidthScale(defaultAttributes)));
		paramBuf.append(",dotscale=");
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getPolydotsHorizontalScale(defaultAttributes)));
		paramBuf.append(' ');
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getPolydotsVerticalScale(defaultAttributes)));
		// --- fill color = black (PsTricks doesn't seem to fill in black by default)
		paramBuf.append(",fillcolor=");
		paramBuf.append(PstricksUtilities.getPsTricksColorName(StyleConstants.getFillColor(defaultAttributes)));

		paramBuf.append("}");
		paramBuf.append(CR_LF);

		// arrow params
		paramBuf.append("\\psset{");

		paramBuf.append("arrowsize=");
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getArrowWidthMinimumMm(defaultAttributes)));
		paramBuf.append(' ');
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getArrowWidthLineWidthScale(defaultAttributes)));
		
		paramBuf.append(",arrowlength=");
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getArrowLengthScale(defaultAttributes)));

		paramBuf.append(",arrowinset=");
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getArrowInsetScale(defaultAttributes)));

		paramBuf.append(",tbarsize=");
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getTBarWidthMinimumMm(defaultAttributes)));
		paramBuf.append(' ');
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getTBarWidthLineWidthScale(defaultAttributes)));

		paramBuf.append(",bracketlength=");
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getBracketLengthScale(defaultAttributes)));

		paramBuf.append(",rbracketlength=");
		paramBuf.append(PEToolKit.doubleToString(StyleConstants.getRoundedBracketLengthScale(defaultAttributes)));
		
		// dotsize -> see above (polydots)
		// arrowscale : default to 1 ok
		
		paramBuf.append("}");
		paramBuf.append(CR_LF);

		return paramBuf;
	}

	/**
	 * return a Pstrick's ParameterString made up of a concatenation of the given parameter and color buffers
	 * Mainly a helper method for outer classes that may need instanciating a ParameterString. 
	 */
	ParameterString createParameterString(StringBuffer paramBuf, StringBuffer userDefinedColourBuf){
		return new ParameterString(paramBuf, userDefinedColourBuf);
	}
	
	/**
	 * an inner class that represents a PsTricks parameter string (useful especially to handle User Defined Colours)
	 *
	 * the basic problem is that, if a new colour has to be defined, this has to be done through e.g. a \\newrgbcolor command
	 * BEFORE the Element command appears in the file. Hence it's necessary to return an object encapsulating
	 * several strings, not only the parameter string that appears inside the Element formated string.
	 */    
	public class ParameterString {

		/* formated parameter string */
		StringBuffer paramBuf;

		/* a formated string for user-defined colour commands (aka \\newrgbcolour... each being separated by CR's)
		 * that may be used for "linecolor" or "fillcolor" for instance.
		 * if non-null, there's a user-defined colour (i.e. a colour that doesn't belong to PsTricks default colour dictionary)
		 * if null, there's no user-defined colour
		 * It's the responsability of the caller to insert this string at the appropriate location */
		StringBuffer userDefinedColourBuf;

		public ParameterString(StringBuffer paramBuf, StringBuffer userDefinedColourBuf){

			this.paramBuf = paramBuf;
			this.userDefinedColourBuf = userDefinedColourBuf;
		}

		/**
		 * @return true if there's at least one user-defined colour that need to be defined
		 */
		public boolean isDefinedColourString(){ 
			return (userDefinedColourBuf != null);
		}

		/**
		 * @return the formated parameter string (w/o leading and trailing brackets)
		 */
		public StringBuffer getParameterBuffer(){ 
			return paramBuf;
		}

		/**
		 * @return a string containing \\newrgbcolor... like commands, separated by CR's,
		 *         or an empty stringBuffer if there's no user-defined colours.
		 */
		public StringBuffer getUserDefinedColourBuffer(){
			if (isDefinedColourString()) return userDefinedColourBuf;
			else return new StringBuffer(); // empty !
		}
	} // inner class





	////////////////////////////////////////////////////////////////////////////////////
	//// DRAWING
	////////////////////////////////////////////////////////////////////////////////////

	/**
	 * "begin{pspicture}(x0,y0)(x1,y1)" (PsTricks) where 
	 * (x0,y0)=lower-left corner and 
	 * (x1,y1)=upper-right corner
	 */
	public class DrawingFormatter implements Formatter {

		/** the Drawing this formatter acts upon */
		private Drawing drawing;
		private Object outputConstraint;

		/**
		 * init this formatter for the given drawing
		 * @param outputConstraint if MAKE_STANDALONE_FILE, prepend/append prolog and epilog.
		 */
		public DrawingFormatter(Drawing drawing,Object outputConstraint){
			this.drawing = drawing;
			this.outputConstraint = outputConstraint;
		}

		/**
		 * @return  a String representing this Drawing in the PsTricks format
		 * !!! PsTricks's default unit = 1cm !!!
		 */
		public String format(){

			StringBuffer buf = new StringBuffer(200);
			buf.append("%PSTricks content-type (pstricks.sty package needed)"+CR_LF);
			buf.append("%Add \\usepackage{pstricks} in the preamble of your LaTeX file"+CR_LF);
			buf.append("%You can rescale the whole picture (to 80% for instance) by using the command \\def"+
				   PstricksConstants.RESCALING_TEX_FUNCTION+"{0.8}"+CR_LF);
			if (outputConstraint == FormatterFactory.MAKE_STANDALONE_FILE) buf.append(fileWrapperProlog);
			buf.append("\\ifx"+PstricksConstants.RESCALING_TEX_FUNCTION+"\\undefined\\def"+
				   PstricksConstants.RESCALING_TEX_FUNCTION+"{1}\\fi"+CR_LF);// allow the user to change scale using \jPicScale
			buf.append("\\psset{unit="+PstricksConstants.RESCALING_TEX_FUNCTION+" mm}"+CR_LF); // make 1mm the default unit and rescale

			// add default parameter values:
			buf.append(createDefaultParameterString());

			/* first we compute the coordinates of the "begin{pspicture}(x0,y0)(x1,y1)" command from the drawing's bounding box */

			//
			// first we compute the location of the upper-right corner of the picture,
			// then we convert it in LaTeX coordinates
			Rectangle2D box = drawing.getBoundingBox(); // recursively calls getBounds2D() on each Element [pending] annoying bug with PicText objects !
			//jpicedt.Log.debug("bbox = "+box);
			buf.append("\\begin{pspicture}(0,0)(");
			buf.append(PEToolKit.doubleToString(box.getMaxX()));
			buf.append(",");
			buf.append(PEToolKit.doubleToString(box.getMaxY()));
			buf.append(")");
			buf.append(CR_LF);

			/*  then for each Element in the Drawing, we call "toFormatedString" and append the returned String to our buffer */
			for (Iterator it=drawing.elements(); it.hasNext(); ){
				Element e = (Element)it.next();
				buf.append(createFormatter(e).format());
			}
			// previously registered not-parsed-commands:
			String s = drawing.getNotparsedCommands();
			if (s != null  && !s.equals("")){
				buf.append("%Begin not parsed");
				buf.append(CR_LF);
				buf.append(s);
				buf.append("%End not parsed");
				buf.append(CR_LF);
			}
			// epilogue
			buf.append("\\end{pspicture}");
			buf.append(CR_LF);
			if (outputConstraint == FormatterFactory.MAKE_STANDALONE_FILE) buf.append(fileWrapperEpilog);
			return buf.toString();
		}
	}

	public static void main(String[] args){
		try {
			Reader reader = new FileReader(args[0]);
			Parser parser = MiscUtilities.createParser();
			Drawing dr = parser.parse(reader); // takes some time...
			Rectangle2D bb = dr.getBoundingBox();
			System.out.println(bb);
			//System.out.println(dr);
			/*
			PstricksFormatter factory = new PstricksFormatter();
			Formatter df = factory.createFormatter(dr,null);
			String buffer = df.format();
			System.out.println(buffer);
			*/
		}
		catch (Exception e){
			//jpicedt.Log.error(e.toString());
			e.printStackTrace();
		}
	}

}
