/*
 ContentType.java - February 16, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic;

import jpicedt.graphic.toolkit.EditorKit;
import jpicedt.graphic.toolkit.AbstractCustomizer;
import jpicedt.graphic.io.formatter.FormatterFactory;
import jpicedt.graphic.view.ViewFactory;

import java.util.*;

/**
 * Specifies a content-type (aka mime-type) for a Drawing document created by an editor-kit. <p>
 * This interface is aimed at lumping together top-level classes needed by a given content-type
 * in the View-Model-Control framework : so far this involves obtaining an instance of EditorKit
 * appropriate for the given content-type, as well as a corresponding FormatterFactory.
 * A <code>configure</code> method is provided for conveniently configuring editor-kits and factories before use.
 * <p>
 * [SR:pending] better move to jpicedt.graphic.io ?
 * @since jpicedt 1.3.2
 * @author Sylvain Reynal
 * @version $Id: ContentType.java,v 1.7.2.1 2007/09/02 11:56:15 reynal Exp $
 *
 */
public interface ContentType {

	/**
	 * Return the presentation name of this content-type
	 */
	String getPresentationName();

	/**
	 * Creates a ViewFactory that's suited for this content-type
	 */
	ViewFactory createViewFactory();

	/**
	 * Creates a FormatterFactory that's suited for this content-type
	 */
	FormatterFactory createFormatter();
	
	/**
	 * Configure the EditorKit and the FormatterFactory returned by the factory methods,
	 * from the given Properties
	 */
	void configure(Properties p);
	
	/**
	 * Returns a customizer panel for this content-type
	 * @return null if no customizer is available for this content-type
	 * @param prop used to init the component or to store preferences on-demand.
	 */
	 AbstractCustomizer createCustomizer(Properties prop);

}
