/*
 SelectionHandler.java - February 15, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic;

import jpicedt.graphic.model.Element;
import jpicedt.graphic.model.Drawing;

import java.util.*;

/**
 * a SelectionHandler allows to manage selection-related behaviours for a given instance of 
 * {@link jpicedt.graphic.model.Drawing Drawing}.   
 * Concrete implementation may generally want to store references 
 * on selected {@link jpicedt.graphic.model.Element Element's} here. This may be easily carried out
 * by relying on Java's collection framework, hence we have specified some useful methods here for
 * this purpose.
 * @since jpicedt 1.3.2
 * @author Sylvain Reynal
 * @version $Id: SelectionHandler.java,v 1.7.2.1 2007/09/02 11:56:17 reynal Exp $
 */
public interface SelectionHandler {
	
	/**
	 * Return the number of selected objects
	 */
	int size();
	
	/**
	 * Return an iterator over the selected Element's
	 */
	Iterator elements();
	
	/**
	 * Return the selected elements wrapped in a Collection
	 */
	Collection asCollection();
	
	/**
	 * Return the selected elements wrapped in an array (may be a convenience call to asCollection)
	 */
	Element[] asArray();
	
	/**
	 * Return whether the given Element is selected or not
	 * @param includeAncestors if true, return true if this Element or one of its ancestors (e.g. a BranchElement) 
	 *        is selected. This may be implemented using reentrance.
	 */
	boolean isSelected(Element e, boolean includeAncestors);
	
	/**
	 * Select all Element's belonging to the given Drawing.
	 */
	void selectAll(Drawing d);
	
	/**
	 * Unselect all selected Element's 
	 */
	void unSelectAll();
	
	/**
	 * Add the given Element to the current selection
	 */
	void addToSelection(Element e);
	
	/**
	 * Replace the current selection with the given element. 
	 */
	void replaceSelection(Element e);

	/**
	 * Unselect the given Element, ie remove it from the current selection.
	 */
	void unSelect(Element e);
	
	/**
	 * Delete all selected Element's from the given Drawing
	 */
	void delete(Drawing d);
	
	/**
	 * Returns a list containing elements in the selection that are of the same type or
	 * inherit the given clazz.
	 */
	ArrayList createFilteredCollection(Class clazz);
	
	/**
	 * Returns whether this selection-handler contains objects that are of the same type of inherit from the
	 * given clazz.
	 * @since jpicedt 1.4pre5
	 */
	boolean containsClass(Class clazz);
	
	
}
