/*
 HelpMessageEvent.java - February 28, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
package jpicedt.graphic.event;

import java.util.EventListener;

import jpicedt.graphic.toolkit.EditorKit;

/**
 * A class that allows an EditorKit to post help-messages (or tooltips) that make sense with the current
 * operation being performed. This may be used e.g. by the UI machinery.
 * @author Sylvain Reynal
 * @version $Id: HelpMessageEvent.java,v 1.5.2.1 2007/09/02 11:56:18 reynal Exp $
 * @since jpicedt 1.3.2
 */
public class HelpMessageEvent  {

    private EditorKit source;
	private String message;

	/**
	 * contruct a new event
	 * @param source the editorkit that sourced the event
	 * @param message the help-message
	 */
    public HelpMessageEvent(EditorKit source, String message){
    
		this.source=source;
		this.message = message;
    }
	
	/**
 	 * Return the editor kit that sourced this event.
 	 */
    public EditorKit getSource(){
		return source;
	}

	/**
	 * Return the help-message.
	 */
    public String getMessage(){
		return message;
	}
	
	/**
	 * Return a String representation of this event for debugging purpose.
	 */
    public String toString(){

		return "HelpMessageEvent@" + Integer.toHexString(hashCode()) 
		    + ", msg=" + message
	    	+ ", source=" + source;
    }
}
