/*
 JPICGroupFormatter.java - August 5, 2003 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.io.formatter;

import jpicedt.graphic.model.*;
import jpicedt.graphic.*;

import java.util.*;

/**
 * A JPIC-XML formatter for PicGroup objects.<br>
 * Syntax : <g> ... </g>.
 * @since jpicedt 1.3.3
 * @author Sylvain Reynal
 * @version $Id: JPICGroupFormatter.java,v 1.4.2.1 2007/09/02 11:56:19 reynal Exp $ 
 *
 */
public class JPICGroupFormatter implements Formatter, JPICConstants {

	private PicGroup group;
	
	/** the factory that produced this formatter */
	private FormatterFactory factory;

	/**
	 * @param obj the PicGroup to be formatted
	 * @param factory the factory that produced this formatter
	 */
	public JPICGroupFormatter(PicGroup group, FormatterFactory factory){

		this.group = group;
		this.factory=factory;

	}

	/**
	 * @return a String representing this Element in the JPIC-XML format.
	 */
	public String format(){

		StringBuffer buf = new StringBuffer(100);
		buf.append("<g>");
		buf.append(CR_LF);

		for(Iterator it=group.children(); it.hasNext();){
			Element obj =  (Element)it.next();
			buf.append(factory.createFormatter(obj).format());
		}
		buf.append("</g>");
		buf.append(CR_LF);
		return buf.toString();
	}
} 
