/*
 JPICSmoothPolygonFormatter.java - August 26, 2003 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Departement de Physique
 Ecole Nationale Superieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.io.formatter;

import jpicedt.graphic.*;
import jpicedt.graphic.model.*;

import java.awt.*;

/**
 * JPIC-XML formatter for PicSmoothPolygon objects.
 * JPIC-XML syntax :
 * <smoothpolygon points="(x1,y1);(x2,y2);...;(xn,yn)" smoothness="c1;c2;...;cn" closed="false|true" other-attribs />
 *
 * @since jpicedt 1.4
 * @author Sylvain Reynal
 * @version $Id: JPICSmoothPolygonFormatter.java,v 1.7.2.1 2007/09/02 11:56:19 reynal Exp $ 
 *
 */
public class JPICSmoothPolygonFormatter implements Formatter, PicObjectConstants, JPICConstants {

	/** the Element this formatter acts upon */
	private PicSmoothPolygon poly;
	/** the factory that produced this formatter */
	private JPICFormatter factory;

	/**
	 * @param poly the PicSmoothPolygon to be formatted
	 * @param factory the factory that produced this formatter
	 */
	public JPICSmoothPolygonFormatter(PicSmoothPolygon poly, JPICFormatter factory){
		this.poly = poly;
		this.factory=factory;
	}

	/**
	 * @return a String representing this Element in the JPIC-XML format 
	 */
	public String format(){
		StringBuffer buf = new StringBuffer(100); // 100 as initial capacity seems to be a good guess
	
		buf.append("<smoothpolygon");
		
		// points=...
		PicPoint[] pts = new PicPoint[poly.getLastPointIndex()+1];
		for (int i = 0; i <= poly.getLastPointIndex(); i++) 
			pts[i]=poly.getPoint(i, null);

		XmlAttributeSet map = new XmlAttributeSet();
		map.putNameValuePair("points", pts);
		map.putNameValuePair("closed", poly.isClosed());
		
		// smoothness=...
		double[] smoothness = new double[poly.getLastPointIndex()+1];
		for (int i = 0; i <= poly.getLastPointIndex(); i++) 
			smoothness[i] = poly.getSmoothCoefficient(i);
		map.putNameValuePair("smoothness", smoothness);
		
		// other attributes...
		map.putCommonAttributes(poly);
		
		buf.append(map.toXML());
		buf.append(" />");
		buf.append(CR_LF);
		return buf.toString();
	}

} 
