/*
 ParserException.java - August 1, 2003 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.io.parser;

import org.xml.sax.SAXParseException;

/**
 * Placeholder for the Exception class ; should serve as the base-class for any Exception related
 * to parsing processes, and may be used as an Exception wrapper as well, e.g. for SAX generated Exception's.
 */
public class ParserException extends Exception {
	
	
	public ParserException(String message){
		super(message);
	}
	
	/**
	 * Convenient constructor for wrapping an Exception into a ParserException
	 */
	public ParserException(Exception ex){
		super(ex);
	}
	
	/**
	 * Return a localized version of the given string, by relying on the currently installed 
	 * jpicedt.Localizer.
	 */
	protected String localize(String str){
		return jpicedt.Localizer.currentLocalizer().get(str);
	}
	
	
	public String getMessage(){
		if (getCause() instanceof SAXParseException){
			SAXParseException saxEx = (SAXParseException)getCause();
			return saxEx.getMessage() + " line="+saxEx.getLineNumber() + " column="+saxEx.getColumnNumber();
		}
		else return super.getMessage();
	}
	
	
	/**
	 * May signal that the content being currently parsed does not correspond to
	 * any recognized/supported format
	 */
	public static class UnrecognizedFileFormat extends ParserException {
		
		public UnrecognizedFileFormat(){
			super("exception.parser.UnrecognizedFileFormat");
		}
		
		public String toString(){
			return localize(getMessage());
		}
	}
	
}
