/*
 DefaultPointIndexIterator.java - September 12, 2003 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.model;

/**
 * Default implementation of the PointIndexIterator interface, which simply iterates in ascending order over all control points.
 */
public class DefaultPointIndexIterator implements PointIndexIterator {
		
		protected int counter;
		protected Element elem;
		
		/**
		 * create a new PointIndexIterator for the given Element
		 */
		public DefaultPointIndexIterator(Element e){
			elem=e;
			counter = elem.getFirstPointIndex();
		}
		
		/**
		 * Returns true if the iteration has more elements
		 */
		public boolean hasNext(){
			if (counter > elem.getLastPointIndex()) return false;
			return true;
		}
		
		/**
		 * Returns the index of the next PicPoint in the iteration.
		 */
		public int next(){
			if (!hasNext()) throw new java.util.NoSuchElementException(Integer.toString(counter));
			counter++;
			return counter-1;
		}
		
		public void reset(){
			counter = elem.getFirstPointIndex();
		}
	}

