/*
 MouseTransformFactory.java - August 23, 2003 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Departement de Physique
 Ecole Nationale Superieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.toolkit;

import jpicedt.graphic.view.*;
import jpicedt.graphic.event.*;
import jpicedt.graphic.model.*;
import jpicedt.graphic.*;
import jpicedt.graphic.grid.*;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;

/**
 * A factory that produces MouseTransform's to be used by the SelectionTool mouse-tool. 
 * @author Sylvain Reynal
 * @since jPicEdt 1.4
 * @version $Id: MouseTransformFactory.java,v 1.6.2.1 2007/09/02 11:56:28 reynal Exp $
 */
public interface MouseTransformFactory {

	/**
	 * Return a MouseTransform whose type is adequate with the given mouse-event.
	 * This can be null if no MouseTransform matches the given event.
	 */
	MouseTransform createMouseTransform(PEMouseEvent e);

	/**
	 * Allows the MouseTransformFactory to do specific graphic rendering when it's installed in a
	 * hosting SelectionTool.
	 * @since jpicedt 1.4
	 */
	void paint(Graphics2D g, Rectangle2D allocation, double scale);

	/** 
	 * called when the associated SelectionTool is being activated in the hosting EditorKit.
	 * Initialization work required before any mouse-event occurs should be done here.
	 * Other initialization work may be carried out in the MouseTransform's themselves.
	 */
	void init();

	/** called when the associated SelectionTool is being deactivated in the hosting EditorKit.
	 * This provides a way for the factory to do some final clean-up, e.g. local buffers, graphic context,...
	 */
	void flush();
	
} // MouseTransformFactory

