/*
 PEMenu.java - January 1, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.toolkit;

import javax.swing.*;

/**
 * This is a subclass of JMenu that allows to add Swing Action's using 
 * the ACCELERATOR_KEY property (contrary to the JMenu implementation).<br>
 */
public class PEMenu extends JMenu {

	/** create a PEMenu from the given Action ; this is merely used to fetch the 
	 * menu mnemonic from the given Action's MNEMONIC_KEY value*/
	public PEMenu(Action a){
		super(a);
		setMnemonic(((Integer)a.getValue(Action.MNEMONIC_KEY)).intValue());
	}

	/** create a PEMenu from the given string, w/o mnemonic */
	public PEMenu(String s){
		super(s);
	}
	/**
	 * set menuitem accelerator to a.getValue(ACCELERATOR_KEY)
	 * @param a Action from which a JMenuItem gets created.
	 */
	public JMenuItem add(Action a) {
		//jpicedt.Log.debug("action="+a);
		if (a instanceof PEToggleAction){
			return add((PEToggleAction)a);
		}
		JMenuItem mi = super.add(a);
		KeyStroke ks =  (KeyStroke)a.getValue(Action.ACCELERATOR_KEY);
		if (ks != null) mi.setAccelerator(ks);
		//mi.setIcon(null);
		return mi;
	}
	
	/**
	 * set menuitem accelerator to a.getValue(ACCELERATOR_KEY)
	 * @param a Action from which a JMenuItem gets created.
	 */
	public JCheckBoxMenuItem add(PEToggleAction a) {
		//jpicedt.Log.debug("action="+a);

		JCheckBoxMenuItem cbmi = new JCheckBoxMenuItem(a);
		KeyStroke ks =  (KeyStroke)a.getValue(Action.ACCELERATOR_KEY);
		if (ks != null) cbmi.setAccelerator(ks);
		//cbmi.setIcon(null);
		add(cbmi);
		return cbmi;
	}
	
} // class
