/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.graphic.toolkit;

import java.awt.*;
import javax.swing.*;
import java.util.*;
import java.net.*;

/**
 * A subclass of JToolBar that knows how to add PEAction's.
 * the abstract superclass for jPicEdt toolbars ; some methods have been added to/overriden from JToolBar, such as :
 * - creating JToggleButton from an Action
 * - adding a Button from an Action, taking i18n res file into account
 *
 * @author Sylvain Reynal
 * @since PicEdt 1.0
*/
public class PEToolBar extends JToolBar {

	/** buttons margin */
	//protected static final Insets jButtonMargin = new Insets(2,2,2,2); // top,left,bottom,right

	/**
	 * simply calls super()
	 */
	public PEToolBar(){
		super();
	}
	
	/**
	 * Creates a JButton from the given Action properties
	 * @param a Action from which this button gets created.
	 */
	public JButton add(Action a){
		JButton b = super.add(a);
		b.setMargin(new Insets(1,1,1,1));
		//b.setRolloverEnabled(true);
		//b.setBorderPainted(false); // [pending]
		b.setAlignmentY(CENTER_ALIGNMENT);
		b.setAlignmentX(CENTER_ALIGNMENT);
		return b;
	}
	
	/**
	 * Creates a JToggleButton from the given PEToggleAction's properties
	 * @param a Action from which this button gets created.
	 */
	public JToggleButton add(PEToggleAction a){

		JToggleButton b = new JToggleButton(a);
		if (b.getIcon()!=null) b.setText(null);
		b.setMargin(new Insets(2,2,2,2));
		//b.setAlignmentY(CENTER_ALIGNMENT);
		//b.setAlignmentX(CENTER_ALIGNMENT);
		b.setRolloverEnabled(true);
        add(b);
        return b;
	}
} // class ToolBar
