
package jpicedt.graphic.toolkit;

import jpicedt.graphic.event.PEMouseEvent;


/**
 * A mouse tool for EditorKit, which allows to zoom in and out while retaining the current scene at the center
 * of the drawing sheet.
 * @author Sylvain Reynal
 * @since jPicEdt 1.3.2
 */
public class ZoomTool extends MouseTool {

	private static final double zoomIncrement = 2.0; // ie zoom = 2.0 * zoom when increased
	private EditorKit kit;
	private CursorFactory cursorFactory = new CursorFactory();
	
	/**
	 * construct a new ZoomTool for the given EditorKit
	 * @since jPicEdt 1.3.2
	 */
	public ZoomTool(EditorKit kit){
		this.kit = kit;
	}
	
	/**
	 * Depending on the button and the modifiers, we increase or decrease zoom, or do nothing.
	 */
	public void mousePressed(PEMouseEvent e) {
		super.mousePressed(e);

		double zoom = e.getCanvas().getZoomFactor(); // fetch current zoom value

		// right button click : back to SELECT_MODE
		if (e.isPopupTrigger()){
			kit.setCurrentMouseTool(EditorKit.SELECT);
			return;
		}

		// shift+left-click : zoom out
		else if (e.isShiftDown()) {
			zoom /= zoomIncrement;
			e.getCanvas().setZoomFactor(zoom, e.getPicPoint());
			return;
		}

		// left-click : zoom in
		else {
			zoom *= zoomIncrement;
			e.getCanvas().setZoomFactor(zoom, e.getPicPoint());
			return;
		}
	}
	
	/** set cursor */
	public void mouseMoved(PEMouseEvent e){
		super.mouseMoved(e);
		kit.postHelpMessage("help-message.Zoom");
		if (e.isShiftDown()) e.getCanvas().setCursor(cursorFactory.getPECursor(CursorFactory.ZOOM_OUT));
		else e.getCanvas().setCursor(cursorFactory.getPECursor(CursorFactory.ZOOM_IN));
	}
	
} // ZoomTool class

