/*
 TestCanvas.java - August 15, 2002 - jPicEdt 1.3.3, a picture editor for LaTeX.
 Copyright (C) 1999-2006 Sylvain Reynal

 Departement de Physique
 Ecole Nationale Superieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.test;

import jpicedt.*;
import jpicedt.graphic.*;
import jpicedt.graphic.event.*;
import jpicedt.graphic.io.formatter.*;
import jpicedt.graphic.toolkit.*;
import jpicedt.graphic.model.*;
import jpicedt.graphic.view.*;
import jpicedt.graphic.grid.*;
import jpicedt.ui.*;
import jpicedt.ui.dialog.*;
import jpicedt.ui.util.*;
import jpicedt.format.output.latex.*;

import javax.swing.*;
import java.io.*;
import java.util.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import bsh.*;
import bsh.util.*;

/**
 * Test class incorporating a BSH interpreter facility for efficient debugging.
 * @author Sylvain Reynal
 * @since jpicedt 1.3.2
 * @version $Id: TestCanvas.java,v 1.30 2006/06/17 22:34:46 reynal Exp $
 */
public class TestCanvas implements PicObjectConstants, Runnable {

	public PECanvas canvas;
	public Interpreter interpreter; // BSH
	private JLabel mouseCoordsLbl = new JLabel();
    public PicText text,text2;
    public PicNodeConnection con;
	
	/** test */
	public static void main(String arg[]){
		TestCanvas test = new TestCanvas();
	}
	
	public TestCanvas(){
		
		JPanel testCanvas = createTestCanvas();
		JConsole console = new JConsole();
		console.setPreferredSize(new Dimension(200,300));
		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, testCanvas, console);
		JFrame frame = new JFrame("test");
		frame.getContentPane().add(splitPane);
		frame.addWindowListener(new WindowAdapter(){public void windowClosing(WindowEvent e){System.exit(0);}});
		//System.out.println("Creating interpreter...");
		interpreter = new Interpreter(console);
		//System.out.println("Setting predefined variables and methods...");
		try {
			interpreter.eval("import jpicedt.*;");
			interpreter.eval("import jpicedt.test.*;");
			interpreter.eval("import jpicedt.graphic.*;");
			interpreter.eval("import jpicedt.graphic.model.*;");
			interpreter.eval("import jpicedt.graphic.event.*;");
			interpreter.eval("import jpicedt.graphic.grid.*;");
			interpreter.eval("import jpicedt.graphic.view.*;");
			interpreter.eval("import jpicedt.graphic.toolkit.*;");
			interpreter.eval("import jpicedt.graphic.io.formatter.*;");
			interpreter.eval("import jpicedt.graphic.io.parser.*;");
			//interpreter.eval("show();");
			interpreter.set("canvas",canvas);
			interpreter.set("drawing",canvas.getDrawing());
			interpreter.set("kit",canvas.getEditorKit());
			interpreter.set("selHandler",canvas.getEditorKit().getSelectionHandler());
				//"canvas=test.canvas;" +
				//"drawing=canvas.getDrawing();");
				//"preferences=jpicedt.JPicEdt.getPreferences();" +
				//"editorkit(){return jpicedt.JPicEdt.getSelectedEditorKit();}");
		}
		catch (Exception ex){ex.printStackTrace();}
		frame.pack();
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		frame.setLocation(screenSize.width-frame.getWidth(),0);
		frame.setVisible(true);
		Thread t = new Thread(this);t.start();
	}
	
	/**
	 * Run the BSH interpreter in interactive mode. 
	 */
	public void run(){
		//System.out.println("Starting interpreter...");
		interpreter.run(); // run interactively
	}
	

	public JPanel createTestCanvas(){
		JPanel pane = new JPanel();
		
		Grid grid = new  Grid(true, true, 10.0, 10.0, Color.LIGHT_GRAY, Grid.DASH);
		//grid.setVisible(false);
		grid.setSnapOn(true);
		canvas = new PECanvas(1.0, new PageFormat(200, 100, 50, 50),grid, null);  
		canvas.addPEMouseInputListener(new MouseHandler());
		//canvas.getEditorKit().addPropertyChangeListener(new PropertyChangeHandler());
		
		EditorKit kit = canvas.getEditorKit();
		PicAttributeSet set = kit.getInputAttributes();
		//set.setAttribute(RIGHT_ARROW, Arrow.ARROW_HEAD);
		//((DefaultSelectionHandler)canvas.getEditorKit().getSelectionHandler()).setHighlightingMode(DefaultSelectionHandler.LOCAL_MODE);

		
		//canvas.addPropertyChangeListener(new PropertyChangeHandler());
		//canvas.addSelectionListener(new SelectionHandler());
		//canvas.addZoomListener(new ZoomHandler());
		//canvas.getEditorKit().addHelpMessageListener(new HelpMessageHandler());
		//canvas.getDrawing().addDrawingListener(new DrawingHandler());
		
		pane.setLayout(new BorderLayout(5,5));
		pane.add(canvas,BorderLayout.CENTER);

		// add control panel
		Box p = new Box(BoxLayout.X_AXIS);
		
		JButton b = new JButton("Join c1/c2");
		b.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				//c1.join(c2);
				canvas.joinSelection();
				//canvas.getEditorKit().setCurrentMouseTool(DrawToolFactory.MULTI_CURVE);
			    //text.setAttribute(TEXT_FRAME, TEXT_BOX_OVAL);
				
		}});
		p.add(b);
		
				
		p.add(this.mouseCoordsLbl);
		

		ActionListener al;
		
		al = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				System.out.println();
				System.out.print("$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$");
				System.out.println("$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$");
				System.out.println();
		}};
		canvas.registerKeyboardAction(al, KeyStroke.getKeyStroke('$'),JComponent.WHEN_FOCUSED);

		al  = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				System.out.println(canvas.getDrawing());
				System.out.println();
		}};
		canvas.registerKeyboardAction(al, KeyStroke.getKeyStroke('a'),JComponent.WHEN_FOCUSED);
 		
		al  = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				canvas.deleteSelection();
		}};
		canvas.registerKeyboardAction(al, KeyStroke.getKeyStroke('s'),JComponent.WHEN_FOCUSED);

		ActionLocalizer localizer = jpicedt.Localizer.currentLocalizer().getActionLocalizer();
		ActionDispatcher dispatcher = new DefaultActionDispatcher(canvas);
		PEAction ac = new EditorKit.EditGeometryAction(dispatcher, localizer);
		canvas.registerKeyboardAction(ac, KeyStroke.getKeyStroke('c'),JComponent.WHEN_FOCUSED);

		pane.add(p,BorderLayout.SOUTH);

		
		// ---- PicNodeConnection test ---
		/*
		PicText nodeA = new PicText(new PicPoint(0,0), "nodeA", set);
		PicText nodeB = new PicText(new PicPoint(20,20), "nodeB", set);
		
		canvas.getDrawing().addElement(nodeA);
		canvas.getDrawing().addElement(nodeB);
		
		PicNodeConnection connectionAB = new PicNodeConnection(nodeA, nodeB, PicNodeConnection.EDGE_NCLINE, set);
		canvas.getDrawing().addElement(connectionAB);
		*/
		
		// --- bitmap for PicText test ---
		/*text = new PicText(new PicPoint(0,0), "1", set);
		text.setDimensions(5,2,0);
		canvas.getDrawing().addElement(text);
		PicAttributeSet set2=new PicAttributeSet(set);   
		//TEXT_VALIGN_BOTTOM  TEXT_VALIGN_BASELINE TEXT_VALIGN_CENTER TEXT_VALIGN_TOP
                set2.setAttribute(TEXT_VERT_ALIGN,TEXT_VALIGN_TOP);
		//TEXT_BOX_OVAL TEXT_BOX_CIRCLE
		set2.setAttribute(TEXT_FRAME, TEXT_BOX_RECTANGLE);
		text2 = new PicText(new PicPoint(30,30), "2", set2);
		text2.setDimensions(5,2,3);
		canvas.getDrawing().addElement(text2);
		con = new PicNodeConnection(text,text2,"ncline",set);
		canvas.getDrawing().addElement(con);*/
		
		PicMultiCurve c1 = new PicMultiCurve(new PicPoint(0,0),new PicPoint(0,10),new PicPoint(10,10),new PicPoint(20,10));
		PicMultiCurve c2 = new PicMultiCurve(new PicPoint(40,20),new PicPoint(50,30),new PicPoint(55,10),new PicPoint(60,20));
		PicMultiCurve c3 = new PicMultiCurve(new PicPoint(0,30),new PicPoint(50,30));
		//c2 = new PicMultiCurve(new PicPoint(40,20));
		//c2 = new PicEllipse(new PicPoint(-30,-30), new PicPoint(30,-30), new PicPoint(30,30),PicEllipse.OPEN);
		//c2.setAngleStart(0.0);c2.setAngleEnd(180.0);
		canvas.getDrawing().addElement(c1);
		canvas.getDrawing().addElement(c2);
		canvas.getDrawing().addElement(c3);
		
		/*
		PicParallelogram para = new PicParallelogram(new PicPoint(0,0), new PicPoint(20,0), new PicPoint(40,50));
		canvas.getDrawing().addElement(para);
		*/
		
		PicEllipse ell = new PicEllipse(new PicPoint(-30,-30), new PicPoint(30,-30), new PicPoint(30,30),PicEllipse.OPEN);
		ell.setAngleStart(0);ell.setAngleEnd(180);
		canvas.getDrawing().addElement(ell);
		
		/*
		PicParallelogram para = new PicParallelogram(new PicPoint(0,0), new PicPoint(10,0), new PicPoint(10,20));
		canvas.getDrawing().addElement(para);
		*/
		
		PicText text = new PicText();
		text.setPoint(PicText.P_ANCHOR, new PicPoint(50,50),null);
		text.setText("x=5+3*y");
		canvas.getDrawing().addElement(text);

		return pane;
	}

	///////////////////////////////////////////////////////////////////
	class MouseHandler extends PEMouseInputAdapter {
		
		public void mouseMoved(PEMouseEvent me){
			mouseCoordsLbl.setText(me.getPicPoint().toString());
		}
		
	}

	///////////////////////////////////////////////////////////////////
	class HelpMessageHandler implements HelpMessageListener {
		
		public void helpMessagePosted(HelpMessageEvent e){
			System.out.println("!!!!!!!!!! "+e.getMessage()+" !!!!!!!!!!");	
		}
	}
	
	///////////////////////////////////////////////////////////////////
	class DrawingHandler implements DrawingListener {
		
		public void changedUpdate(DrawingEvent e){
			jpicedt.Log.debug("got DrawingEvent e = " + e);
		}
	}
	///////////////////////////////////////////////////////////////////
	class SelectionHandler implements SelectionListener {
		
		public void selectionUpdate(SelectionEvent e){
			jpicedt.Log.debug("got SelectionEvent e = " + e);
		}
	}
	///////////////////////////////////////////////////////////////////
	class ZoomHandler implements ZoomListener {
		
		public void zoomUpdate(ZoomEvent e){
			jpicedt.Log.debug("got ZoomEvent e = " + e);
		}
	}
	/////////////////////////////////////////////////////////////////////
	class PropertyChangeHandler implements PropertyChangeListener { 

		public void propertyChange(PropertyChangeEvent e){
			jpicedt.Log.debug("propertyChange");
			jpicedt.Log.debugAppendLn("source="+e.getSource());
			jpicedt.Log.debugAppendLn("property name="+e.getPropertyName());
			jpicedt.Log.debugAppendLn("old value="+e.getOldValue());
			jpicedt.Log.debugAppendLn("new value="+e.getNewValue());
		}
	}
}
