/*  DialogAbout.java - jPicEdt, a picture editor for LaTeX.
    Copyright (C)   Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.ui.dialog;

import jpicedt.Version;
import jpicedt.Localizer;
import jpicedt.ui.*;
import jpicedt.graphic.PEToolKit;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;

/**
 * "About..." dialog box ; displays various informations such as copyright, author(s),...
 * @author Sylvain Reynal
 * @since PicEdt 1.0
 * @version $Id: DialogAbout.java,v 1.6 2006/06/17 22:34:47 reynal Exp $
 *
 */
public class DialogAbout extends JDialog implements ActionListener{

	JButton buttonOk;

	/**
	 * Construct a new "about..." dialog
	 */
	public DialogAbout() {

		super();
		setTitle(Localizer.currentLocalizer().get("about.AboutPicEdt")+" "+Version.getVersion());
		setModal(true);
		setResizable(true);
		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);

		buttonOk = new JButton(Localizer.currentLocalizer().get("button.OK"));
		buttonOk.addActionListener(this);

		JPanel buttonPanel = new JPanel(new FlowLayout(),false);
		buttonPanel.add(buttonOk);

		JPanel logoPanel = new JPanel(new FlowLayout(),false);
		logoPanel.setBorder(new EmptyBorder(10, 10, 10, 10));
		logoPanel.add(PEToolKit.createJLabel("Frankenstein"));

		JPanel infoPanel = new JPanel(new GridLayout(9,1,5,5),true);
		infoPanel.setBorder(new EmptyBorder(10, 60, 10, 10));
		infoPanel.add(new JLabel("jPicEdt " + Version.getVersion() + " Built " + Version.getBuildDate()));
		infoPanel.add(new JLabel(Localizer.currentLocalizer().get("about.APictureEditorFor")));
		infoPanel.add(new JLabel("Copyleft ! Sylvain Reynal"));
		infoPanel.add(new JLabel("E.N.S.E.A. - Dept. of Physics"));
		infoPanel.add(new JLabel("6, avenue du Ponceau"));
		infoPanel.add(new JLabel("F-95014 CERGY Cedex"));
		infoPanel.add(new JLabel("Fax : +33 (0) 130 736 667"));
		infoPanel.add(new JLabel("reynal@ensea.fr"));
		infoPanel.add(new JLabel("http://www.jpicedt.org"));

		JPanel caveatPanel = new JPanel();
		JTextArea caveatTA = new JTextArea(10,50);
		caveatTA.setLineWrap(true);
		caveatTA.setEditable(false);
		JScrollPane scrollCaveat = new JScrollPane(caveatTA);
		caveatTA.append(Localizer.currentLocalizer().get("gpl.line1"));
		caveatTA.append(Localizer.currentLocalizer().get("gpl.line2"));
		caveatTA.append(Localizer.currentLocalizer().get("gpl.line3"));
		caveatTA.append(Localizer.currentLocalizer().get("gpl.line4"));
		caveatTA.append(Localizer.currentLocalizer().get("gpl.line5"));
		caveatTA.append(Localizer.currentLocalizer().get("gpl.line6"));
		caveatTA.append(Localizer.currentLocalizer().get("gpl.line7"));
		caveatTA.append(Localizer.currentLocalizer().get("gpl.line8"));
		caveatTA.append(Localizer.currentLocalizer().get("gpl.line9"));
		caveatTA.append(Localizer.currentLocalizer().get("gpl.line10"));
		caveatPanel.add(scrollCaveat);
		caveatPanel.setBorder(BorderFactory.createEtchedBorder());

		JPanel upperPanel = new JPanel(new BorderLayout(),false);
		upperPanel.add(logoPanel, BorderLayout.WEST);
		upperPanel.add(infoPanel, BorderLayout.CENTER);
		upperPanel.add(caveatPanel, BorderLayout.SOUTH);
		upperPanel.setBorder(BorderFactory.createEtchedBorder());

		this.getContentPane().setLayout(new BorderLayout(5,5));
		this.getContentPane().add(upperPanel, BorderLayout.NORTH);
		this.getContentPane().add(buttonPanel, BorderLayout.SOUTH);

		Dimension dlgSize = this.getPreferredSize();
		setSize(dlgSize);
		Dimension frameSize = getSize();
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		if (frameSize.height > screenSize.height)
			frameSize.height = screenSize.height;
		if (frameSize.width > screenSize.width)
			frameSize.width = screenSize.width;
		setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);

		//this.pack();
		this.setVisible(true);
	}


	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == buttonOk) {      
			dispose();    
		}
	}
}

