/*
 DockableEditorKitToolBar.java - April 16, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://trashx.ensea.fr/jpicedt/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.ui.dialog;

import jpicedt.graphic.toolkit.*;
import jpicedt.graphic.*;
import jpicedt.graphic.event.*;
import jpicedt.ui.*;
import jpicedt.ui.action.*;
import jpicedt.*;

import javax.swing.*;
import java.util.*;
import java.beans.*;
import java.awt.*;

/**
 * A toolbar that offers various tools for drawing, editing, selecting...
 * @author Sylvain Reynal
 * @since PicEdt 1.3
 * @version $Id: DockableEditorKitToolBar.java,v 1.19.2.1 2007/09/02 11:56:30 reynal Exp $
 */
public class DockableEditorKitToolBar extends JPanel implements PropertyChangeListener {

	/** key for persistent storage */
	public static final String KEY = "dockable-panel.Toolkit";
	
	private ArrayList buttons = new ArrayList();
	private String currentMouseTool=EditorKit.SELECT;
	
	// --- class methods ------------------------------------------------

	/**
	 * @param actionDispatcher the ActionDispatcher that provides a
	 *        target when this Action gets invoked.
	 * @param actionLocalizer the ActionLocalizer that feeds this Action with localized properties ;
	 *        if null, the Action's NAME property is simply initialized to actionName, but other 
	 *        properties are not set
	 */
	public DockableEditorKitToolBar() {

		super(new GridLayout(8,4,5,5)); // 3*7=21 buttons. [reynal:16/01/2003]
		this.setBorder(BorderFactory.createEtchedBorder());

		// a set of mutually exclusive buttons, composing the tools palette :
		ButtonGroup bg = new ButtonGroup();
		JToggleButton b;

		ActionRegistry actionRegistry = JPicEdt.getActionRegistry();
		
		// add kit's registered mouse tool :
		String[][] allToolNames = EditorKit.getAvailableToolNames();
		for (int i=0; i<allToolNames.length; i++){
			for (int j=0; j<allToolNames[i].length; j++){
				Action a = actionRegistry.getAction(allToolNames[i][j]);
				b = addToggle(a);
				bg.add(b);
				if (allToolNames[i][j].equals(EditorKit.SELECT)) b.setSelected(true);
			}
		}

		// local/global mode :
		addToggle(actionRegistry.getAction(EditorKit.ToggleEditPointsModeAction.KEY));
		
	}
	
	/**
	 * Creates a JToggleButton from the given PEAction's properties
	 * @param a Action from which this button gets created.
	 */
	public JToggleButton addToggle(Action a){

		JToggleButton b = new JToggleButton(a);
		if (b.getIcon()!=null) b.setText(null);
		b.setMargin(new Insets(2,2,2,2));
		//b.setAlignmentY(CENTER_ALIGNMENT);
		//b.setAlignmentX(CENTER_ALIGNMENT);
		b.setRolloverEnabled(true);
        add(b);
		buttons.add(b);
        return b;
	}
	
	/**
	 * Creates a JButton from the given PEAction's properties
	 * @param a Action from which this button gets created.
	 */
	public JButton add(Action a){

		JButton b = new JButton(a);
		if (b.getIcon()!=null) b.setText(null);
		b.setMargin(new Insets(2,2,2,2));
		//b.setAlignmentY(CENTER_ALIGNMENT);
		//b.setAlignmentX(CENTER_ALIGNMENT);
		b.setRolloverEnabled(true);
        add(b);
		buttons.add(b);
        return b;
	}

	/**
	 * Implementation of PropertyChangeListener interface
	 * Allows this tool-bar to get notified of change from the target EditorKit, e.g. when
	 * a right-click switches the current EditorKit's mousetool back to "select-tool", or
	 * when a new mousetool gets selected from the "Tools" menu or from a BSH script.
	 * <p>
	 * This DockablePanel gets registered as a PropertyChangeListener (with events sourced from EditorKit's)
	 * from inside MDIManager whenever a new Board is added.
	 * @param e the PropertyChangeEvent ; only the "getNewValue()" method is used here.
	 */
	public void propertyChange(PropertyChangeEvent e){
		if (e.getPropertyName()==EditorKit.EDIT_MODE_CHANGE){
			this.currentMouseTool = (String)e.getNewValue();
			//System.out.println("EditorKit.EDIT_MODE_CHANGE : newValue = " + e.getNewValue());
			for (Iterator it = buttons.iterator(); it.hasNext();){
				Object o = it.next();
				if (!(o instanceof JToggleButton)) continue; // filter out not selectable buttons				
				JToggleButton c = (JToggleButton)o;
				Action a = c.getAction();
				if (e.getNewValue().equals(a.getValue(Action.ACTION_COMMAND_KEY))){ // actionCommand is used to build a PEAction
					c.setSelected(true);
					c.doClick();
				}
			}
		}
		// synchronize state of new board with this toolbar: 
		else if (e.getPropertyName()==MDIManager.ACTIVE_BOARD_CHANGE){
			if (e.getNewValue()==null) {}
			else {
				PEDrawingBoard b = (PEDrawingBoard)e.getNewValue();
				EditorKit kit = b.getCanvas().getEditorKit();
				kit.setCurrentMouseTool(currentMouseTool);
			}
		}
	}

	
} // classe EditorKitToolBar
