/*
 RenderingCustomizer.java - April 14, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.ui.dialog;

import jpicedt.Localizer;
import jpicedt.MiscUtilities;
import jpicedt.graphic.toolkit.AbstractCustomizer;
import jpicedt.graphic.PEToolKit;
import jpicedt.graphic.PECanvas;
import jpicedt.widgets.*;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.util.*;

/**
 * a panel for editing rendering-preferences (anti-aliasing,...) 
 *
 * @author Sylvain Reynal
 * @since jPicEdt 1.3
 * @version $Id: RenderingCustomizer.java,v 1.5.2.1 2007/09/02 11:56:30 reynal Exp $
 */
public class RenderingCustomizer extends AbstractCustomizer {

	private Properties preferences;
	private JLabel l;
	private JCheckBox antiAliasingCB, qualityCB,textAntiAliasingCB,fractionalMetricsCB,ditherCB;


	/**
	 * creates a new panel for general preferences. The given Properties is used to load widgets values,
	 * then to store new values.
	 */
	public RenderingCustomizer(Properties preferences){

		this.preferences = preferences;
		JPanel p;

		p = new JPanel(new GridLayout(5,1,5,5));
		p.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Localizer.currentLocalizer().get("preferences.Rendering")));

		p.add(antiAliasingCB = new JCheckBox(Localizer.currentLocalizer().get("preferences.Rendering.anti-aliasing")));
		p.add(textAntiAliasingCB = new JCheckBox(Localizer.currentLocalizer().get("preferences.Rendering.text-anti-aliasing")));
		p.add(qualityCB = new JCheckBox(Localizer.currentLocalizer().get("preferences.Rendering.quality")));
		qualityCB.setToolTipText(Localizer.currentLocalizer().get("preferences.Rendering.quality.tooltip"));
		p.add(ditherCB = new JCheckBox(Localizer.currentLocalizer().get("preferences.Rendering.dither")));
		p.add(fractionalMetricsCB = new JCheckBox(Localizer.currentLocalizer().get("preferences.Rendering.fractional-metrics")));

		add(p, BorderLayout.NORTH);
		
	}

	/**
	 * @return the panel title, used e.g. for Border or Tabpane title. 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public String getTitle(){ 
		return Localizer.currentLocalizer().get("preferences.Rendering");	
	}
	
	
	/**
	 * @return the Icon associated with this panel, used e.g. for TabbedPane decoration 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public Icon getIcon(){ 
		return null;	
	}
	
	
	/**
	 * @return the tooltip string associated with this panel 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public String getTooltip(){ return Localizer.currentLocalizer().get("preferences.Rendering.tooltip");	}

	/**
	 * Load widgets display content with a default value retrieved from the Options class. 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public void loadDefault() {
		antiAliasingCB.setSelected(false);
		qualityCB.setSelected(false);
		textAntiAliasingCB.setSelected(false);
		fractionalMetricsCB.setSelected(false);
		ditherCB.setSelected(false);
	}

	/**
	 * Load widgets value from the Options class. 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public void load() {
		RenderingHints rh = MiscUtilities.parseRenderingHints(preferences);
		if (rh.get(RenderingHints.KEY_ANTIALIASING)!=null)
			antiAliasingCB.setSelected(rh.get(RenderingHints.KEY_ANTIALIASING).equals(RenderingHints.VALUE_ANTIALIAS_ON));
		if (rh.get(RenderingHints.KEY_RENDERING)!=null)
			qualityCB.setSelected(rh.get(RenderingHints.KEY_RENDERING).equals(RenderingHints.VALUE_RENDER_QUALITY));
		if (rh.get(RenderingHints.KEY_TEXT_ANTIALIASING)!=null)
			textAntiAliasingCB.setSelected(rh.get(RenderingHints.KEY_TEXT_ANTIALIASING).equals(RenderingHints.VALUE_TEXT_ANTIALIAS_ON));
		if (rh.get(RenderingHints.KEY_FRACTIONALMETRICS)!=null)
			fractionalMetricsCB.setSelected(rh.get(RenderingHints.KEY_FRACTIONALMETRICS).equals(RenderingHints.VALUE_FRACTIONALMETRICS_ON));
		if (rh.get(RenderingHints.KEY_DITHERING)!=null)
			ditherCB.setSelected(rh.get(RenderingHints.KEY_DITHERING).equals(RenderingHints.VALUE_DITHER_ENABLE));
	}

	/**
	 * Store current widgets value to the Properties object given in the constructor.
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	 public void store(){
		RenderingHints rh = new RenderingHints(null); 
		String str;
		if (antiAliasingCB.isSelected()) rh.put(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
		else rh.put(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_OFF);
		if (textAntiAliasingCB.isSelected()) rh.put(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
		else rh.put(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		if (!qualityCB.isSelected()) rh.put(RenderingHints.KEY_RENDERING,RenderingHints.VALUE_RENDER_SPEED);
		else rh.put(RenderingHints.KEY_RENDERING,RenderingHints.VALUE_RENDER_QUALITY);
		if (ditherCB.isSelected()) rh.put(RenderingHints.KEY_DITHERING,RenderingHints.VALUE_DITHER_ENABLE);
		else rh.put(RenderingHints.KEY_DITHERING,RenderingHints.VALUE_DITHER_DISABLE);
		if (fractionalMetricsCB.isSelected()) rh.put(RenderingHints.KEY_FRACTIONALMETRICS,RenderingHints.VALUE_FRACTIONALMETRICS_ON);
		else rh.put(RenderingHints.KEY_FRACTIONALMETRICS,RenderingHints.VALUE_FRACTIONALMETRICS_OFF);
		MiscUtilities.formatRenderingHints(rh, preferences);
	}

} // class


