/*
 UIColorPanel.java - December 31, 2001 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.ui.dialog;

import jpicedt.Localizer;
import jpicedt.graphic.toolkit.AbstractCustomizer;
import jpicedt.graphic.grid.Grid;
import jpicedt.ui.internal.InternalFrameMDIManager;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.IOException;
import java.util.MissingResourceException;

/**
 * a panel for UI colours editing (grid, desktop,...)
 * [pending] selection highlighter colour
 * @author Sylvain Reynal
 * @since PicEdt 1.3
 */
public class UIColorCustomizer extends AbstractCustomizer implements ActionListener {

	private JButton gridColorB, selectionColorB, endPointColorB; 
	private JButton boundingareaColorB, desktopColorB, textColorB;
	private Properties preferences;

	/**
	 * construct a new panel for UI colours editing
	 */
	public UIColorCustomizer(Properties preferences){
		this.preferences  = preferences;
		JPanel p = new JPanel(new GridLayout(2,1,5,5));
		p.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Localizer.currentLocalizer().get("preferences.Colors")));

		gridColorB = new JButton(Localizer.currentLocalizer().get("preferences.Grid"));
		gridColorB.setToolTipText(Localizer.currentLocalizer().get("preferences.Grid"));
		gridColorB.addActionListener(this);
		p.add(gridColorB);

		desktopColorB = new JButton(Localizer.currentLocalizer().get("preferences.Desktop"));
		desktopColorB.setToolTipText(Localizer.currentLocalizer().get("preferences.Desktop"));
		desktopColorB.addActionListener(this);
		p.add(desktopColorB);

		add(p, BorderLayout.NORTH);
	}

	/**
	 * @return the panel title, used e.g. for Border or Tabpane title. 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public String getTitle(){ 
		return Localizer.currentLocalizer().get("preferences.Colors");	
	}


	/**
	 * @return the Icon associated with this panel, used e.g. for TabbedPane decoration 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public Icon getIcon(){ 
		return null;	
	}


	/**
	 * @return the tooltip string associated with this panel 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public String getTooltip(){ 
		return Localizer.currentLocalizer().get("preferences.ColorChooser");	
	}

	/**	/**
	 * Load widgets display content with a default value retrieved from the Options class. 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public void loadDefault() {
		
		/*
		selectionColorB.setBackground(Options.selectionColorDEFAULT);
		endPointColorB.setBackground(Options.endPointColorDEFAULT);
		textColorB.setBackground(Options.textColorDEFAULT);
		boundingareaColorB.setBackground(Options.boundingareaColorDEFAULT);
		*/
		gridColorB.setBackground(Grid.colorDEFAULT);
		desktopColorB.setBackground(InternalFrameMDIManager.desktopColorDEFAULT);
	}

	/**
	 * Load widgets value from the Options class.
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public void load() {
		/*
		selectionColorB.setBackground(Options.selectionColor);
		endPointColorB.setBackground(Options.endPointColor);
		textColorB.setBackground(Options.textColor);
		boundingareaColorB.setBackground(Options.boundingareaColor);
		*/
		gridColorB.setBackground(jpicedt.MiscUtilities.parseProperty(preferences,
															Grid.KEY_GRID_COLOR,Grid.colorDEFAULT));
		desktopColorB.setBackground(jpicedt.MiscUtilities.parseProperty(preferences,
				InternalFrameMDIManager.KEY_DESKTOP_COLOR,InternalFrameMDIManager.desktopColorDEFAULT));

	}	
	
	/**
	 * Store current widgets value, presumably to a file or to a dedicated storage class
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	 public void store(){
		 store(Grid.KEY_GRID_COLOR, gridColorB);
		 store(InternalFrameMDIManager.KEY_DESKTOP_COLOR, desktopColorB);
		/*
		Options.selectionColor = selectionColorB.getBackground();
		Options.endPointColor = endPointColorB.getBackground();
		Options.boundingareaColor = boundingareaColorB.getBackground();
		Options.textColor = textColorB.getBackground();
		*/
	}
	
	// convenience used by store
	private void store(String key, JButton button){
		preferences.setProperty(key, Integer.toString(button.getBackground().getRGB()));
	}

	/** open Color Chooser */
	public void actionPerformed(ActionEvent e){

		if (e.getSource() instanceof JButton){
			JButton b = (JButton)e.getSource();
			Color newColor = JColorChooser.showDialog(null, b.getText(), b.getBackground());
			if (newColor != null) b.setBackground(newColor);
		}
	}
} // PanelColor


