/*
 ViewFormattedStringDialog.java - December 31, 2001 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.ui.dialog;

import jpicedt.graphic.PECanvas;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.io.*;

/**
 * This class implements a dialog box that displays the LaTeX/PsTricks/... formatted string
 * and allows the user to edit, then (re)parse the text. 
 * corresponding to a given object
 *
 * @since  jPicEdt 1.2.a
 * @author Sylvain Reynal
 * @version $Id: ViewFormattedStringDialog.java,v 1.7.2.1 2007/09/02 11:56:30 reynal Exp $
 *
 */
public class ViewFormattedStringDialog extends JDialog implements ActionListener{

	private PECanvas canvas;
	private JTextArea ta;
	private JButton buttonOk, buttonReload, buttonParse;

	/**
	 * @param str String to be displayed
	 * @since jPicEdt 1.2.a
	 */
	public ViewFormattedStringDialog(PECanvas canvas) {

		super();
		setTitle(jpicedt.Localizer.currentLocalizer().get("preferences.LatexString"));
		setModal(false);
		setResizable(true);
		getContentPane().setLayout(new BorderLayout());
		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		
		this.canvas = canvas;

		JPanel panelString = new JPanel();
		panelString.setBorder(BorderFactory.createEtchedBorder());

		ta = new JTextArea( 40, 60 );
		ta.setEditable(true);
		load();
		
		JScrollPane scrollString = new JScrollPane(ta);
		//panelString.add(scrollString);

		JPanel panelButtons = new JPanel();
		panelButtons.setBorder(BorderFactory.createEtchedBorder());

		buttonOk = new JButton(jpicedt.Localizer.currentLocalizer().get("button.OK"));
		buttonOk.setDefaultCapable(true);
		buttonOk.addActionListener(this);
		panelButtons.add(buttonOk);

		buttonReload = new JButton(jpicedt.Localizer.currentLocalizer().get("button.Reload"));
		buttonReload.addActionListener(this);
		panelButtons.add(buttonReload);

		buttonParse = new JButton(jpicedt.Localizer.currentLocalizer().get("preferences.UpdateDrawing"));
		buttonParse.addActionListener(this);
		panelButtons.add(buttonParse);

		getContentPane().add(scrollString,BorderLayout.CENTER);
		getContentPane().add(panelButtons,BorderLayout.SOUTH);
		getRootPane().setDefaultButton(buttonOk);

		pack();

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = getSize();
		if (frameSize.height > screenSize.height)
			frameSize.height = screenSize.height;
		if (frameSize.width > screenSize.width)
			frameSize.width = screenSize.width;
		setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
		
		setVisible(true);
		
	}

	/**
	 * Reload the content of the TextArea with the formatted representation of the Drawing this Dialog
	 * is attached to.
	 */
	private void load(){
		StringWriter sw = new StringWriter();
		try {
			canvas.write(sw,false); // write whole drawing
			ta.setText(sw.toString());
		}
		catch (IOException ioex){ioex.printStackTrace();}
	}
	
	/**
	 * Reload the Drawing this dialog is attached to, from the content of the TextArea.
	 */
	private void parse(){
		try {
			StringReader sr = new StringReader(ta.getText());
			canvas.read(sr, jpicedt.MiscUtilities.createParser());
		}
		catch (jpicedt.graphic.io.parser.ParserException pEx){
			JOptionPane.showMessageDialog(
				null,
				jpicedt.Localizer.currentLocalizer().get("exception.ParserWarning") + " :\n" + pEx,
				jpicedt.Localizer.currentLocalizer().get("action.editorkit.Paste"),
				JOptionPane.ERROR_MESSAGE);
		}
	}


	/** 
	 * Dispose this dialog box when "ok" is pressed.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource()==buttonOk) dispose();
		else if (e.getSource()==buttonReload) load();
		else if (e.getSource()==buttonParse) parse();		
	}

} // class



