/*
 DebugFocusManager.java - January 1, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.ui.util;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * A subclass of Swing's DefaultFocusManager that aims at helping us debug FocusEvent related bugs<p>
 * To set this class as Swing's FocusManager, just say :<br>
 * <code>FocusManager.setCurrentManager(an_instance_of_DebugFocusManager);</code>
 *  
 * @since jPicEdt
 * @author Sylvain Reynal
 */
public class DebugFocusManager extends DefaultFocusManager  {


	/**
	 * Cause the focus manager to set the focus on the next focusable component<p>
	 * Overriden so as to display information about the component.
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public void focusNextComponent(Component c){
		
		System.out.println("DebugFocusManager.focusNextComponent : " + c);
		super.focusNextComponent(c);
	}
	
	/**
	 * Cause the focus manager to set the focus on the previous focusable component<p>
	 * Overriden so as to display information about the component.
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public void focusPreviousComponent(Component c){
		
		System.out.println("DebugFocusManager.focusPreviousComponent : " + c);
		super.focusPreviousComponent(c);
	}
	
	/**
	 * Called by a JComponent when a key event occus.<p>
	 * Overriden so as to display information about the component.
	 * @param c the focused component
	 * @param ke the key event
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public void processKeyEvent(Component c, KeyEvent ke){
		
		System.out.println("DebugFocusManager.processKeyEvent : " + ke + " on component " + c);
		super.processKeyEvent(c,ke);
	}

	
} // class 

