/*
 Fragments.java - August 11, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.ui.util;

import jpicedt.JPicEdt;
import jpicedt.ui.dialog.PEFileChooser;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;


/**
 * Fragments menu management
 * @since jPicEdt
 * @author Sylvain Reynal
 * @version $Id: Fragments.java,v 1.4.2.1 2007/09/02 11:56:30 reynal Exp $
 */
public class Fragments  {

	
	/**
	 * Return a JMenu containing a hierarchy of JMenu's and JMenuItem's built from
	 * the fragments contained, first in the installation fragment directory, then
	 * in the user fragment directory.
	 * @return an empty JMenu if no fragments were found in either directory.
	 */
	public static JMenu createMenu(){
		// 1°) scan install fragment directory :
		String jpicedtHome = jpicedt.MiscUtilities.getJPicEdtHome();
		JMenu installFragmentsMenu = null;
		if (jpicedtHome!=null) installFragmentsMenu=createMenu(new File(jpicedtHome,"fragments"));
		// 2°) scan user fragment directory :
		JMenu userFragmentsMenu = null; 
		if (JPicEdt.getUserSettingsDirectory() != null) 
			userFragmentsMenu = createMenu(new File(JPicEdt.getUserSettingsDirectory(),"fragments"));
		// build array :
		JMenu menu = new JMenu("Fragments"); // string not used !
		if (installFragmentsMenu != null) {
			Component[] cc = installFragmentsMenu.getMenuComponents();
			for(int i=0; i<cc.length; i++){
				menu.add(cc[i]);
			}
		}
		menu.add(new JSeparator());
		if (userFragmentsMenu != null) {
			Component[] cc = userFragmentsMenu.getMenuComponents();
			for(int i=0; i<cc.length; i++){
				menu.add(cc[i]);
			}
		}
		return menu;
	}

	/**
	 * Return a JMenu containing a hierarchy of JMenu's and JMenuItem's built from
	 * the fragments contained in the given directory and its children.
	 * @param directory path where to look for fragments, or subdirectories.
	 * @return null if directory (or one of its children) doesn't exit, or contains no fragment.
	 */
	private static JMenu createMenu(File directory){
		if (!directory.exists()) return null;
		String[] dirContent = directory.list();
		if (dirContent==null) return null; // directory is empty
		Arrays.sort(dirContent);
		// [pending] sort dir content
		JMenu menu = null; // so that we return null if nothing gets added...
		for (int i=0; i<dirContent.length; i++){
			String fileName = dirContent[i];
			File file = new File(directory, fileName); // create full path name
			// 1°) file is a subdirectory :
			if (file.isDirectory()){
				JMenu subMenu = createMenu(file);
				if (subMenu != null) {
					if (menu==null) menu = new JMenu(directory.getName());
					menu.add(subMenu);
				}
			}
			// 2°) file is a fragment :
			else {
				int indexOfDot = fileName.indexOf(".");
				if (indexOfDot==-1) indexOfDot = fileName.length();
				String label = fileName.substring(0,indexOfDot).replace('_',' ');
				JMenuItem menuItem = new JMenuItem(label);
				menuItem.addActionListener(new JPicEdt.FragmentInsertAction(file.getPath()));
				if (menu==null) menu = new JMenu(directory.getName());
				menu.add(menuItem);
			}
		}
		return menu;
	}
	
}


