/*
 HtmlViewer.java - March 3, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package jpicedt.ui.util;

import jpicedt.Localizer;

import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;
import java.net.*;

/**
 * A frame for displaying HTML content (e.g. on-line help, license,...)
 *
 * @author Sylvain Reynal.
 * @since PicEdt 1.1.2
 * @version $Id: HtmlViewer.java,v 1.10.2.1 2007/09/02 11:56:30 reynal Exp $
 *
 */
public class HtmlViewer extends JFrame implements HyperlinkListener{

	private JEditorPane textPane;
	private JLabel statusLbl;

	/**
	 * Construct a new HTML Window with the given file and title
	 * @param fileName name of resource accessible from the current classloader
	 */
	public HtmlViewer(String fileName, String title) {
		this(HtmlViewer.class.getResource(fileName),title);
	}
	/**
	 * Construct a new HTML Window with the given file and title
	 */
	public HtmlViewer(URL contentUrl, String title) {

		super(title);
		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		getContentPane().setLayout(new BorderLayout(5,5));

		textPane = new JEditorPane();
		textPane.setEditable(false);
		textPane.addHyperlinkListener(this);

		// sizing
		JScrollPane scroller = new JScrollPane(textPane);
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		scroller.setPreferredSize(new Dimension(screenSize.width/2, screenSize.height/2));
		this.getContentPane().add(scroller,BorderLayout.CENTER);
		
		// status bar
		statusLbl = new JLabel("Status...");
		this.getContentPane().add(statusLbl,BorderLayout.SOUTH);

		//pack();
		setSize(getPreferredSize());
		Dimension frameSize = getSize();
		if (frameSize.height > screenSize.height)
			frameSize.height = screenSize.height;
		if (frameSize.width > screenSize.width)
			frameSize.width = screenSize.width;
		setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
		setVisible(true);

		// install HTML page
		try {
			statusLbl.setText("Loading " + contentUrl + "...");
			textPane.setPage(contentUrl);
			statusLbl.setText(contentUrl.toString());
		}
		catch (Exception ex){
			ex.printStackTrace();
			JOptionPane.showMessageDialog(this,
							ex.toString(),
							//Localizer.currentLocalizer().get("exception.NetworkError"),
							Localizer.currentLocalizer().get("misc.PicEdtHelp"),
							JOptionPane.ERROR_MESSAGE);
			//textPane.setText(ex.toString());
			dispose();
			return;
		}
		
	}

	/**
	 * called when a click occurs on a HTML hyperlink (aka <a href=...>)
	 */
	public void hyperlinkUpdate(HyperlinkEvent e) {

		//System.out.println(e);
		
		if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {

			JEditorPane pane = (JEditorPane) e.getSource();
			//System.out.println(e.getDescription());

			// relative link :
			if (e instanceof HTMLFrameHyperlinkEvent && !e.getDescription().startsWith("http://")  && !e.getDescription().startsWith("../api-doc")) { // [pending] ../../api-doc
				HTMLFrameHyperlinkEvent  evt = (HTMLFrameHyperlinkEvent)e;
				HTMLDocument doc = (HTMLDocument)pane.getDocument();
				statusLbl.setText("Loading " + e.getURL().toString() + "...");
				doc.processHTMLFrameHyperlinkEvent(evt);
				statusLbl.setText(e.getURL().toString());
			}
			else {
				JFrame f = new HtmlViewer(e.getURL(),e.getURL().toString());
				f.setLocation(this.getLocation().x+30,this.getLocation().y+30);
			}
		}
	}
	
	/** test */
	public static void main(String args[]){
		try {
			new HtmlViewer(new URL(args[0]),args[0]);
		}
		catch (Exception ex){
			ex.printStackTrace();
		}
	}
} // class



