/*
 ModalInternalFrame.java - April 15, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://trashx.ensea.fr/jpicedt/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.widgets;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.beans.*;

/**
 * Re-implementation of JDialog using JInternalFrame's.
 * @author Sylvain Reynal
 * @since jpicedt 1.3.2
 * @version $Id: ModalInternalFrame.java,v 1.5.2.1 2007/09/02 11:56:31 reynal Exp $
 * <p>
 *
 */
public class ModalInternalFrame extends JInternalFrame {

	final JPanel glass = new JPanel();

	/**
	 * Constructor to be used to display a JOptionPane wrapped in modal JInternalFrame
	 */
	public ModalInternalFrame(String title, JRootPane rootPane, Component desktop, JOptionPane optionPane) {
		this(title,rootPane,desktop,(JComponent)optionPane);
		// Define close behavior
		optionPane.addPropertyChangeListener(new OptionPanePropertyChangeListener());
		// Change frame border
		putClientProperty("JInternalFrame.frameType","optionDialog");
	}


	/**
	 * constructor to be used to wrap any JComponent in a modal internal frame
	 */
	public ModalInternalFrame(String title, JRootPane rootPane, Component desktop, JComponent innerPane) {
		super(title);

		// create opaque glass pane
		glass.setOpaque(false);

		// Attach dummy mouse listeners to filter events
		MouseInputAdapter adapter = new MouseInputAdapter(){};
		glass.addMouseListener(adapter);
		glass.addMouseMotionListener(adapter);

		// Add option pane to this internal frame
		getContentPane().add(innerPane, BorderLayout.CENTER);


		// Size frame
		Dimension size = getPreferredSize();
		Dimension rootSize = desktop.getSize();

		setBounds((rootSize.width - size.width) / 2,(rootSize.height - size.height) / 2,size.width, size.height);
		desktop.validate();
		try {
			setSelected(true);
		} catch (PropertyVetoException ignored) {}

		// Add modal internal frame to glass pane
		glass.add(this);

		// Change glass pane to our panel
		rootPane.setGlassPane(glass);

		// Show glass pane, then modal dialog
		glass.setVisible(true);
	}

	private class OptionPanePropertyChangeListener implements PropertyChangeListener {

		public void propertyChange(PropertyChangeEvent event) {
			if (isVisible() && (event.getPropertyName().equals(JOptionPane.VALUE_PROPERTY) ||
			                    event.getPropertyName().equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
				try {
					setClosed(true);
				} catch (PropertyVetoException ignored) {}
				setVisible(false);
				glass.setVisible(false);
			}
		}
	}

	public void setVisible(boolean value) {
		super.setVisible(value);
		if (value) {
			_startModal();
		} else {
			_stopModal();
		}
	}

	private synchronized void _startModal() {
		try {
			if (SwingUtilities.isEventDispatchThread()) {
				EventQueue theQueue = getToolkit().getSystemEventQueue();
				while (isVisible()) {
					AWTEvent event = theQueue.getNextEvent();
					Object source = event.getSource();
					if (event instanceof ActiveEvent) {
						((ActiveEvent)event).dispatch();
					}
					else if (source instanceof Component) {
						((Component)source).dispatchEvent(event);
					}
					else if (source instanceof MenuComponent) {
						((MenuComponent)source).dispatchEvent(event);
					}
					else {
						System.err.println("Unable to dispatch: " + event);
					}
				}
			}
			else {
				while (isVisible()) wait();
			}
		} catch (InterruptedException ignored) {}
	}

	private synchronized void _stopModal() {
		notifyAll();
	}

	///////////////////////////////////////////////////////////////////////////////
	/**
	 * test - code snippet from Java Developper Connection.
	 */
	public static void main(String args[]) {
		final JFrame frame = new JFrame("Modal Internal Frame");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		final JDesktopPane desktop = new JDesktopPane();

		ActionListener showModal = new ActionListener() {
			                           Integer ZERO = new Integer(0);
			                           Integer ONE = new Integer(1);
			                           public void actionPerformed(ActionEvent e) {

				                           // Manually construct an input popup
				                           JOptionPane optionPane = new JOptionPane(
				                                                        "Print?", JOptionPane.QUESTION_MESSAGE,
				                                                        JOptionPane.YES_NO_OPTION);

				                           // Construct a message internal frame popup
				                           JInternalFrame modal = new ModalInternalFrame("Really Modal",
				                                                  frame.getRootPane(), desktop, optionPane);

				                           modal.setVisible(true); // block until ok/cancel

				                           Object value = optionPane.getValue();
				                           if (value.equals(ZERO)) {
					                           System.out.println("Selected Yes");
				                           } else if (value.equals(ONE)) {
					                           System.out.println("Selected No");
				                           } else {
					                           System.err.println("Input Error");
				                           }
			                           }
		                           };

		JInternalFrame internal = new JInternalFrame("Opener");
		desktop.add(internal);

		JButton button = new JButton("Open");
		button.addActionListener(showModal);

		Container iContent = internal.getContentPane();
		iContent.add(button, BorderLayout.CENTER);
		internal.setBounds(25, 25, 200, 100);
		internal.setVisible(true);

		Container content = frame.getContentPane();
		content.add(desktop, BorderLayout.CENTER);
		frame.setSize(500, 300);
		frame.setVisible(true);
	}
}


