package se.kth.bankingwebapp;

import static se.kth.bankingwebapp.DatabaseManager.DB;
import static se.kth.bankingwebapp.Util.*;

import java.io.*;
import java.util.List;

import javax.servlet.*;
import javax.servlet.http.*;

public class Accounts extends HttpServlet {

    protected void doGet(HttpServletRequest req, HttpServletResponse resp)
            throws ServletException, IOException {
        
        PrintWriter os = resp.getWriter();
        
        if (!loggedIn(req)) {
            os.print("<html><head><link rel=stylesheet href='style.css' type='text/css' /></head><body><center>Access denied<br/>Please <a href='login'>login</a>.</center></body></html>");
            return;
        }
        
        printPage(req, os, null);
    }
    
    
    protected void doPost(HttpServletRequest req, HttpServletResponse resp)
            throws ServletException, IOException {
        
        PrintWriter os = resp.getWriter();
        
        if (!loggedIn(req)) {
            os.print("<html><head><link rel=stylesheet href='style.css' type='text/css' /></head><body><center>Access denied<br/>Please <a href='login'>login</a>.</center></body></html>");
            return;
        }
        
        String cmd = getParam(req, "cmd");
        
        String msg = null;
        if (cmd.equals("transfer"))
            msg = doTransfer(req, os);
        
        printPage(req, os, msg);
    }
    
    
    private void printPage(HttpServletRequest req, PrintWriter os, String msg) throws IOException {
        int userId = getLoginId(req);
        String name = DB.getUserName(userId);
        List<String> accounts = DB.getAccountsFor(userId);
        
        os.print("<html>\n");
        os.print("<head><link rel=stylesheet href='style.css' type='text/css' /></head>");
        os.print("<h1>Overview</h1>");
        os.print("<p>Welcome " + name + ".</p>");
        
        os.print("<h2>Accounts</h2>");
        os.print("<center>");
        os.print("<table id='accounttbl'>\n");
        os.print("<tr class='header'><th>Account</th><th style='text-align: right;'>Balance</th></tr>\n");
        for (String account : accounts)
            os.print("<tr><td>"+account+"</td><td  style='text-align: right;'>"+formatCurrency(DB.getBalance(userId, account))+"</td></tr>\n");
        
        os.print("</table>\n");
        os.print("</center>");
        
        os.print("<h2>Transfer</h2>\n");
        
        os.print("<center>");
        os.print("<form action='accounts' method='post'>\n");
        os.print("<table border=0>");
        os.print("<tr><td>From account:</td><td><select name='src'>");
        for (String account : accounts)
            os.print("<option value='" + account + "'>" + account + "</option>");
        os.print("</select></td></tr>\n");
        os.print("<tr><td>To account:</td><td><select name='dst'>");
        for (String account : accounts)
            os.print("<option value='" + account + "'>" + account + "</option>");
        os.print("</select></td></tr>\n");
        
        os.print("<tr><td>Ammount:</td><td><input type='text' style='width: 100px;' name='ammount' /> <input type='submit' value='transfer' name='cmd' /></td></tr>\n");
        os.print("</table>\n");
        
        if (msg != null)
            os.print("<div class='msg'>" + msg + "</div>");
        
        os.print("</form>\n");
        os.print("</center>");
        
        os.print("<hr />");
        os.print("<form action='login' method='post'>");
        os.print("<div style='float: right;'>Logged in as <b>" + name + "</b>");
        os.print(" <input type='submit' value='logout' name='cmd' /><br />");
        os.print(" " + req.getSession(false).getId());
        os.print("</div></form>");
        
        os.print("</html>\n");
    }
    
    
    private int getLoginId(HttpServletRequest req) {
        if (!loggedIn(req))
            throw new IllegalStateException("Not logged in");
        return (Integer) req.getSession().getAttribute("loginId");
    }
    
    
    private boolean loggedIn(HttpServletRequest req) {
        HttpSession session = req.getSession(false);
        return session != null && session.getAttribute("loginId") != null;
    }
    
    
    private String doTransfer(HttpServletRequest req, PrintWriter os)
            throws IOException {
        
        int userId = getLoginId(req);
        String src = getParam(req, "src");
        String dst = getParam(req, "dst");
        String ammount = getParam(req, "ammount");
        try {
            return DB.transfer(userId, src, dst, Integer.parseInt(ammount));
        } catch (NumberFormatException nfe) {
            return "Bad ammount.";
        }
        
    }
    
}
