package se.kth.bankingwebapp;

import java.sql.*;
import java.util.*;

public enum DatabaseManager {
    
    DB;
    
    Connection c = null;
    
    
    public void init() throws SQLException, ClassNotFoundException {
        Class.forName("org.hsqldb.jdbcDriver" );
        c = DriverManager.getConnection("jdbc:hsqldb:mem:bankdb", "sa", "");
        initTables();
    }
    
    
    private void initTables() throws SQLException {
        String sql = "CREATE TABLE user (" +
                         "id INT NOT NULL IDENTITY," +
                         "name VARCHAR(80) NOT NULL," +
                         "pass VARCHAR(80) NOT NULL" +
                     ")";
        PreparedStatement createTbl = c.prepareStatement(sql);
        createTbl.executeUpdate();
        createTbl.close();
        
        sql = "CREATE TABLE account (" +
                  "userId INT," +
                  "account VARCHAR(80)," +
                  "balance INT" +
              ")";
        createTbl = c.prepareStatement(sql);
        createTbl.executeUpdate();
        createTbl.close();
        
        String[][] users = {
                { "alpha", "a", "Account A", "Account B", "Account C" },
                { "beta", "b", "Account D", "Account E" },
                { "gamma", "c", "Account F", "Account G" },
                { "delta", "d", "Account H", "Account I", "Account J" }
        };
        
        Random rnd = new Random();
        for (int userId = 0; userId < users.length; userId++) {
            addUser(userId, users[userId][0], users[userId][1]);
            for (int j = 2; j < users[userId].length; j++)
                addAccount(userId, users[userId][j], 1000 + rnd.nextInt(500));
        }
    }
    
    
    public void addUser(int id, String user, String pass) throws SQLException {
        String sql = "INSERT INTO user(id, name, pass) VALUES (?, ?, ?)";
        PreparedStatement addUser = c.prepareStatement(sql);
        addUser.setInt(1, id);
        addUser.setString(2, user);
        addUser.setString(3, pass);
        addUser.executeUpdate();
        addUser.close();
    }
    
    
    /*private int getUserCount() {
        int count = 0;
        try {
            String sql = "SELECT COUNT(*) AS count FROM user";
            PreparedStatement ps = c.prepareStatement(sql);
            ResultSet rs = ps.executeQuery();
            rs.first();
            count = rs.getInt("count");
            rs.close();
            ps.close();
        } catch (SQLException sqle) {
            System.err.println("Could not validate user.");
            sqle.printStackTrace();
        }
        return count;
    }*/
    
    
    private int getFreshUserId() {
        int freshId;
        try {
            String sql = "SELECT MAX(id)+1 AS freshid FROM user";
            PreparedStatement ps = c.prepareStatement(sql);
            ResultSet rs = ps.executeQuery();
            rs.next();
            freshId = rs.getInt("freshid");
            rs.close();
            ps.close();
        } catch (SQLException sqle) {
            System.err.println("Could not validate user.");
            sqle.printStackTrace();
            return -1;
        }
        return freshId;
    }
    
    
    private void addAccount(int userId, String account, int balance) throws SQLException {
        String sql = "INSERT INTO account (userId, account, balance) VALUES (?, ?, ?)";
        PreparedStatement addAccount = c.prepareStatement(sql);
        addAccount.setInt(1, userId);
        addAccount.setString(2, account);
        addAccount.setInt(3, balance);
        addAccount.executeUpdate();
        addAccount.close();
    }
    
    
    public boolean isUserPassValid(String name, String pass) {
        boolean valid = false;
        try {
            String sql = "SELECT * FROM user WHERE name = ? AND pass = ?";
            PreparedStatement ps = c.prepareStatement(sql);
            ps.setString(1, name);
            ps.setString(2, pass);
            ResultSet rs = ps.executeQuery();
            if (rs.next())
                valid = true;
            rs.close();
            ps.close();
        } catch (SQLException sqle) {
            System.err.println("Could not validate user.");
            sqle.printStackTrace();
        }
        return valid;
    }


    public int getUserId(String name) {
        int id = -1;
        try {
            String sql = "SELECT id FROM user WHERE name = ?";
            PreparedStatement ps = c.prepareStatement(sql);
            ps.setString(1, name);
            ResultSet rs = ps.executeQuery();
            if (rs.next())
                id = rs.getInt("id");
            rs.close();
            ps.close();
        } catch (SQLException sqle) {
            System.err.println("Could not get user id.");
            sqle.printStackTrace();
        }
        return id;
    }


    public String getUserName(int id) {
        String name = null;
        try {
            String sql = "SELECT name FROM user WHERE id = ?";
            PreparedStatement ps = c.prepareStatement(sql);
            ps.setInt(1, id);
            ResultSet rs = ps.executeQuery();
            if (rs.next()) {
                name = rs.getString("name");
            } else
                System.err.println("No username found for user with id " + id);
            rs.close();
            ps.close();
        } catch (SQLException sqle) {
            System.err.println("Could not get user name.");
            sqle.printStackTrace();
        }
        return name;
    }

    
    public int getBalance(int userId, String account) {
        int bal = -1;
        try {
            String sql = "SELECT balance FROM account " +
            		 "WHERE userId = ? AND account = ?";
            PreparedStatement ps = c.prepareStatement(sql);
            ps.setInt(1, userId);
            ps.setString(2, account);
            ResultSet rs = ps.executeQuery();
            if (rs.next())
                bal = rs.getInt("balance");
            rs.close();
            ps.close();
        } catch (SQLException sqle) {
            System.err.println("Could not get balance.");
            sqle.printStackTrace();
        }
        return bal;
    }
    

    public String transfer(int userId, String src, String dst, int ammount) {
        
        if (getBalance(userId, src) < ammount)
            return "Insufficient funds.";
        
        try {
            String sql =
                "UPDATE account SET balance = balance + ? WHERE account = ?";
            PreparedStatement ps = c.prepareStatement(sql);
            ps.setInt(1, -ammount);
            ps.setString(2, src);
            ps.executeUpdate();
            
            ps.setInt(1, ammount);
            ps.setString(2, dst);
            ps.executeUpdate();
            
            ps.close();
        } catch (SQLException sqle) {
            System.err.println("Could not transfer.");
            sqle.printStackTrace();
            return "Error during transfer";
        }
        
        return String.format("Successfully transfered %s from %s to %s.",
                Util.formatCurrency(ammount), src, dst);
    }
  
    
    public List<String> getAccountsFor(int userId) {
        List<String> accs = new ArrayList<String>();
        try {
            String sql = "SELECT account FROM account WHERE userId = ?";
            PreparedStatement ps = c.prepareStatement(sql);
            ps.setInt(1, userId);
            ResultSet rs = ps.executeQuery();
            while (rs.next())
                accs.add(rs.getString("account"));
            rs.close();
            ps.close();
        } catch (SQLException sqle) {
            System.err.println("Could not get balance.");
            sqle.printStackTrace();
        }
        Collections.sort(accs);
        return accs;
    }

  
    
    public Set<Integer> getAccountHolderIds() {
        Set<Integer> ids = new HashSet<Integer>();
        try {
            String sql = "SELECT id FROM user";
            PreparedStatement ps = c.prepareStatement(sql);
            ResultSet rs = ps.executeQuery();
            while (rs.next())
                ids.add(rs.getInt("id"));
            rs.close();
            ps.close();
        } catch (SQLException sqle) {
            System.err.println("Could not get balance.");
            sqle.printStackTrace();
        }
        return ids;
    }
    
    
    public int getBalance(int userId) {
        int totalBalance = 0;
        for (String account : getAccountsFor(userId))
            totalBalance += getBalance(userId, account);
        return totalBalance;
    }
    
    
    public String addUser(String newname, String newpass) {
        
        newname = newname.trim();
        newpass = newpass.trim();
        
        if (newname.isEmpty() || newpass.isEmpty())
            return "Name and/or pass empty!";
        
        if (getUserId(newname) != -1)
            return "Name already in use.";
        
        int freshId = getFreshUserId();
        if (freshId == -1)
            return "Could not generate fresh user id.";
        
        try {
            addUser(freshId, newname, newpass);
            addAccount(freshId, "A new account", 0);
        } catch (SQLException e) {
            e.printStackTrace();
        }
        
        return "Successfully added " + newname + ".";
    }


    public String removeUser(int userId) {
        
        String username = getUserName(userId);
        
        if (username == null)
            return "No such user.";
        
        try {
            String sql = "DELETE FROM account WHERE userId = ?";
            PreparedStatement ps = c.prepareStatement(sql);
            ps.setInt(1, userId);
            ps.executeUpdate();
            
            sql = "DELETE FROM user WHERE id = ?";
            ps = c.prepareStatement(sql);
            ps.setInt(1, userId);
            ps.executeUpdate();
            
            ps.close();
        } catch (SQLException sqle) {
            System.err.println("Could remove user.");
            sqle.printStackTrace();
            return "Could not remove user.";
        }
        
        return String.format("Successfully removed %s.", username);
    }
    
    
}
