/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.latex;

import jpicedt.format.input.util.*;
import jpicedt.graphic.model.*;
import jpicedt.graphic.PicPoint;

/**
 * PicLine :<ul>
 * <li>  %Line 0 0 (x0,y0)(x1,y1) dash=value
 * <li>  %Line 0 1 (x0,y0)(x1,y1) dash=value
 * <li>  %Line 1 0 (x0,y0)(x1,y1) dash=value
 * <li>  %Line 1 1 (x0,y0)(x1,y1) dash=value (dash is optional)
 * <li>  Any string (\multiput, etc...)
 * <li>  %End Line
 * </ul>
 * @author Sylvain Reynal
 * @since jpicedt 1.3
 * @version $Id: PicLineExpression.java,v 1.7.2.1 2007/09/02 11:56:08 reynal Exp $
 *
 */
public class PicLineExpression extends SequenceExpression  implements ExpressionConstants {

	private EditPointConstraint constraint = PicMultiCurve.SMOOTHNESS;
	
    public PicLineExpression(Pool pool){
	    
		super(true); // throw IncompleteSequence Exception
		add(new LaTeXInstanciationExpression("%Line", new PicMultiCurve(new PicPoint(),new PicPoint()),pool));
		add(WHITE_SPACES);
		add(new PicArrowTypeExpression(pool));
		add(new LaTeXPicPointExpression(0,constraint,pool)); // no constraints
		add(WHITE_SPACES);
		add(new LaTeXPicPointExpression(3,constraint,pool));
		add(WHITE_SPACES);
		add(new OptionalExpression(new PicDashStatement(pool)));
		add(new PicEndExpression("%End Line"));
    }
	
	public String toString(){
		return "[PicLineExpression]";
	}

	
} // PicLineExpression
