/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.latex;

import jpicedt.format.input.util.*;
import jpicedt.graphic.model.*;

/**
 * %Rectangle(x0,y0)(x1,y1) <br> 
 * or %Rectangle(x0,y0)(x1,y1) filled|blacken|shade|whiten dash=value <br>
 * Any string <br>
 * %End Rectangle <br>
 * @author Sylvain Reynal
 * @since jpicedt 1.3
 * @version $Id: PicRectangleExpression.java,v 1.5.2.1 2007/09/02 11:56:08 reynal Exp $
 *
 */
public class PicRectangleExpression extends SequenceExpression  implements ExpressionConstants {

    public PicRectangleExpression(Pool pool){

		super(true); // throw IncompleteSequence Exception
		add(new LaTeXInstanciationExpression("%Rectangle", new PicParallelogram(),pool));
		add(WHITE_SPACES);
		add(new LaTeXPicPointExpression(PicParallelogram.P_BL,pool));  
		add(WHITE_SPACES);
		add(new LaTeXPicPointExpression(PicParallelogram.P_TR,pool)); // no need to set BR since b2tVec and l2rVec are orthogonal
		add(WHITE_SPACES);
		add(new OptionalExpression(new AlternateExpression(new PicDashStatement(pool), new PicColorExpression(pool))));
		add(WHITE_SPACES);
		add(new OptionalExpression(new AlternateExpression(new PicDashStatement(pool), new PicColorExpression(pool))));
		add(new PicEndExpression("%End Rectangle"));
    }
	
	public String toString(){
		return "[PicRectangleExpression]";
	}
	
} // PicRectangleExpression
 
