/*
 *  jPicEdt, a picture editor for LaTeX.
 *  Copyright (C) 1999-2007  Sylvain Reynal
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Sylvain Reynal
 *  Département de Physique
 *  Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 *  6, avenue du Ponceau
 *  95014 CERGY CEDEX
 *  FRANCE
 *  Tel : 00 +33 130 736 245
 *  Fax : 00 +33 130 736 667
 *  e-mail : reynal@ensea.fr
 *  jPicEdt web page : http://www.jpicedt.org
 */
package jpicedt.format.input.pstricks;

import jpicedt.format.output.pstricks.PstricksUtilities;
import jpicedt.format.input.util.*;
import jpicedt.graphic.model.*;

/**
 * Parses Pstricks's arrows, as given by the PsTricks's documentation (p.29).<br>
 * <br>
 * This is based on the following rules : 
 * <ul>
 * <li> an opening "{" ;
 * <li> "<", ">", "(",... or even "", to set the first arrow ; 
 * <li> a "-" in the middle ;
 * <li> same thing on the right side to set the second arrow ;
 * <li> a closing "}".
 * </ul>
 * If this expression is successfully parsed, the attribute set associated with the Pool's currentObj is modified
 * accordingly.
 *
 * @author Sylvain Reynal
 * @version $Id: PSTArrowExpression.java,v 1.4.2.1 2007/09/02 11:56:11 reynal Exp $ 
 */
public class PSTArrowExpression extends SequenceExpression implements PicObjectConstants {

	private Pool pool;

	/**
	 * Constructor for the PSTArrowExpression object
	 */
	public PSTArrowExpression(Pool pl) {

		super(true);// throw IncompleteSequence Exception
		pool = pl;

		add(new LiteralExpression("{"));
		add(
			new WordExpression("}", true) {// postfix = closing "}", swallowed
				public void action(ParserEvent e) {
					if (DEBUG) System.out.println(e);
					String s = (String)(e.getValue());
					Arrow[] arrows = PstricksUtilities.createArrowsFromPstricksString(s);
					pool.currentObj.setAttribute(LEFT_ARROW, arrows[0]);
					pool.currentObj.setAttribute(RIGHT_ARROW, arrows[1]);
				}
			});
	}
}

