/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.pstricks;

import jpicedt.format.input.util.*;
import jpicedt.format.output.pstricks.PstricksUtilities;
import jpicedt.graphic.model.*;

import java.awt.*;
import java.util.*;

/**
 * Used by PSTParametersExpression to parse statements involving colours, for instance
 *  "linecolor=green" (predefined colour) or "fillcolor=MyGray" (user-defined colours).
 * <p>
 * If no pstricks's native colour matches the RHS, we fetch a HashMap from the Pool's hashtable 
 * (value associated with key PstricksParser.KEY_USER_COLOURS) and look up the given colour ; 
 * this HashMap may've been filled by UserDefinedColorExpression in the course of the parsing process.
 * @author Sylvain Reynal
 * @version $Id: PSTColorExpression.java,v 1.4.2.1 2007/09/02 11:56:11 reynal Exp $ 
 *
 */
public class PSTColorExpression extends SequenceExpression {

    private Pool pool;
    private PicAttributeName attribute;
	private Pool.Key setKey;

    /**
     * @param pl parser's pool
     * @param tag LHS tag (e.g. "linecolor" or "fillcolor") for the StatementExpression
     * @param attribute name of attribute to modify (must be a predefined PicAttributeName of type "Color")
	 * @param attributeSetKey used to fetch the attribute set in which parsed parameters are stored. 
     */
    public PSTColorExpression(Pool pl, String tag, PicAttributeName attributeName, Pool.Key attributeSetKey){

		super(false); // doesn't throw IncompleteSequenceException (which means that there might be nothing b/w [ and ])

		pool = pl;
		attribute = attributeName;
		this.setKey = attributeSetKey;

		add(new LiteralExpression(tag+"=")); // e.g. "linecolor"
		add(new ParseColorValue());
    }
	
	private class ParseColorValue extends WordExpression {
		public ParseColorValue(){
			// no postfix, swallows as many letters-or-digit as possible (since user-defined colours may contain digits)
			super(null,false,true,true);
		}
		public void action(ParserEvent e){
			if (DEBUG) System.out.println(e); 
			String colourStr = (String)(e.getValue());
			//System.out.println("!!!!!!!!!!!!!!!!!!!! FOUND COLOR : " + colourStr);
			Color colour = PstricksUtilities.getPsTricksColor(colourStr);
			if (colour == null) { // if not a native colour, look up hashtable
				HashMap userColoursMap = (HashMap)pool.get(PstricksParser.KEY_USER_COLOURS);
				colour = (Color)(userColoursMap.get(colourStr));
				if (colour == null) // if nor a native colour, nor a user-defined colour, default to black...
					colour = Color.black;
			}
			pool.setAttribute(setKey,attribute, colour);
		}
	}
} // class

