/*
 PSTDotStyleExpression.java - September 3, 2003 - jPicEdt, a picture editor for LaTeX.
 Copyright (C)  Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
package jpicedt.format.input.pstricks;

import jpicedt.format.input.util.*;
import jpicedt.graphic.io.formatter.*;
import jpicedt.graphic.model.*;
import java.awt.*;

/**
 * Parses statements similar to "dotstyle=style" where style is one of PicObjectConstants predefined dot styles.
 * @author Sylvain Reynal
 * @since jpicedt 1.4
 * @version $Id: PSTDotStyleExpression.java,v 1.3.2.1 2007/09/02 11:56:12 reynal Exp $
 * <p>
 *
 */
public class PSTDotStyleExpression extends SequenceExpression implements PicObjectConstants {

	private Pool pool;
	private Pool.Key attributeSetKey;


	/**
	 * @param pl parser's pool
	 * @param attributeSetKey used to fetch the attribute set in which parsed parameters are stored. 
	 */
	public PSTDotStyleExpression(Pool pl, Pool.Key attributeSetKey) {

		super(false);// doesn't throw IncompleteSequenceException

		pool = pl;
		this.attributeSetKey = attributeSetKey;
	}
	
	public boolean interpret(Context context) throws REParserException { 
		
		if (!context.matchAndMove("dotstyle=")) return false;
		WHITE_SPACES_OR_EOL.interpret(context);
		if (context.matchAndMove("none")) pool.setAttribute(attributeSetKey, POLYDOTS_STYLE,NONE); // not really allowed, though...
		else if (context.matchAndMove("*")) pool.setAttribute(attributeSetKey, POLYDOTS_STYLE,POLYDOTS_DISK);
		else if (context.matchAndMove("o")) pool.setAttribute(attributeSetKey, POLYDOTS_STYLE,POLYDOTS_CIRCLE);
		else if (context.matchAndMove("+")) pool.setAttribute(attributeSetKey, POLYDOTS_STYLE,POLYDOTS_PLUS);
		else if (context.matchAndMove("triangle*")) pool.setAttribute(attributeSetKey, POLYDOTS_STYLE,POLYDOTS_TRIANGLE_FILLED);
		else if (context.matchAndMove("triangle")) pool.setAttribute(attributeSetKey, POLYDOTS_STYLE,POLYDOTS_TRIANGLE);
		else if (context.matchAndMove("square*")) pool.setAttribute(attributeSetKey, POLYDOTS_STYLE,POLYDOTS_SQUARE_FILLED);
		else if (context.matchAndMove("square")) pool.setAttribute(attributeSetKey, POLYDOTS_STYLE,POLYDOTS_SQUARE);
		else if (context.matchAndMove("pentagon*")) pool.setAttribute(attributeSetKey, POLYDOTS_STYLE,POLYDOTS_PENTAGON_FILLED);
		else if (context.matchAndMove("pentagon")) pool.setAttribute(attributeSetKey, POLYDOTS_STYLE,POLYDOTS_PENTAGON);
		else if (context.matchAndMove("|")) pool.setAttribute(attributeSetKey, POLYDOTS_STYLE,POLYDOTS_DISK); // not supported yet
		else throw new REParserException.SyntaxError(context,this);
		return true;
	}
}

