/*
 PSTInstanciationExpression.java - March 29, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C)  Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.format.input.pstricks;

import jpicedt.graphic.model.*;
import jpicedt.format.input.util.*;

import java.util.*;
import java.io.*;

/**
 * Instanciates a new Element by cloning the given object, when a given tag gets found.
 * Then adds it to the current PicGroup in the Pool (i.e. <code>Pool.currentGroup</code>)
 * @author Sylvain Reynal
 * @version $Id: PSTInstanciationExpression.java,v 1.5.2.1 2007/09/02 11:56:12 reynal Exp $ 
 */
public class PSTInstanciationExpression extends LiteralExpression  implements ExpressionConstants {

	private Element prototype;
	private Pool pool;
	private String tag;
	private PSTParametersExpression paramExp;

	/**
	 * @param tag expression to be matched
	 * @param prototype element to be instanciated by cloning
	 * @param pl pool where to add the instanciated element ; also used to fetch the current attribute set for PsTricks
	 */
	public PSTInstanciationExpression(String tag, Element prototype,Pool pl){

		super(tag);
		this.tag = tag;
		this.prototype = prototype;
		this.pool = pl;
		paramExp = new PSTParametersExpression(this.pool, Pool.CURRENT_OBJ_ATTRIBUTES);
	}

	public void action(ParserEvent e) throws REParserException {

		if (DEBUG) System.out.println(e);
		pool.currentObj = (Element)prototype.clone();
		pool.currentObj.getAttributeSet().setAttributes(pool.getAttributeSet(PstricksParser.KEY_ATTRIBUTES)); // overrides non-default values only
		pool.currentGroup.addChild(pool.currentObj);
		// possibly retrieve a parameter string associated with the tag, if it's been previously set by 
		// PsObjectExpression :
		HashMap map = (HashMap)pool.get(PstricksParser.KEY_NEWPSOBJECTS); 
		String paramStr = (String)map.get(tag);
		if (paramStr != null){ // ok, there exists such a tag, previously set by a \\newpsobject command
			try {
				Context tmpContext = new Context(new StringReader(paramStr));
				paramExp.interpret(tmpContext); // set attributes for currentObj
			} catch (REParserException.EOF ex){}
		}
		
	}

	public String toString(){

		return "[PSTInstanciationExpression:tag="+tag+" prototype="+prototype.getClass().getName()+"]";
	}
}

