/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.pstricks;

import jpicedt.format.input.util.*;
import jpicedt.graphic.model.*;
import jpicedt.graphic.PicPoint;

/**
 * <ul>
 * <li>\\psframe[parameters](x0,y0)(x1,y1)
 * <li>\\psframe*[parameters](x0,y0)(x1,y1)
 * </ul>
 * @author Sylvain Reynal
 * @version $Id: PsFrameExpression.java,v 1.5.2.1 2007/09/02 11:56:12 reynal Exp $ 
 */
public class PsFrameExpression extends SequenceExpression implements PicObjectConstants {

	private Pool pool;

	/**
	 * Uses default tag.
	 */
	public PsFrameExpression(Pool pl){
		this(pl,null);
	}
	/**
	 * @param tag if null, default to "\\psframe"
	 */
	public PsFrameExpression(Pool pl, String tag){

		super(true); // throw IncompleteSequence Exception
		pool = pl;
		if (tag == null) tag = "\\psframe";
		add(new PSTInstanciationExpression(tag, new PicParallelogram(),pool));
		add(WHITE_SPACES_OR_EOL);
		add(new OptionalExpression(new LiteralExpression("*"){
			                           public void action(ParserEvent e){
				                           if (DEBUG) System.out.println(e);
				                           pool.currentObj.setAttribute(FILL_STYLE,SOLID);}}));
		add(WHITE_SPACES_OR_EOL);
		// add optional param here
		add(new OptionalExpression(new EnclosingExpression("[",new PSTParametersExpression(pool,Pool.CURRENT_OBJ_ATTRIBUTES),"]"))); // push in object's attributeSet
		add(WHITE_SPACES_OR_EOL);
		add(new PSTPicPointExpression(PicParallelogram.P_BL,pool));
		add(WHITE_SPACES_OR_EOL);
		add(new PSTPicPointExpression(PicParallelogram.P_TR,pool));
	}
	public String toString(){
		return "[PsFrameExpression]";
	}
}
