/*  AlternateExpression.java - jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.util;

import java.util.*;

/**
 * A regular expression that mimics the "x|y" RegExp syntax.
 * @since jpicedt 1.3
 * @author Sylvain Reynal
 * @version $Id: AlternateExpression.java,v 1.4.2.1 2007/09/02 11:56:13 reynal Exp $
 *
 */
public class AlternateExpression extends AbstractRegularExpression {

	private ArrayList expressionList=new ArrayList();;

	/**
	 * Creates a new AlternateExpression with the given expr as first element
	 */
	public AlternateExpression(AbstractRegularExpression expr){
		expressionList.add(expr);
	}

	/**
	 * Creates a new AlternateExpression with the given expr as first and second element
	 */
	public AlternateExpression(AbstractRegularExpression expr1,AbstractRegularExpression expr2){
		expressionList.add(expr1);
		expressionList.add(expr2);
	}

	/**
	 * Creates a new AlternateExpression with the given expr as first, second and third element
	 */
	public AlternateExpression(AbstractRegularExpression expr1,AbstractRegularExpression expr2,AbstractRegularExpression expr3){
		expressionList.add(expr1);
		expressionList.add(expr2);
		expressionList.add(expr3);
	}

	/**
	 * Creates a new AlternateExpression with no element
	 */
	public AlternateExpression(){}

	/**
	 * Add a new child to this AlternateExpression.
	 */
	public void add(AbstractRegularExpression expr){expressionList.add(expr);}

	/**
	 * Calls "action" with value=Integer(index of first expression parsed with success, 
	 * starting from 0).
	 * @return TRUE as soon as one of the expressions contained in this AlternateExpression has been found ; 
	 * FALSE if no match.<p>
	 */
	public boolean interpret(Context c) throws REParserException {

		int j=0;
		for(Iterator i=expressionList.iterator(); i.hasNext();){
			AbstractRegularExpression are = (AbstractRegularExpression)i.next();
			if (are.interpret(c) == true) {
				action(new ParserEvent(this, c, true, new Integer(j)));
				return true;
			}
			j++;
		}
		//action(new ParserEvent(this, c, false, null));
		return false;
	}

	/**
	 * Return a list containing every expression in this AlternateExpression
	 */
	public ArrayList getExpressionList(){
		return expressionList;
	}
	
	/**
	 *
	 */
	public String toString(){

		String s = "[AlternateExpression:";
		int j=1;
		for(Iterator i=expressionList.iterator(); i.hasNext();){
			StringBuffer subExpStr = new StringBuffer(((AbstractRegularExpression)i.next()).toString());
			int pos=0;
			while(true){
				pos = subExpStr.toString().indexOf('\n',pos);
				if (pos == -1) break;
				subExpStr.insert(pos+1,'\t');
				pos += 2;
			}
			s += "\n\t\t|" + new Integer(j) + subExpStr + " ";
			j++;
		}
		return s + "]";

	}
}
