/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.format.input.util;

import jpicedt.graphic.model.*;

/**
 * An expression that can instanciate a new Element by cloning the given graphic element when it finds a given
 * litteral tag, then add it to the current PicGroup in the pool.
 * @since jpicedt 1.3
 * @author Sylvain Reynal
 * @version $Id: InstanciationExpression.java,v 1.4.2.1 2007/09/02 11:56:13 reynal Exp $
 *
 */
public class InstanciationExpression extends LiteralExpression  implements ExpressionConstants {

	private Element prototype;
	private Pool pool;
	private String tag;
	private PicAttributeSet attributes; 

	/**
	 * @param tag string to be matched
	 * @param prototype element to be cloned, then added to the current PicGroup
	 * @param attributes attribute set used to set the attributes for the newly cloned element ; 
	 *        can be null, in which case the attribute set of the prototype is used (yet it's static in this case)
	 * @param pl pool used to share variable across the grammar tree.
	 */
	public InstanciationExpression(String tag, Element prototype, PicAttributeSet attributes, Pool pl){

		super(tag);
		this.tag = tag;
		this.prototype = prototype;
		this.pool = pl;
		this.attributes = attributes;
	}

	/**
	 * Called when the tag-string was matched ; instanciate a new element, then add it to the current PicGroup.
	 */
	public void action(ParserEvent e){

		if (DEBUG) System.out.println(e);
		pool.currentObj = (Element)prototype.clone();
		if (attributes != null) pool.currentObj.setAttributeSet(attributes);
		pool.currentGroup.addChild(pool.currentObj);
	}

	public String toString(){

		return "[InstanciationExpression:tag="+tag+" prototype="+prototype.getClass().getName()+"]";
	}
}

