/*
 RegExExpression.java - July 31, 2003 - jPicEdt 1.3.3, a picture editor for LaTeX.
 Copyright (C)  Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.format.input.util;

import java.util.*;
import java.util.regex.*;

/** 
 * An expression specified by a java.util.regex.Pattern regular expression. The search ranges from the current
 * caret position in Context, to the end of the current block (exclusive).
 * @since jpicedt 1.3
 * @author Sylvain Reynal
 * @version $Id: RegExExpression.java,v 1.5.2.1 2007/09/02 11:56:14 reynal Exp $
 *
 */
public class RegExExpression extends AbstractRegularExpression {

    private Pattern pattern;

    /**
     * @param rePatter The pattern used to build the GNU RegExp to be matched
     */
    public RegExExpression(String rePattern){
		try {
			this.pattern = Pattern.compile(rePattern);
		}
		catch (PatternSyntaxException e){e.printStackTrace();}
    }

    /**
     * @return TRUE if re.getMatch returned non-null
     * Calls "action" with value=re.getMatch.
     */
    public boolean interpret(Context context) throws REParserException {

		boolean matched = context.lookingAt(pattern); // move caret past matched string if found
		if (matched){
			action(new ParserEvent(this, context, true,  pattern));
			return true;
		}
		return false;
    }

    /**
     *
     */
    public String toString(){
		return "[RegExExpression:\"" + pattern + "\"]";
    }
}
