/*
 EepicCustomizer.java - March 4, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

 package jpicedt.format.output.eepic;

import jpicedt.Localizer;
import jpicedt.graphic.toolkit.AbstractCustomizer;
import jpicedt.graphic.PEToolKit;
import jpicedt.widgets.DecimalNumberField;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;

/**
 * a panel for Epic/Eepic preferences editing
 * @author Sylvain Reynal
 * @since PicEdt 1.3
 */
public class EepicCustomizer extends AbstractCustomizer implements EepicConstants {

	private DecimalNumberField thinLinesTF, thickLinesTF;
	private JTextArea prologTA, epilogTA;
	private Properties preferences;

	/**
	 * construct a new panel for epic/eepic preferences editing.
	 * @param preferences Properties used to init the widgets fields, and to store the values
	 * when "storePreferences" is called.
	 */
	public EepicCustomizer(Properties preferences){

		this.preferences = preferences;
		
		// init main box
		Box box = new Box(BoxLayout.Y_AXIS);
		// temp. buffers
		JPanel p;
		JLabel l;

		/* Sub-Panel 1 :generic parameters for Eepic */
		p = new JPanel(new GridLayout(2,2,5,5));
		p.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
		            Localizer.currentLocalizer().get("format.eepic.Parameters")));


		l = new JLabel(Localizer.currentLocalizer().get("format.eepic.thin-lines-max-width"));
		l.setToolTipText(Localizer.currentLocalizer().get("format.eepic.thin-lines-max-width.tooltip"));
		p.add(l);
		thinLinesTF = new DecimalNumberField(0,10,true);
		p.add(thinLinesTF);

		l = new JLabel(Localizer.currentLocalizer().get("format.eepic.thick-lines-max-width"));
		l.setToolTipText(Localizer.currentLocalizer().get("format.eepic.thick-lines-max-width.tooltip"));
		p.add(l);
		thickLinesTF = new DecimalNumberField(0,10,true);
		p.add(thickLinesTF);

		box.add(p);

		/* Sub-Panel 2 : prolog and epilog */

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		c.gridwidth = GridBagConstraints.REMAINDER; 
		
		p = new JPanel(gbl);
		p.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
		            Localizer.currentLocalizer().get("format.PrologEpilog")));

		l = new JLabel(" "+Localizer.currentLocalizer().get("format.Prolog"));
		l.setToolTipText(Localizer.currentLocalizer().get("format.Prolog.tooltip"));
		gbl.setConstraints(l,c);
		p.add(l);
		prologTA = new JTextArea(5,50);
		gbl.setConstraints(prologTA,c);
		p.add(prologTA);

		l = new JLabel(" "+Localizer.currentLocalizer().get("format.Epilog"));
		l.setToolTipText(Localizer.currentLocalizer().get("format.Epilog.tooltip"));
		gbl.setConstraints(l,c);
		p.add(l);
		epilogTA = new JTextArea(5,50);
		gbl.setConstraints(epilogTA,c);
		p.add(epilogTA);

		box.add(p);

		// completed :
		add(box, BorderLayout.NORTH);
	}

	/**
	 * @return the panel title, used e.g. for Border or Tabpane title. 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public String getTitle(){ 
		return "Epic/eepic";	
	}


	/**
	 * @return the Icon associated with this panel, used e.g. for TabbedPane decoration 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public Icon getIcon(){ 
		return null;	
	}


	/**
	 * @return the tooltip string associated with this panel 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public String getTooltip(){ 
		return Localizer.currentLocalizer().get("format.eepic.tooltip");	
	}

	/**
	 * Load widgets display content with a default value retrieved from the EepicConstants interface.
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public void loadDefault() {
		thinLinesTF.setValue(DEFAULT_THIN_LINES_MAX_WIDTH);
		thickLinesTF.setValue(DEFAULT_THICK_LINES_MAX_WIDTH);
		prologTA.setText(DEFAULT_FILE_WRAPPER_PROLOG);
		epilogTA.setText(DEFAULT_FILE_WRAPPER_EPILOG);
	}

	/**
	 * Load widgets value from the Properties object given in the constructor.
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public void load() {

		// from EepicConstants :
		thinLinesTF.setText(preferences.getProperty(KEY_THIN_LINES_MAXWIDTH,PEToolKit.doubleToString(DEFAULT_THIN_LINES_MAX_WIDTH)));
		thickLinesTF.setText(preferences.getProperty(KEY_THICK_LINES_MAXWIDTH,PEToolKit.doubleToString(DEFAULT_THICK_LINES_MAX_WIDTH)));
		prologTA.setText(preferences.getProperty(KEY_FILE_WRAPPER_PROLOG,DEFAULT_FILE_WRAPPER_PROLOG));
		epilogTA.setText(preferences.getProperty(KEY_FILE_WRAPPER_EPILOG,DEFAULT_FILE_WRAPPER_EPILOG));
	}	

	/**
	 * Store current widgets value to the Properties object given in the constructor,
	 * then update LatexFormatter accordingly.
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	 public void store(){
		preferences.setProperty(KEY_THIN_LINES_MAXWIDTH, thinLinesTF.getText());
		preferences.setProperty(KEY_THICK_LINES_MAXWIDTH, thickLinesTF.getText());
		preferences.setProperty(KEY_FILE_WRAPPER_PROLOG,prologTA.getText());
		preferences.setProperty(KEY_FILE_WRAPPER_EPILOG,epilogTA.getText());
		EepicFormatter.configure(preferences); // update
	}
} // PanelLatex
