/*
 LatexPanel.java - December 31, 2001 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://trashx.ensea.fr/jpicedt/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.format.output.latex;

import jpicedt.Localizer;
import jpicedt.graphic.toolkit.AbstractCustomizer;
import jpicedt.graphic.PEToolKit;
import jpicedt.widgets.DecimalNumberField;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;

/**
 * a panel for LaTeX preferences editing (emulation parameters,...)
 * @author Sylvain Reynal
 * @since PicEdt 1.3
 * @version $Id: LatexCustomizer.java,v 1.6.2.1 2007/09/02 11:55:13 reynal Exp $
 *
 */
public class LatexCustomizer extends AbstractCustomizer implements LatexConstants {

	private DecimalNumberField linethicknessTF, emLineLengthTF, maxEmLineSlopeTF, maxLatexDiskDiameterTF;
	private DecimalNumberField emCircleSegLengthTF, maxLatexCircleDiameterTF;
	private JTextArea prologTA, epilogTA;
	private Properties preferences;

	/**
	 * construct a new panel for LaTeX preferences editing.
	 * @param preferences Properties used to init the widgets fields, and to store the values
	 * when "storePreferences" is called.
	 */
	public LatexCustomizer(Properties preferences){

		this.preferences = preferences;
		
		// init main box
		Box box = new Box(BoxLayout.Y_AXIS);
		// temp. buffers
		JPanel p;
		JLabel l;

		/* Sub-Panel 1 :generic parameters for LaTeX */
		p = new JPanel(new GridLayout(2,2,5,5));
		p.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
		            Localizer.currentLocalizer().get("format.latex.Parameters")));


		l = new JLabel(" "+Localizer.currentLocalizer().get("format.latex.MaxCircleDiameter.label")+"  (mm) :");
		l.setToolTipText(Localizer.currentLocalizer().get("format.latex.MaxCircleDiameter.tooltip"));
		p.add(l);
		maxLatexCircleDiameterTF = new DecimalNumberField(0,10,true);
		p.add(maxLatexCircleDiameterTF);

		l = new JLabel(" "+Localizer.currentLocalizer().get("format.latex.MaxDiskDiameter.label")+"  (mm) :");
		l.setToolTipText(Localizer.currentLocalizer().get("format.latex.MaxDiskDiameter.tooltip"));
		p.add(l);
		maxLatexDiskDiameterTF = new DecimalNumberField(0,10,true);
		p.add(maxLatexDiskDiameterTF);

		box.add(p);

		/* Sub-Panel 2 : emulated LaTeX configuration */

		p = new JPanel(new GridLayout(3,2,5,5));
		p.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
		            Localizer.currentLocalizer().get("format.latex.Emulation")));

		l = new JLabel(" "+Localizer.currentLocalizer().get("format.latex.EmLineLength.label")+"  (mm) :");
		l.setToolTipText(Localizer.currentLocalizer().get("format.latex.EmLineLength.tooltip"));
		p.add(l);
		emLineLengthTF = new DecimalNumberField(0,10,true);
		p.add(emLineLengthTF);

		l = new JLabel(" "+Localizer.currentLocalizer().get("format.latex.EmMaxLineSlope.label"));
		l.setToolTipText(Localizer.currentLocalizer().get("format.latex.EmMaxLineSlope.tooltip"));
		p.add(l);
		maxEmLineSlopeTF = new DecimalNumberField(0,10,true);
		p.add(maxEmLineSlopeTF);

		l  = new JLabel(" "+Localizer.currentLocalizer().get("format.latex.EmCircleSegLength.label")+"  (mm) :");
		l.setToolTipText(Localizer.currentLocalizer().get("format.latex.EmCircleSegLength.tooltip"));
		p.add(l);
		emCircleSegLengthTF = new DecimalNumberField(0,10,true);
		p.add(emCircleSegLengthTF);

		box.add(p);

		/* Sub-Panel 3 : prolog and epilog */

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		c.gridwidth = GridBagConstraints.REMAINDER; 
		
		p = new JPanel(gbl);
		p.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
		            Localizer.currentLocalizer().get("format.PrologEpilog")));

		l = new JLabel(" "+Localizer.currentLocalizer().get("format.Prolog"));
		l.setToolTipText(Localizer.currentLocalizer().get("format.Prolog.tooltip"));
		gbl.setConstraints(l,c);
		p.add(l);
		prologTA = new JTextArea(5,50);
		gbl.setConstraints(prologTA,c);
		p.add(prologTA);

		l = new JLabel(" "+Localizer.currentLocalizer().get("format.Epilog"));
		l.setToolTipText(Localizer.currentLocalizer().get("format.Epilog.tooltip"));
		gbl.setConstraints(l,c);
		p.add(l);
		epilogTA = new JTextArea(5,50);
		gbl.setConstraints(epilogTA,c);
		p.add(epilogTA);

		box.add(p);
		
		// completed :
		add(box, BorderLayout.NORTH);
	}

	/**
	 * @return the panel title, used e.g. for Border or Tabpane title. 
	 */
	public String getTitle(){ 
		return "LaTeX";	
	}


	/**
	 * @return the Icon associated with this panel, used e.g. for TabbedPane decoration 
	 */
	public Icon getIcon(){ 
		return null;	
	}


	/**
	 * @return the tooltip string associated with this panel 
	 */
	public String getTooltip(){ 
		return Localizer.currentLocalizer().get("format.latex.tooltip");	
	}

	/**
	 * Load widgets display content with a default value retrieved from the LatexConstants interface.
	 */
	public void loadDefault() {
		// from LatexConstants :
		emLineLengthTF.setValue(DEFAULT_EM_LINE_LENGTH);
		maxEmLineSlopeTF.setValue(DEFAULT_MAX_EM_LINE_SLOPE);
		maxLatexCircleDiameterTF.setValue(DEFAULT_MAX_CIRCLE_DIAMETER);
		maxLatexDiskDiameterTF.setValue(DEFAULT_MAX_DISK_DIAMETER);
		emCircleSegLengthTF.setValue(DEFAULT_MAX_EM_CIRCLE_SEGMENT_LENGTH);
		prologTA.setText(DEFAULT_FILE_WRAPPER_PROLOG);
		epilogTA.setText(DEFAULT_FILE_WRAPPER_EPILOG);
	}

	/**
	 * Load widgets value from the Properties object given in the constructor.
	 */
	public void load() {

		// from LatexConstants :
		emLineLengthTF.setText(preferences.getProperty(KEY_EM_LINE_LENGTH,PEToolKit.doubleToString(DEFAULT_EM_LINE_LENGTH)));
		maxEmLineSlopeTF.setText(preferences.getProperty(KEY_MAX_EM_LINE_SLOPE,PEToolKit.doubleToString(DEFAULT_MAX_EM_LINE_SLOPE)));
		maxLatexCircleDiameterTF.setText(preferences.getProperty(KEY_MAX_CIRCLE_DIAMETER,PEToolKit.doubleToString(DEFAULT_MAX_CIRCLE_DIAMETER)));
		maxLatexDiskDiameterTF.setText(preferences.getProperty(KEY_MAX_DISK_DIAMETER,PEToolKit.doubleToString(DEFAULT_MAX_DISK_DIAMETER)));
		emCircleSegLengthTF.setText(preferences.getProperty(KEY_MAX_EM_CIRCLE_SEGMENT_LENGTH,PEToolKit.doubleToString(DEFAULT_MAX_EM_CIRCLE_SEGMENT_LENGTH)));
		prologTA.setText(preferences.getProperty(KEY_FILE_WRAPPER_PROLOG,DEFAULT_FILE_WRAPPER_PROLOG));
		epilogTA.setText(preferences.getProperty(KEY_FILE_WRAPPER_EPILOG,DEFAULT_FILE_WRAPPER_EPILOG));
	}	

	/**
	 * Store current widgets value to the Properties object given in the constructor,
	 * then update LatexFormatter accordingly.
	 */
	 public void store(){
		preferences.setProperty(KEY_EM_LINE_LENGTH, emLineLengthTF.getText());
		preferences.setProperty(KEY_MAX_CIRCLE_DIAMETER, maxLatexCircleDiameterTF.getText());
		preferences.setProperty(KEY_MAX_DISK_DIAMETER, maxLatexDiskDiameterTF.getText());
		preferences.setProperty(KEY_MAX_EM_CIRCLE_SEGMENT_LENGTH,emCircleSegLengthTF.getText());
		preferences.setProperty(KEY_MAX_EM_LINE_SLOPE,maxEmLineSlopeTF.getText());
		preferences.setProperty(KEY_FILE_WRAPPER_PROLOG,prologTA.getText());
		preferences.setProperty(KEY_FILE_WRAPPER_EPILOG,epilogTA.getText());
		LatexFormatter.configure(preferences); // update
	}
} // PanelLatex
