/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org/
*/

package jpicedt.format.output.latex;

import jpicedt.graphic.*;
import jpicedt.graphic.model.*;
import jpicedt.graphic.io.formatter.*;
import jpicedt.graphic.PicPoint;

import java.awt.*;

/**
 * An interface that specifies common formatting behaviours for Drawable objects
 */
public class PicTextFormatter implements Formatter, PicObjectConstants ,LatexConstants{

	/** the Element this formater acts upon */
	private PicText obj;
	private FormatterFactory factory;

	/**
	 *
	 */
	public PicTextFormatter(PicText obj, FormatterFactory factory){
		this.obj = obj;
		this.factory=factory;;
	}

	/**
	  * @return A string representation of a PicText object in the LaTeX format
	  * We don't need PicEdt special comments (e.g. %Ellipse... whith the whole bunch of parameters) 
	  * for PicText objects, since the formating produces a single \\makebox command, which can be easily parsed.
	  * \\makebox(0,0)[b]{hello} -> not framed
	  * \\framebox{$x=1$} -> framed
	  * There's no support for dashboxes any more, use a PicRectangle in conjonction with a makebox instead.
	  * @param The zero-point of the LaTeX picture in pixel-coordinates
	  *
	  * @since PicEdt 1.0
	  */
	public String format(){

		StringBuffer buf = new StringBuffer(100);

		// put command :
		buf.append("\\put");
		buf.append(obj.getPoint(PicText.P_ANCHOR,null));
		buf.append("{\\");
		if (!obj.isFramed()) { // \\makebox(0,0)[bl]{hello} -> not framed
			buf.append("makebox(0,0)");
			buf.append("[");

			if (obj.getVertAlign()== TEXT_VALIGN_TOP) buf.append("t");
			else if (obj.getVertAlign()== TEXT_VALIGN_BOTTOM) buf.append("b");
			else buf.append("c"); // middle or baseline
			
			if (obj.getHorAlign()==TEXT_HALIGN_LEFT) buf.append("l");
			else if (obj.getHorAlign()==TEXT_HALIGN_RIGHT) buf.append("r");
			else buf.append("c");

			buf.append("]");
		}
		else { // \\framebox{$x=1$} -> framed
			buf.append("framebox");
		}

		buf.append("{");
		buf.append(obj.getText());
		buf.append("}");
		buf.append("}");buf.append(CR_LF);buf.append(CR_LF);
		return buf.toString();
	}


} // class PicTextLatexFormatter
