/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org/
*/

package jpicedt.format.output.pstricks;

import jpicedt.graphic.io.formatter.*;
import jpicedt.graphic.*;
import jpicedt.graphic.model.*;

import java.awt.*;

/**
 * Pstricks formatter for PicParallelogram paraects.
 * @author Vincent Guirardel, Sylvain Reynal
 * @since jpicedt 1.4
 * @version $Id: PicParallelogramFormatter.java,v 1.5.2.1 2007/09/02 11:55:13 reynal Exp $ 
 */
public class PicParallelogramFormatter implements Formatter, PicObjectConstants, PstricksConstants {

	/** the Element this formatter acts upon */
	private PicParallelogram para;
	/** the producing factory */
	private PstricksFormatter factory;

	public PicParallelogramFormatter(PicParallelogram para, PstricksFormatter factory){
		this.para = para;
		this.factory=factory;
	}

	/**
	 * \\psframe[param](x1,y1)(x2,y2) => not used anymore [SR:pending]<br>
	 * \\pspolygon[param](x1,y1)...(x4,y4)
	 */
	public String format(){
		StringBuffer buf = new StringBuffer(100); // 100 as initial capacity seems to be a good guess
		// first handle possibly user-defined colours
		PstricksFormatter.ParameterString paramStr = factory.createParameterString(para);
		if (paramStr.isDefinedColourString()) buf.append(paramStr.getUserDefinedColourBuffer());

		// pspolygon
		buf.append("\\pspolygon");
		buf.append("[");
		buf.append(paramStr.getParameterBuffer());
		buf.append("]");
		buf.append(para.getPoint(PicParallelogram.P_BL,null));
		buf.append(para.getPoint(PicParallelogram.P_BR,null));
		buf.append(para.getPoint(PicParallelogram.P_TR,null));
		buf.append(para.getPoint(PicParallelogram.P_TL,null));
		buf.append(CR_LF);

		return buf.toString();
	}
	
	/*
		// psframe
		buf.append("\\psframe");
		buf.append("[");
		buf.append(paramStr.getParameterBuffer());
		buf.append("]");
		buf.append(para.getPoint(PicRectangle.P_BL,null));
		buf.append(para.getPoint(PicRectangle.P_TR,null));
	*/	


} 
