/*
 ContentType.java - February 16, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Departement de Physique
 Ecole Nationale Superieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.format.output.pstricks;

import jpicedt.graphic.toolkit.EditorKit;
import jpicedt.graphic.toolkit.AbstractCustomizer;
import jpicedt.graphic.io.formatter.FormatterFactory;
import jpicedt.graphic.ContentType;
import jpicedt.graphic.view.ViewFactory;

import java.util.*;

/**
 * ContentType for the Pstricks format.
 * @author Sylvain Reynal
 * @version $Id: PstricksContentType.java,v 1.8.2.1 2007/09/02 11:55:14 reynal Exp $ 
 */
public class PstricksContentType implements ContentType {

	protected String name, mime;
	
	/**
	 * @return the presentation name of this content-type
	 */
	public String getPresentationName(){
		return "PSTricks";
	}

	/**
	 * Creates a ViewFactory that's suited for this content-type
	 */
	public ViewFactory createViewFactory(){
		return new PstricksViewFactory();
	}
	

	/**
	 * creates a FormatterFactory that's suited for this content-type
	 * default implementation returns null.
	 */
	public FormatterFactory createFormatter(){
		return new PstricksFormatter();
	}
	
	/**
	 * Returns a customizer panel for this content-type
	 * @param props used to init the UI component or to store user-preferences on demand
	 */
	 public AbstractCustomizer createCustomizer(Properties props){
		 return new PstricksCustomizer(props);
	 }
	 
	/**
	 * Configure the EditorKit and the FormatterFactory returned by the factory methods,
	 * from the given Properties
	 */
	public void configure(Properties p){
		PstricksFormatter.configure(p);
	}

}
