/*
 ContentType.java - February 16, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic;

import jpicedt.graphic.toolkit.EditorKit;
import jpicedt.graphic.toolkit.AbstractCustomizer;
import jpicedt.graphic.io.formatter.JPICFormatter;
import jpicedt.graphic.io.formatter.FormatterFactory;
import jpicedt.graphic.view.DefaultViewFactory;
import jpicedt.graphic.view.ViewFactory;

import java.util.Properties;

/**
 * Default implementation of the ContentType interface suited for the JPIC-XML language.  
 * @since jpicedt 1.3.2
 * @author Sylvain Reynal
 * @version $Id: DefaultContentType.java,v 1.7.2.1 2007/09/02 11:56:15 reynal Exp $
 */
public class DefaultContentType implements ContentType {

	protected String name, mime;
	
	/**
	 * Return the presentation name of the JPIC-XML content-type
	 */
	public String getPresentationName(){
		return "JPIC-XML";
	}

	/**
	 * Creates a ViewFactory that's suited for this content-type
	 */
	public ViewFactory createViewFactory(){
		return new DefaultViewFactory();
	}
	
	/**
	 * creates a FormatterFactory suited for the JPIC-XML content-type
	 */
	public FormatterFactory createFormatter(){
		return new JPICFormatter();
	}

	/**
	 * Returns a customizer panel for this content-type. This implementation returns null.
	 */
	 public AbstractCustomizer createCustomizer(Properties prop){
		 return null;
	 }
	
	
	/**
	 * Configure the EditorKit and the FormatterFactory (as returned by factory methods),
	 * from the given Properties. 
	 */
	public void configure(Properties p){
	}
	
}
