/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.ensea.fr/pub/picedt.html
*/

package jpicedt.graphic.event;

import java.util.EventListener;

import jpicedt.graphic.model.Drawing;
import jpicedt.graphic.model.Element;

/**
 * Interface for notifications of changes in a Drawing.
 * [pending] for ATTRIBUTE_CHANGE events, we should've a subclass aka DrawingEvent.AttributeChangeEvent, that'd tell
 * us WHAT attribute really changed.
 * @author Sylvain Reynal
 * @version $Id: DrawingEvent.java,v 1.8.2.1 2007/09/02 11:56:18 reynal Exp $
 * @since jpicedt 1.3.2
 */
public interface DrawingEvent  {

	/**
	 * Return the source of this event
	 */
    public Drawing getDrawing();

	/**
	 * Return the graphic element that changed
	 */
    public Element getElement();

	/**
	 * Return the event type
	 */
    public EventType getType();

	/**
	 * enumeration for DrawingEvent types
	 */
	public class EventType {
		
		/** an element geometry changed */
		public static final EventType GEOMETRY_CHANGE = new EventType("geometry-change");
		/** an element attribute or attribute set changed */
		public static final EventType ATTRIBUTE_CHANGE = new EventType("attribute-change");
		/** the text content of a text element changed */
		public static final EventType TEXT_CHANGE = new EventType("text-change");
		/** a child was inserted to the changed-element */
		public static final EventType INSERT = new EventType("insert");
		/** a child was removed from the changed-element */
		public static final EventType REMOVE = new EventType("remove");
		/** a child was replaced by another element in the changed-element */
		public static final EventType REPLACE = new EventType("replace");
		
		private String type;

		private EventType(String str){
			this.type = str;
		}
		
		public String toString(){
			return type;
		}
	}
		
}
