/*
 PEMouseListener.java - January 3, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.event;

import java.util.EventListener;

/**
 * The listener for receiving mouse and mouse motion events. In comparison with 
 * {@link javax.swing.event.MouseInputListener javax.swing.event.MouseInputListener},
 * the benefit of using this listener is that MouseEvent's are translated into PEMouseEvent's, and the latter
 * contain enhanced data e.g. model-coordinates in addition to usual screen-coordinates. 
 * @author Sylvain Reynal
 * @version $Id: PEMouseInputListener.java,v 1.5.2.1 2007/09/02 11:56:18 reynal Exp $
 * @see jpicedt.graphic.event.PEMouseEvent
 * @since jpicedt 1.3.2
 */
public interface PEMouseInputListener extends EventListener {

	/** Invoked when a mouse button has been pressed on a component. */
	public void mousePressed(PEMouseEvent e);
	
	/** Invoked when a mouse button has been released on a component. */
	public void mouseReleased(PEMouseEvent e);
	
	/** Invoked when the mouse button has been clicked (pressed and released) on a component. */
	public void mouseClicked(PEMouseEvent e);	
	
	/**  Invoked when the mouse button has been moved on a component (with no buttons down). */
	public void mouseMoved(PEMouseEvent e);
	
	/** Invoked when a mouse button is pressed on a component and then dragged. */
	public void mouseDragged(PEMouseEvent e);
	
	/** Invoked when the mouse enters a component. */
	public void mouseEntered(PEMouseEvent e);
	
	/** nvoked when the mouse exits a component. */
	public void mouseExited(PEMouseEvent e);
	
} // interface

