/*
 SelectionEvent.java - February 15, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.event;

import jpicedt.graphic.model.Element;
import jpicedt.graphic.PECanvas;

import java.util.EventObject;
import java.util.Iterator;

/**
 * Class for notifications of changes that occured in a SelectionHandler (strongly mimics 
 * {@link javax.swing.event.CaretEvent javax.swing.event.CaretEvent})
 * @author Sylvain Reynal
 * @version $Id: SelectionEvent.java,v 1.5.2.1 2007/09/02 11:56:18 reynal Exp $
 * @since jpicedt 1.3.2
 */
public class SelectionEvent extends EventObject {

	/** the event type */
	protected EventType type;
	/** the elements that were (un)selected */
	protected Element[] elements;
	
	/**
	 * a new SelectionEvent sourced from the given PECanvas
	 * Use this constructor when several elements were (un)selected.
	 * @param source the originator of the event
	 * @param elements the elements that were (un)selected
	 * @param the event type
	 */
	public SelectionEvent(PECanvas source, Element[] elements, EventType type){
		super(source);
		this.type=type;
		this.elements=elements;
	}
	
	/**
	 * a new SelectionEvent sourced from the given PECanvas. 
	 * Use this constructor when only one element was (un)selected.
	 * @param source the originator of the event
	 * @param element the element that was (un)selected
	 * @param the event type
	 */
	public SelectionEvent(PECanvas source, Element element, EventType type){
		super(source);
		this.type=type;
		elements = new Element[1];
		elements[0] = element;
	}

	/**
	 * Return the graphic elements that got selected/unselected.
	 */
    public Element[] getElements(){
		return elements;
	}

	/**
	 * Return the event type
	 */
    public EventType getType(){
		return type;
	}

	/**
	 * a textual representation of this event
	 */
	public String toString(){
			String s = "[SelectionEvent@" + Integer.toHexString(hashCode()) + ", type=" + getType();
			for (int i=0; i<elements.length; i++){
				s += ", (un)selected element["+i+"]=" + getElements()[i];
			}
			s += ", new-selection-buffer-content=";
			for(Iterator it = ((PECanvas)getSource()).selection(); it.hasNext();){
				s += ((Element)it.next()).toString() + ",";
			}
			s += "]";
			return s;
	}

	//////////////////////////////////////////////////////////////

	/**
	 * enumeration for SelectionEvent types
	 */
	public static class EventType {
		
		/** an element was selected */
		public static final EventType SELECT = new EventType("selected");
		/** an element was unselected */
		public static final EventType UNSELECT = new EventType("unselected");
		
		private String type;

		private EventType(String str){
			this.type = str;
		}
		
		public String toString(){
			return type;
		}
	}
		
}
