/*
 ZoomEvent.java - February 19, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package jpicedt.graphic.event;

import jpicedt.graphic.PECanvas;
import jpicedt.graphic.PicPoint;

import java.util.*;
import javax.swing.event.*;

/**
 * Class for notifications of zoom-factor changes sourced by a PECanvas.
 * @author Sylvain Reynal
 * @version $Id: ZoomEvent.java,v 1.5.2.1 2007/09/02 11:56:18 reynal Exp $
 * @since jpicedt 1.3.2
 */
public class ZoomEvent extends EventObject {

	/** the old zoom factor */
	private double oldZoom;
	/** the new zoom factor */
	private double newZoom;
	/** the coordinate of the point that was at the center of the canvas before the zoom changed */
	private PicPoint ptCenter;
	
	/**
	 * a new ZoomEvent sourced from the given PECanvas
	 * @param source the originator of the event
	 * @param oldZoom the previous zoom value
	 * @param newZoom the new zoom value
	 * @param ptCenter the point (in model-coordinates) that is expected to be at the center of the
	 *        view-port. Can be null.
	 */
	public ZoomEvent(PECanvas source, double oldZoom, double newZoom, PicPoint ptCenter){
		super(source);
		this.oldZoom = oldZoom;
		this.newZoom = newZoom;
		if (ptCenter != null) this.ptCenter = new PicPoint(ptCenter);
		else this.ptCenter = null;
	}

	/**
	 * Return the old zoom factor value
	 */
	public double getOldZoomValue(){
		return oldZoom;
	}

	/**
	 * Return the new zoom factor value 
	 */
	public double getNewZoomValue(){
		return newZoom;
	}

	/**
	 * Return the coordinates of the point that was at the center of the canvas before the zoom changed.
	 * This is aimed at allowing javax.swing.ViewPort, javax.swing.Scrollpane, etc... to update their view accordingly.
	 */
	public PicPoint getCenterPoint(){
		return ptCenter;
	}
		
	/**
	 * a textual representation of this event
	 */
	public String toString(){
		return "source=" + source
		+ ", old zoom = " + oldZoom
		+ ", new zoom = " + newZoom
		+ ", center point = " + ((ptCenter==null) ? "null" : ptCenter.toString());
	}
		
}
