/*
 FormatterFactory.java - February 26, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Departement de Physique
 Ecole Nationale Superieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.io.formatter;

import jpicedt.graphic.model.Element;
import jpicedt.graphic.model.Drawing;

public interface FormatterFactory {
	
	/** a constraint to indicate that the created Formatter should be able to 
	 *  create a standalone String which once save, can be directly fed to, e.g., latex or ghostview,...
	 * as opposed to a "picture environment" file which still has to be included in a LaTeX file
	 */
	public static final String MAKE_STANDALONE_FILE = "make-standalone-file";
	
	/** 
	 * @return a Formatter able to format the given Element according to the format of this factory
	 */
	Formatter createFormatter(Element e);
	
	/** 
	 * @return a Formatter able to format the given Drawing according to the format of this factory ;
	 *         this may reliy on calls to <code>createFormatter(Element e)</code> on the elements
	 *         of the drawing, plus creating auxiliary
	 * @param outputConstraint constraint used by the factory to create a specific Formatter on-the-fly
	 */
	Formatter createFormatter(Drawing d, Object outputConstraint);
	
}
