/*
 JPICMultiCurveFormatter.java - August 26, 2003 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Departement de Physique
 Ecole Nationale Superieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.io.formatter;

import jpicedt.graphic.*;
import jpicedt.graphic.model.*;
import java.util.HashMap;

import java.awt.*;

/**
 * JPIC-XML formatter for PicMultiCurve objects.
 * JPIC-XML syntax :
 * <multicurve points="(x1,y1);(x2,y2);...;(xn,yn)" polydots-attribs other-attribs />
 *
 * @since jpicedt 1.4
 * @author Sylvain Reynal
 * @version $Id: JPICMultiCurveFormatter.java,v 1.7.2.1 2007/09/02 11:56:19 reynal Exp $ 
 *
 */
public class JPICMultiCurveFormatter implements Formatter, PicObjectConstants, JPICConstants {

	/** the Element this formatter acts upon */
	private PicMultiCurve curve;
	/** the factory that produced this formatter */
	private JPICFormatter factory;

	/**
	 * @param curve the PicMultiCurve to be formatted
	 * @param factory the factory that produced this formatter
	 */
	public JPICMultiCurveFormatter(PicMultiCurve curve, JPICFormatter factory){
		this.curve = curve;
		this.factory=factory;
	}

	public String format(){
		StringBuffer buf = new StringBuffer(100); // 100 as initial capacity seems to be a good guess
		PicAttributeSet set = curve.getAttributeSet();
	
		buf.append("<multicurve");
		
		PicPoint[] pts = new PicPoint[curve.getLastPointIndex()+1];
		for (int i = 0; i <= curve.getLastPointIndex(); i++) 
			pts[i]=curve.getPoint(i, null);
		
		XmlAttributeSet map = new XmlAttributeSet();
		map.putNameValuePair("points", pts);

		// polydots :
		if (StyleConstants.getPolydotsStyle(set)!=NONE){
			map.putNameValuePair(POLYDOTS_STYLE,StyleConstants.getPolydotsStyle(set));
			map.putNameValuePair(POLYDOTS_SIZE_MINIMUM_MM,StyleConstants.getPolydotsSizeMinimumMm((set)));
			map.putNameValuePair(POLYDOTS_SIZE_LINEWIDTH_SCALE,StyleConstants.getPolydotsSizeLineWidthScale(set));
			map.putNameValuePair(POLYDOTS_ANGLE,StyleConstants.getPolydotsAngle(set));
			map.putNameValuePair(POLYDOTS_SCALE_H,StyleConstants.getPolydotsHorizontalScale(set));
			map.putNameValuePair(POLYDOTS_SCALE_V,StyleConstants.getPolydotsVerticalScale(set));
			map.putNameValuePair(POLYDOTS_SUPERIMPOSE,StyleConstants.isPolydotsSuperimposed(set));
		}
		
		//buf.append("\" closed="); // not necessary, since the nb of bezier-points is enough to know 
		//buf.append(curve.isClosed()); // whether curve is closed or not.
		
		map.putCommonAttributes(curve);
		buf.append(map.toXML()); // translate map into XML name=value pairs
		buf.append(" />");
		buf.append(CR_LF);
		return buf.toString();
	
		
	}

} 
