/*
 JPICParallelogramFormatter.java - August 26, 2003 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Departement de Physique
 Ecole Nationale Superieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.io.formatter;

import jpicedt.graphic.*;
import jpicedt.graphic.model.*;

import java.awt.*;

/**
 * JPIC-XML formatter for PicParallelogram objects.
 * JPIC-XML syntax :
 * <parallelogram p1="(x1,y1)" p2="(x2,y2)" p3="(x3,y3)" other-attribs />
 *
 * @since jpicedt 1.4
 * @author Sylvain Reynal
 * @version $Id: JPICParallelogramFormatter.java,v 1.5.2.1 2007/09/02 11:56:19 reynal Exp $ 
 *
 */
public class JPICParallelogramFormatter implements Formatter, PicObjectConstants, JPICConstants {

	/** the Element this formatter acts upon */
	private PicParallelogram obj;
	/** the factory that produced this formatter */
	private JPICFormatter factory;

	/**
	 * @param obj the PicParallelogram to be formatted
	 * @param factory the factory that produced this formatter
	 */
	public JPICParallelogramFormatter(PicParallelogram obj, JPICFormatter factory){
		this.obj = obj;
		this.factory=factory;
	}

	/**
	 * PsTricks formating algorithm
	 * ex :     \psframe[framearc=0.5,linecolor=red,linewidth=0.2,fillstyle=solid,fillcolor=blue](2,2)(5,7)
	 * @return a String representing this Element in the PsTricks 
	 */
	public String format(){
		StringBuffer buf = new StringBuffer(100); // 100 as initial capacity seems to be a good guess
	
		buf.append("<parallelogram");
		// simply add parallelogram's three specification points, which are BL, BR and TR, since it's enough
		// to create a new PicEllipseFromParallelo when re-parsing + add angles
		XmlAttributeSet map = new XmlAttributeSet();
		map.putNameValuePair("p1",obj.getPoint(PicParallelogram.P_BL,null));
		map.putNameValuePair("p2",obj.getPoint(PicParallelogram.P_BR,null));
		map.putNameValuePair("p3",obj.getPoint(PicParallelogram.P_TR,null));
		map.putCommonAttributes(obj);
		buf.append(map.toXML());
		buf.append(" />");
		buf.append(CR_LF);
		return buf.toString();
	}

} 
