/*
 JPICTextFormatter.java - August 5, 2003 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Departement de Physique
 Ecole Nationale Superieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
package jpicedt.graphic.io.formatter;

import jpicedt.graphic.*;
import jpicedt.graphic.model.*;

import java.awt.*;

/**
 * A formatter for PicText objects in the JPIC-XML format :<br>
 * <text vert-align="bottom|baseline|center-v|top" hor-align="left|center-h|right" anchor-point="(x,y)" box="no_frame|rectangle|circle|oval">text here !</text>
 * where : <br>
 * <ul>
 * <li>refpoint = B|b|t for vertical and l|r for horizontal (B = baseline)
 * <li>rotation = any angle in degree or U,L,D,R,N,W,S or E.
 * </ul>
 * @since jpicedt 1.4
 * @author Sylvain Reynal
 * @version $Id: JPICTextFormatter.java,v 1.8.2.2 2007/09/02 11:56:20 reynal Exp $ 
*/
public class JPICTextFormatter implements Formatter, PicObjectConstants, JPICConstants {

	/** the Element this formatter acts upon */
	private PicText obj;
	/** the factory that produced this formatter */
	private JPICFormatter factory;

	/**
	 * @param obj the PicText to be formatted
	 * @param factory the factory that produced this formatter
	 */
	public JPICTextFormatter(PicText obj, JPICFormatter factory){
		this.obj = obj;
		this.factory=factory;
	}
	
	/**
	 * @return a String representing this Element in the PsTricks 
	 */
	public String format(){
		
		StringBuffer buf = new StringBuffer(100);
		
		buf.append("<text");
		
		XmlAttributeSet map = new XmlAttributeSet();
		
		// alignment
		map.putNameValuePair(TEXT_VERT_ALIGN.getName(),obj.getVertAlign());
		map.putNameValuePair(TEXT_HOR_ALIGN.getName(),obj.getHorAlign());
	
		// anchor-point
		map.putNameValuePair("anchor-point",obj.getPoint(PicText.P_ANCHOR,null));
		
		// box
		map.putNameValuePair(TEXT_FRAME.getName(),obj.getFrameType());

		// rotation (degrees)
		double angle = ((Double)obj.getAttribute(TEXT_ROTATION)).doubleValue();
		map.putNameValuePair(TEXT_ROTATION.getName(),angle,0.0); // def=0.0 deg

		map.putCommonAttributes(obj);

		// text
		buf.append(map.toXML());
		buf.append(" >"); // close tag
		buf.append(CR_LF);
		appendEncoded(buf,obj.getText());
		buf.append(CR_LF);
		buf.append("</text>");
		buf.append(CR_LF);
		return buf.toString();
		
	}
	
	private void appendEncoded(StringBuffer buf, String text){
		for (int i=0; i<text.length(); i++){
			char c = text.charAt(i);
			switch (c){
				case '<' : buf.append("&lt;"); break;
				case '>' : buf.append("&gt;"); break;
				case '&' : buf.append("&amp;"); break;
				default : buf.append(c);
			}
		}
	}

} 
