/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.graphic.model;

/**
 * A wrapper for an attribute name that can be pushed into a PicAttributeSet.
 * This allows typesafe enumerations, and besides enforces runtime checking as for 
 * attributes values that can be safely attached to this PicAttributeName.
 * @author Sylvain Reynal
 * @since jpicedt 1.3.2
 * @version $Id: PicAttributeName.java,v 1.9.2.1 2007/09/02 11:56:26 reynal Exp $
 * <p>
 */
public class PicAttributeName {

    private String name;
	protected Class allowedValueClass; 

    /**
     * @param name PsTrick's name for this attribute
	 * @param allowedValueClass which class is allowed for attribute values
     */
    protected PicAttributeName(String name, Class allowedValueClass){ 
		this.name = name; 
		this.allowedValueClass = allowedValueClass;
	}
    
    public String toString(){
		return name;
	}

	/**
	 * Return the name of the attribute described by this PicAttributeName
	 */
	public String getName(){
		return name;
	}
	
	/**
	 * Return the class that is allowed for attribute values
	 */
	public Class getAllowedValueClass(){
		return allowedValueClass;
	}
    
} // class

