/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.graphic.model;

import java.awt.*;

/** 
 * Useful constants for the Element interface. 
 *
 * @author Sylvain Reynal
 * @since jpicedt 1.3.2
 * @version $Id: PicObjectConstants.java,v 1.16.2.1 2007/09/02 11:56:27 reynal Exp $
 */
public interface PicObjectConstants {

    /**
     * a PostScript point in mm
     */
    double PS_POINT = 25.6 / 72.0; // 0.35555... mm

    /**
     * Linestyle
     */
    String NONE = "none";
    String SOLID = "solid"; // PsTricks default
    String DASHED = "dashed";
    String DOTTED = "dotted";

    /**
     * predefined dashes (in mm) and dotsep (in mm)
     */
    Double DASH1 = new Double(1.0);
    Double DASH2 = new Double(2.0);
    Double DASH3 = new Double(3.0);
    Double DOT_SEP1 = new Double(1.0);
    Double DOT_SEP2 = new Double(2.0);
    Double DOT_SEP3 = new Double(3.0);

    /**
     * PsTricks's Dimen
     */
    String OUTER="outer";
    String MIDDLE="middle";
    String INNER="inner";

    /**
     * Fillstyle
     */
    // NONE and SOLID already defined
    String VLINES = "vlines";
    String VLINES_FILLED = "vlines-filled";
    String HLINES = "hlines";
    String HLINES_FILLED = "hlines-filled";
    String CROSSHATCH = "crosshatch";
    String CROSSHATCH_FILLED = "crosshatch-filled";

    /**
     * Polydots style
     */
	String POLYDOTS_DISK = "polydots-disk";
	String POLYDOTS_CIRCLE = "polydots-circle";
	String POLYDOTS_PLUS = "polydots-plus";
	String POLYDOTS_TRIANGLE = "polydots-triangle";
	String POLYDOTS_TRIANGLE_FILLED = "polydots-triangle-filled";
	String POLYDOTS_SQUARE = "polydots-square";
	String POLYDOTS_SQUARE_FILLED = "polydots-square-filled";
	String POLYDOTS_PENTAGON = "polydots-pentagon";
	String POLYDOTS_PENTAGON_FILLED = "polydots-pentagon-filled";
	
	/**
	 * Text attributes 
	 */
	 
	String TEXT_VALIGN_BOTTOM = "bottom";
	String TEXT_VALIGN_BASELINE = "baseline"; // for PsTricks only
	String TEXT_VALIGN_CENTER = "center-v"; // vertical
	String TEXT_VALIGN_TOP = "top";
	
	String TEXT_HALIGN_LEFT = "left";
	String TEXT_HALIGN_CENTER = "center-h"; // horizontal
	String TEXT_HALIGN_RIGHT = "right";

	String TEXT_BOX_NO_FRAME = "noframe";
	String TEXT_BOX_RECTANGLE = "rectangle";
	String TEXT_BOX_CIRCLE = "circle";
	String TEXT_BOX_OVAL = "oval";
 

	/**
     * Attribute Names
     */
    PicAttributeName LINE_STYLE = new PicAttributeName("stroke-style",String.class);
    PicAttributeName LINE_COLOR = new PicAttributeName("stroke-color",Color.class);
    PicAttributeName LINE_WIDTH = new PicAttributeName("stroke-width",Double.class);
    PicAttributeName DASH_TRANSPARENT = new PicAttributeName("stroke-dash-transparent",Double.class);
    PicAttributeName DASH_OPAQUE= new PicAttributeName("stroke-dash-opaque",Double.class);
    PicAttributeName DOT_SEP = new PicAttributeName("stroke-dotsep",Double.class);
    PicAttributeName DOUBLE_LINE = new PicAttributeName("stroke-doubleline",Boolean.class); // true/false
    PicAttributeName DOUBLE_SEP= new PicAttributeName("stroke-doubleline-sep",Double.class);
    PicAttributeName DOUBLE_COLOR = new PicAttributeName("stroke-doubleline-color",Color.class);
    PicAttributeName OVER_STRIKE = new PicAttributeName("stroke-overstrike",Boolean.class); // do overstrike or not
	PicAttributeName OVER_STRIKE_WIDTH = new PicAttributeName("stroke-overstrike-width",Double.class); // width of overstriking
    PicAttributeName OVER_STRIKE_COLOR = new PicAttributeName("stroke-overstrike-color",Color.class); // border color
	
    PicAttributeName SHADOW = new PicAttributeName("shadow",Boolean.class);
    PicAttributeName SHADOW_SIZE= new PicAttributeName("shadow-size",Double.class);
    PicAttributeName SHADOW_ANGLE= new PicAttributeName("shadow-angle",Double.class);
    PicAttributeName SHADOW_COLOR= new PicAttributeName("shadow-color",Color.class);
	
    PicAttributeName DIMEN = new PicAttributeName("dimen",String.class);

    PicAttributeName FILL_STYLE = new PicAttributeName("fill-style",String.class);
    PicAttributeName FILL_COLOR = new PicAttributeName("fill-color",Color.class);
    PicAttributeName HATCH_WIDTH = new PicAttributeName("fill-hatch-width",Double.class);
    PicAttributeName HATCH_SEP= new PicAttributeName("fill-hatch-sep",Double.class);
    PicAttributeName HATCH_COLOR = new PicAttributeName("fill-hatch-color",Color.class);
    PicAttributeName HATCH_ANGLE = new PicAttributeName("fill-hatch-angle",Double.class);

    PicAttributeName LEFT_ARROW = new PicAttributeName("left-arrow",Arrow.class);
    PicAttributeName RIGHT_ARROW= new PicAttributeName("right-arrow",Arrow.class);
	PicAttributeName ARROW_GLOBAL_SCALE_WIDTH= new PicAttributeName("arrow-global-scale-width",Double.class);
	PicAttributeName ARROW_GLOBAL_SCALE_LENGTH= new PicAttributeName("arrow-global-scale-length",Double.class);
	PicAttributeName ARROW_WIDTH_MINIMUM_MM= new PicAttributeName("arrow-head-width-minimum",Double.class);
	PicAttributeName ARROW_WIDTH_LINEWIDTH_SCALE= new PicAttributeName("arrow-head-width-linewidth-scale",Double.class);
	PicAttributeName ARROW_LENGTH_SCALE= new PicAttributeName("arrow-head-length-scale",Double.class);
	PicAttributeName ARROW_INSET_SCALE= new PicAttributeName("arrow-head-inset-scale",Double.class);
	PicAttributeName TBAR_WIDTH_MINIMUM_MM= new PicAttributeName("arrow-tbar-width-minimum",Double.class);
	PicAttributeName TBAR_WIDTH_LINEWIDTH_SCALE= new PicAttributeName("arrow-tbar-width-linewidth-scale",Double.class);
	PicAttributeName BRACKET_LENGTH_SCALE= new PicAttributeName("arrow-bracket-length-scale",Double.class);
	PicAttributeName RBRACKET_LENGTH_SCALE= new PicAttributeName("arrow-rbracket-length-scale",Double.class);
	// dotsize -> see below for polydots
	
    PicAttributeName POLYDOTS_STYLE= new PicAttributeName("polydots-style",String.class); // style of ps-dotted AbstractCurve's (aka \\psdots)
	PicAttributeName POLYDOTS_SUPERIMPOSE= new PicAttributeName("polydots-superimpose",Boolean.class); // superimpose dots on top of underlying curve or not
	// the two followings attributes are also used for circle/disk arrows:
	PicAttributeName POLYDOTS_SIZE_MINIMUM_MM= new PicAttributeName("polydots-size-minimum",Double.class); // dotsize (1) = dim
	PicAttributeName POLYDOTS_SIZE_LINEWIDTH_SCALE= new PicAttributeName("polydots-size-linewidth-scale",Double.class); // dotsize (2) = num
	PicAttributeName POLYDOTS_SCALE_H= new PicAttributeName("polydots-scale-h",Double.class); // dotscale (1)
	PicAttributeName POLYDOTS_SCALE_V= new PicAttributeName("polydots-scale-v",Double.class); // dotscale (2)
	PicAttributeName POLYDOTS_ANGLE= new PicAttributeName("polydots-angle",Double.class); // dotangle
	
	PicAttributeName TEXT_VERT_ALIGN = new PicAttributeName("text-vert-align",String.class); 
	PicAttributeName TEXT_HOR_ALIGN = new PicAttributeName("text-hor-align",String.class); 
	PicAttributeName TEXT_FRAME = new PicAttributeName("text-frame",String.class);
	PicAttributeName TEXT_ROTATION = new PicAttributeName("text-rotation",Double.class); // angle in degrees

 
    PicAttributeName PST_CUSTOM = new PicAttributeName("pstcustom",String.class); // custom pstricks parameters
} 

