/*
 StyleConstants.java - March 30, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.model;

import java.awt.*;

/**
 * Utility class to get or set attributes in a type-safe manner
 * @author Sylvain Reynal
 * @since jpicedt 1.3.2
 * @see jpicedt.graphic.model.PicObjectConstants
 * @version $Id: StyleConstants.java,v 1.11.2.1 2007/09/02 11:56:27 reynal Exp $
 * 
 */
public class StyleConstants implements PicObjectConstants {

	///////////////////// STROKE /////////////////////
	/**
	 * Return the line style 
	 */
	public static String  getLineStyle(PicAttributeSet set){
		return getStringAttribute(set,LINE_STYLE);
	}
	 
	/**
	 * set the line style 
	 */
	public static void setLineStyle(PicAttributeSet set, String style){
		set.setAttribute(LINE_STYLE,style);
	}

	/**
	 * Return the stroke color 
	 */
	public static Color getLineColor(PicAttributeSet set){
		return getColorAttribute(set,LINE_COLOR);
	}

	/**
	 * set the stroke color 
	 */
	public static void setLineColor(PicAttributeSet set, Color c){
		set.setAttribute(LINE_COLOR,c);
	}

	/**
	 * Return the line width, guaranteed to be positive.
	 */
	public static double getLineWidth(PicAttributeSet set){
		return Math.abs(getDoubleAttribute(set,LINE_WIDTH));
	}

	/**
	 * set the line width
	 */
	public static void setLineWidth(PicAttributeSet set, double width){
		set.setAttribute(LINE_WIDTH,new Double(width));
	}
	
	/**
	 * Return the transparent dash length , guaranteed to be positive.
	 */
	public static double  getDashTransparent(PicAttributeSet set){
		return Math.abs(getDoubleAttribute(set,DASH_TRANSPARENT));
	}

	/**
	 * set the transparent dash length 
	 */
	public static void setDashTransparent(PicAttributeSet set, double d){
		set.setAttribute(DASH_TRANSPARENT,new Double(d));
	}
	
	/**
	 * Return the opaque dash length , guaranteed to be positive.
	 */
	public static double  getDashOpaque(PicAttributeSet set){
		return Math.abs(getDoubleAttribute(set,DASH_OPAQUE));
	}

	/**
	 * set the opaque dash length
	 */
	public static void setDashOpaque(PicAttributeSet set, double d){
		set.setAttribute(DASH_OPAQUE,new Double(d));
	}
	
	/**
	 * Return the dot separation, guaranteed to be positive.
	 */
	public static double getDotSep(PicAttributeSet set){
		return Math.abs(getDoubleAttribute(set,DOT_SEP));
	}

	/**
	 * set the dot separation
	 */
	public static void setDotSep(PicAttributeSet set, double d){
		set.setAttribute(DOT_SEP,new Double(d));
	}

	///////////////// DOUBLE LINE //////////////////////////
	/**
	 * Return whether double-line is on or off
	 */
	public static boolean isDoubleLine(PicAttributeSet set){
		return getBooleanAttribute(set,DOUBLE_LINE);
	}

	/**
	 * set whether  double-line is on or off
	 */
	public static void setDoubleLine(PicAttributeSet set, boolean b){
		set.setAttribute(DOUBLE_LINE,new Boolean(b));
	}

	/**
	 * Return the DoubleLine separation, guaranteed to be positive.
	 */
	public static double getDoubleLineSep(PicAttributeSet set){
		return Math.abs(getDoubleAttribute(set,DOUBLE_SEP));
	}

	/**
	 * set the DoubleLine separation
	 */
	public static void setDoubleLineSep(PicAttributeSet set, double d){
		set.setAttribute(DOUBLE_SEP,new Double(d));
	}

	/**
	 * Return the double-line colour 
	 */
	public static Color getDoubleLineColor(PicAttributeSet set){
		return getColorAttribute(set,DOUBLE_COLOR);
	}

	/**
	 * Return the double-line colour 
	 */
	public static void setDoubleLineColor(PicAttributeSet set, Color c){
		set.setAttribute(DOUBLE_COLOR,c);
	}

        ///////////////////////////// OVERSTRIKE ///////////////////////////
   
	/**
	 * Return whether overstrike is on or off
	 */
	public static boolean isOverStrike(PicAttributeSet set){
		return getBooleanAttribute(set, OVER_STRIKE);
	}

	/**
	 * set whether  overstrike is on or off
	 */
	public static void setOverStrike(PicAttributeSet set, boolean b){
		set.setAttribute(OVER_STRIKE,new Boolean(b));
	}

	/**
	 * Return the overstrike width, guaranteed to be positive.
	 */
	public static double getOverStrikeWidth(PicAttributeSet set){
		return Math.abs(getDoubleAttribute(set,OVER_STRIKE_WIDTH));
	}

	/**
	 * set the overstrike width
	 */
	public static void setOverStrikeWidth(PicAttributeSet set, double width){
		set.setAttribute(OVER_STRIKE_WIDTH,new Double(width));
	}

	/**
	 * Return the border color
	 */
	public static Color getOverStrikeColor(PicAttributeSet set){
		return getColorAttribute(set,OVER_STRIKE_COLOR);
	}

	/**
	 * set the border color
	 */
	public static void setOverStrikeColor(PicAttributeSet set, Color c){
		set.setAttribute(OVER_STRIKE_COLOR,c);
	}


	/////////////////// SHADOW /////////////////////
	/**
	 * Return whether shadow is on or off
	 */
	public static boolean isShadow(PicAttributeSet set){
		return getBooleanAttribute(set, SHADOW);
	}

	/**
	 * set whether  shadow is on or off
	 */
	public static void setShadow(PicAttributeSet set, boolean b){
		set.setAttribute(SHADOW,new Boolean(b));
	}

	/**
	 * Return the shadow size , guaranteed to be positive.
	 */
	public static double getShadowSize(PicAttributeSet set){
		return Math.abs(getDoubleAttribute(set, SHADOW_SIZE));
	}

	/**
	 * set the ShadowSize
	 */
	public static void setShadowSize(PicAttributeSet set, double d){
		set.setAttribute(SHADOW_SIZE,new Double(d));
	}

	/**
	 * Return the shadow angle, guaranteed to lie b/w -180 and 180
	 */
	public static double getShadowAngle(PicAttributeSet set){
		return getDoubleAttribute(set, SHADOW_ANGLE)%180; // b/w -180 and 180
	}

	/**
	 * set the shadow angle
	 */
	public static void setShadowAngle(PicAttributeSet set, double d){
		set.setAttribute(SHADOW_ANGLE,new Double(d));
	}

	/**
	 * Return the shadow colour
	 */
	public static Color getShadowColor(PicAttributeSet set){
		return getColorAttribute(set,SHADOW_COLOR);
	}

	/**
	 * set the shadow colour
	 */
	public static void setShadowColor(PicAttributeSet set, Color c){
		set.setAttribute(SHADOW_COLOR,c);
	}

	
	//////////////////// DIMEN ///////////////////////////////
	/**
	 * Return a String (OUTER, MIDDLE or INNER) indicating whether the dimensions of boxes, circles,...
	 *         refer to the inside, outside or middle of the boundary.
	 */
	public static String getDimen(PicAttributeSet set){
		return getStringAttribute(set,DIMEN);
	}

	/**
	 * set the "dimen" parameter to OUTER, MIDDLE or INNER.
	 */
	public static void setDimen(PicAttributeSet set, String style){
		set.setAttribute(DIMEN,style);
	}

	//////////////////////// FILL //////////////////////////
	/**
	 * Return the fill style
	 */
	public static String getFillStyle(PicAttributeSet set){
		return getStringAttribute(set,FILL_STYLE);
	}

	/**
	 * set the fill style to NONE, SOLID, HLINES, ...
	 */
	public static void setFillStyle(PicAttributeSet set, String style){
		set.setAttribute(FILL_STYLE,style);
	}
	
	/**
	 * Return the fill colour
	 */
	public static Color getFillColor(PicAttributeSet set){
		return getColorAttribute(set,FILL_COLOR);
	}

	/**
	 * set the fill colour
	 */
	public static void setFillColor(PicAttributeSet set, Color c){
		set.setAttribute(FILL_COLOR,c);
	}

	///////////////// HATCH /////////////////////////////////////
	/**
	 * Return the hatch width, guaranteed to be positive
	 */
	public static double getHatchWidth(PicAttributeSet set){
		return Math.abs(getDoubleAttribute(set, HATCH_WIDTH));
	}

	/**
	 * set the hatch width
	 */
	public static void setHatchWidth(PicAttributeSet set, double d){
		set.setAttribute(HATCH_WIDTH,new Double(d));
	}

	/**
	 * Return the distance b/w hatches, guaranteed to be positive 
	 */
	public static double getHatchSep(PicAttributeSet set){
		return Math.abs(getDoubleAttribute(set, HATCH_SEP));
	}

	/**
	 * set the distance b/w hatches 
	 */
	public static void setHatchSep(PicAttributeSet set, double d){
		set.setAttribute(HATCH_SEP,new Double(d));
	}

	/**
	 * Return the hatch colour
	 */
	public static Color getHatchColor(PicAttributeSet set){
		return getColorAttribute(set,HATCH_COLOR);
	}

	/**
	 * set the hatch colour
	 */
	public static void setHatchColor(PicAttributeSet set, Color c){
		set.setAttribute(HATCH_COLOR,c);
	}

	/**
	 * Return the hatch angle, guaranteed to be b/w -90 and 90. 
	 */
	public static double getHatchAngle(PicAttributeSet set){
		double a = getDoubleAttribute(set, HATCH_ANGLE)%180; // b/w -180 and 180
		if (a<-90) a+= 180;
		else if (a>90) a-=180;
		return a;
	}

	/**
	 * set the hatch angle
	 */
	public static void setHatchAngle(PicAttributeSet set, double d){
		set.setAttribute(HATCH_ANGLE,new Double(d));
	}

	///////////////// ARROW //////////////////////////////
	
	/**
	 * Return the left arrow
	 */
	public static Arrow getLeftArrow(PicAttributeSet set){
		return (Arrow)set.getAttribute(LEFT_ARROW);
	}

	/**
	 * set the left arrow
	 */
	public static void setLeftArrow(PicAttributeSet set,Arrow a){
		set.setAttribute(LEFT_ARROW,a);
	}

	/**
	 * Return the right arrow 
	 */
	public static Arrow getRightArrow(PicAttributeSet set){
		return (Arrow)set.getAttribute(RIGHT_ARROW);
	}

	/**
	 * set the right arrow
	 */
	public static void setRightArrow(PicAttributeSet set,Arrow a){
		set.setAttribute(RIGHT_ARROW,a);
	}


	/**
	 * Returns the scale-factor for the arrow width
	 */
	public static double getArrowGlobalScaleWidth(PicAttributeSet set){
		return getDoubleAttribute(set, ARROW_GLOBAL_SCALE_WIDTH);
	}
	
	/**
	 * Sets the scale-factor for the arrow width
	 */
	public static void setArrowGlobalScaleWidth(PicAttributeSet set, double d){
		set.setAttribute(ARROW_GLOBAL_SCALE_WIDTH, new Double(d));
	}

	/**
	 * Returns the scale-factor for the arrow length
	 */
	public static double getArrowGlobalScaleLength(PicAttributeSet set){
		return getDoubleAttribute(set, ARROW_GLOBAL_SCALE_LENGTH);
	}
	
	/**
	 * Sets the scale-factor for the arrow length
	 */
	public static void setArrowGlobalScaleLength(PicAttributeSet set, double d){
		set.setAttribute(ARROW_GLOBAL_SCALE_LENGTH, new Double(d));
	}

	/**
	 * Returns the linewidth scale-factor for the arrow width (width = line-width-scale * line-width + minimum_size)
	 */
	public static double getArrowWidthLineWidthScale(PicAttributeSet set){
		return getDoubleAttribute(set, ARROW_WIDTH_LINEWIDTH_SCALE);
	}
	
	/**
	 * Sets the the linewidth scale-factor for the arrow width (width = line-width-scale * line-width + minimum_size)
	 */
	public static void setArrowWidthLineWidthScale(PicAttributeSet set, double d){
		set.setAttribute(ARROW_WIDTH_LINEWIDTH_SCALE, new Double(d));
	}

	/**
	 * Returns the relative size for arrows (width = linewidth-scale * line-width + minimum_size)
	 */
	public static double getArrowWidthMinimumMm(PicAttributeSet set){
		return getDoubleAttribute(set, ARROW_WIDTH_MINIMUM_MM);
	}
	
	/**
	 * Sets the relative-size for arrows (width = linewidth-scale * line-width + minimum_size)
	 */
	public static void setArrowWidthMinimumMm(PicAttributeSet set, double d){
		set.setAttribute(ARROW_WIDTH_MINIMUM_MM, new Double(d));
	}
	
	/**
	 * Returns the arrow length scale-factor (actual length (or height) = actual width * ARROW_LENGTH)
	 */
	public static double getArrowLengthScale(PicAttributeSet set){
		return getDoubleAttribute(set, ARROW_LENGTH_SCALE);
	}
	
	/**
	 * Sets the arrow length scale-factor (actual length (or height) = actual width * ARROW_LENGTH)
	 */
	public static void setArrowLengthScale(PicAttributeSet set, double d){
		set.setAttribute(ARROW_LENGTH_SCALE, new Double(d));
	}
	
	/**
	 * Returns the arrow inset scale-factor (actual inset = ARROW_INSET * actual length)
	 */
	public static double getArrowInsetScale(PicAttributeSet set){
		return getDoubleAttribute(set, ARROW_INSET_SCALE);
	}
	
	/**
	 * Sets the arrow inset scale-factor (actual inset = ARROW_INSET * actual length)
	 */
	public static void setArrowInsetScale(PicAttributeSet set, double d){
		set.setAttribute(ARROW_INSET_SCALE, new Double(d));
	}
	
	/**
	 * Returns the linewidth-scale for T-bars (width = linewidth-scale * line-width + relative_width)
	 */
	public static double getTBarWidthLineWidthScale(PicAttributeSet set){
		return getDoubleAttribute(set, TBAR_WIDTH_LINEWIDTH_SCALE);
	}
	
	/**
	 * Sets the linewidth-scale for T-bars (width = linewidth-scale * line-width + relative_width)
	 */
	public static void setTBarWidthLineWidthScale(PicAttributeSet set, double d){
		set.setAttribute(TBAR_WIDTH_LINEWIDTH_SCALE, new Double(d));
	}

	/**
	 * Returns the relative-width for T-bars (width = linewidth-scale * line-width + relative_width)
	 */
	public static double getTBarWidthMinimumMm(PicAttributeSet set){
		return getDoubleAttribute(set, TBAR_WIDTH_MINIMUM_MM);
	}
	
	/**
	 * Sets the relative-width for T-bars (width = linewidth-scale * line-width + relative_width)
	 */
	public static void setTBarWidthMinimumMm(PicAttributeSet set, double d){
		set.setAttribute(TBAR_WIDTH_MINIMUM_MM, new Double(d));
	}

	/**
	 * Returns the bracket length scale-factor (actual length (or height) = actual width * BRACKET_LENGTH)
	 */
	public static double getBracketLengthScale(PicAttributeSet set){
		return getDoubleAttribute(set, BRACKET_LENGTH_SCALE);
	}
	
	/**
	 * Sets the bracket length scale-factor (actual length (or height) = actual width * BRACKET_LENGTH)
	 */
	public static void setBracketLengthScale(PicAttributeSet set, double d){
		set.setAttribute(BRACKET_LENGTH_SCALE, new Double(d));
	}

	/**
	 * Returns the rounded-bracket length scale-factor (actual length (or height) = actual width * BRACKET_LENGTH)
	 */
	public static double getRoundedBracketLengthScale(PicAttributeSet set){
		return getDoubleAttribute(set, RBRACKET_LENGTH_SCALE);
	}
	
	/**
	 * Sets the rounded-bracket length scale-factor (actual length (or height) = actual width * BRACKET_LENGTH)
	 */
	public static void setRoundedBracketLengthScale(PicAttributeSet set, double d){
		set.setAttribute(RBRACKET_LENGTH_SCALE, new Double(d));
	}
	
	///////////////// PST_CUSTOM //////////////////////////////
	
	/**
	 * Return the pst custom parameters
	 */
	public static String getPstCustom(PicAttributeSet set){
		return (String)set.getAttribute(PST_CUSTOM);
	}

	/**
	 * set pst custom parameters
	 */
	public static void setPstCustom(PicAttributeSet set,String s){
		set.setAttribute(PST_CUSTOM,s);
	}

	/////////////////// PSDOTS ///////////////////////////////////
	
	/**
	 * Returns the style of polygon dots
	 * @return POLYDOTS_DISK, POLYDOTS_CIRCLE, POLYDOTS_PLUS, POLYDOTS_TRIANGLE, POLYDOTS_TRIANGLE_FILLED, 
	 *         POLYDOTS_SQUARE, POLYDOTS_SQUARE_FILLED, POLYDOTS_PENTAGON, 
	 *         or POLYDOTS_PENTAGON_FILLED.
	 */
	public static String getPolydotsStyle(PicAttributeSet set){
		return (String)set.getAttribute(POLYDOTS_STYLE);
	}
	
	/**
	 * Sets the style of polygon dots
	 */
	public static void setPolydotsStyle(PicAttributeSet set, String s){
		set.setAttribute(POLYDOTS_STYLE,s);
	}

	/**
	 * Returns true if dots superimpose on underlying curve
	 */
	public static boolean isPolydotsSuperimposed(PicAttributeSet set){
		return getBooleanAttribute(set,POLYDOTS_SUPERIMPOSE);
	}
	
	/**
	 * Sets whether dots superimpose on underlying curve or not
	 */
	public static void setPolydotsSuperimposed(PicAttributeSet set, boolean b){
		set.setAttribute(POLYDOTS_SUPERIMPOSE, new Boolean(b));
	}
	
	/**
	 * Returns the line-width scale for polygon dots (size = linewidth-scale * line-width + minimum_size)
	 * @see #getPolydotsSizeMinimumMm
	 */
	public static double getPolydotsSizeLineWidthScale(PicAttributeSet set){
		return getDoubleAttribute(set, POLYDOTS_SIZE_LINEWIDTH_SCALE);
	}
	
	/**
	 * Sets the line-width scale for polygon dots (size = linewidth-scale * line-width + minimum_size)
	 * @see #setPolydotsSizeMinimumMm
	 */
	public static void setPolydotsSizeLineWidthScale(PicAttributeSet set, double d){
		set.setAttribute(POLYDOTS_SIZE_LINEWIDTH_SCALE, new Double(d));
	}

	/**
	 * Returns the minimum size of polygon dots (size = linewidth-scale * line-width + minimum_size)
	 * @see #getPolydotsSizeMinimumMm
	 */
	public static double getPolydotsSizeMinimumMm(PicAttributeSet set){
		return getDoubleAttribute(set, POLYDOTS_SIZE_MINIMUM_MM);
	}
	
	/**
	 * Sets the minimum size of polygon dots (size = linewidth-scale * line-width + minimum_size)
	 * @see #setPolydotsSizeLineWidthScale
	 */
	public static void setPolydotsSizeMinimumMm(PicAttributeSet set, double d){
		set.setAttribute(POLYDOTS_SIZE_MINIMUM_MM, new Double(d));
	}

	/**
	 * Returns the factor by which dots are to be scaled horizontally
	 */
	public static double getPolydotsHorizontalScale(PicAttributeSet set){
		return getDoubleAttribute(set, POLYDOTS_SCALE_H);
	}
	
	/**
	 * Sets the factor by which dots are to be scaled horizontally
	 */
	public static void setPolydotsHorizontalScale(PicAttributeSet set, double d){
		set.setAttribute(POLYDOTS_SCALE_H, new Double(d));
	}
	
	/**
	 * Returns the factor by which dots are to be scaled vertically
	 */
	public static double getPolydotsVerticalScale(PicAttributeSet set){
		return getDoubleAttribute(set, POLYDOTS_SCALE_V);
	}
	
	/**
	 * Sets the factor by which dots are to be scaled vertically
	 */
	public static void setPolydotsVerticalScale(PicAttributeSet set, double d){
		set.setAttribute(POLYDOTS_SCALE_V, new Double(d));
	}

	/**
	 * Returns the factor by which dots are to be rotated (in degrees and CCW)
	 */
	public static double getPolydotsAngle(PicAttributeSet set){
		return getDoubleAttribute(set, POLYDOTS_ANGLE);
	}
	
	/**
	 * Sets the factor by which dots are to be rotated (in degrees and CCW)
	 */
	public static void setPolydotsAngle(PicAttributeSet set, double d){
		set.setAttribute(POLYDOTS_ANGLE, new Double(d));
	}


	///////////////////////////////////////////////////////////////////////
	/** */
	private static double getDoubleAttribute(PicAttributeSet set, PicAttributeName name){
		return ((Double)set.getAttribute(name)).doubleValue();
	}

	/** */
	private static boolean getBooleanAttribute(PicAttributeSet set, PicAttributeName name){
		Object o = set.getAttribute(name);
		if (o instanceof Boolean) return ((Boolean)o).booleanValue();
		if (o instanceof String) return (((String)o).equals("true")||((String)o).equals("yes")||((String)o).equals("on"));
		if (o==null) return false;
		return true;
	}

	/** */
	private static Color getColorAttribute(PicAttributeSet set, PicAttributeName name){
		return (Color)set.getAttribute(name);
	}

	/** */
	private static String getStringAttribute(PicAttributeSet set, PicAttributeName name){
		return (String)set.getAttribute(name);
	}
	
}
