/*
 TextEditable.java - 2001 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.model;

import jpicedt.graphic.toolkit.*;
import jpicedt.graphic.*;
import jpicedt.graphic.event.*;

import java.awt.*;
import java.awt.font.*;
import java.awt.geom.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.*;

/**
 * An abstract class for Element's that have a textual representation, and [pending:removed as of 1.3.2]
 * whose text can be graphically edited
 *
 * @author Sylvain Reynal
 * @since jPicEdt 1.3.1
 * @version $Id: TextEditable.java,v 1.11.2.1 2007/09/02 11:56:27 reynal Exp $
 *
 */

public abstract class TextEditable extends DefaultLeafElement  {

	////////////////////////////
	//// PUBLIC CONSTANT FIELDS
	////////////////////////////

	// end points
	public static final int FIRST_PT = 0;
	public static final int P_ANCHOR = 0;
	public static final int LAST_PT = 0;

	/** string argument */
	protected String textString;


	/**
	 * Construct a new TextEditable with the given content and attribute set
	 */
	protected TextEditable(String textString, PicPoint anchorPoint, PicAttributeSet set){
		super(1,set); // one specification point
		this.textString = textString;
		setSpecificationPoint(P_ANCHOR,anchorPoint);
	}

	/**
	 * Construct a new TextEditable with the given content and a default attribute set
	 */
	protected TextEditable(String textString, PicPoint anchorPoint){
		super(1); // one specification point
		this.textString = textString;
		setSpecificationPoint(P_ANCHOR,anchorPoint);
	}

	/**
	 * Construct a clone of the given TextEditable
	 */
	protected TextEditable(TextEditable e){
		super(e);
		this.textString = e.textString;
	}


	/**
	 * Set the given attribute name to the given value for this AbstractElement
	 */
	public void setAttribute(PicAttributeName name, Object value){
		attributeSet.setAttribute(name,value);
		// if (name == TEXT_ROTATION) 
		//	fireChangedUpdate(DrawingEvent.EventType.TEXT_CHANGE); // because this forces updating the bitmap if applicable
		//  VG: it shouldn't.
		// else 
		// peut-etre un changement de couleur devrait relancer text_change
		fireChangedUpdate(DrawingEvent.EventType.ATTRIBUTE_CHANGE);
	}

	//////////////////////////////////
	/// OPERATIONS ON CONTROL POINTS
	//////////////////////////////////

	// inherited
	
	//////////////////////////////////////
	// TEXT SPECIFIC
	//////////////////////////////////////

	/**
	 * Return the string argument (e.g of the corresponding \\makebox{} command)
	 */
	public String getText(){
		return new String(textString);
	}

	/**
	 * Set the string argument (e.g. of the \\makebox{} command)
	 */
	public void setText(String str){
		this.textString = str;
		fireChangedUpdate(jpicedt.graphic.event.DrawingEvent.EventType.TEXT_CHANGE); // force updating everything in the view
	}
		
}


