/*
 AbstractActionLocalizer.java - January 2, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.toolkit;
import javax.swing.Icon;
import javax.swing.KeyStroke;


/**
 * This interface, once properly implemented, may be used as a parameter for a PEAction, and can feed it
 * with localized properties (e.g. tooltip, mnemonic...). Concrete implementation may be backed, for instance,
 * by a ResourceBundle initialized from a properties file.<p>
 * @since jPicEdt 1.3.2
 * @author Sylvain Reynal
 * @version $Id: ActionLocalizer.java,v 1.6.2.1 2007/09/02 11:56:28 reynal Exp $
 */
public interface ActionLocalizer {

	/**
	 * @return a localized Action name (ie Swing Action.NAME property) for the given name
	 * @param name a non-localized action name
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public String getActionName(String actionName);
	
	/**
	 * @return a localized Action tooltip for the given name (ie Swing Action.SHORT_DESCRIPTION)  
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public String getActionTooltip(String actionName);
	
	/**
	 * @return a localized Action helper (ie Swing Action.LONG_DESCRIPTION) 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public String getActionHelper(String actionName);
	
	/**
	 * @return a localized Action mnemonic (ie Swing Action.MNEMONIC_KEY) 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public Integer getActionMnemonic(String actionName);
	
	/**
	 * @return a localized Action accelerator keystroke (ie Swing Action.ACCELERATOR_KEY) 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public KeyStroke getActionAccelerator(String actionName);
	
	/**
	 * @return a localized Icon (ie Swing Action.SMALL_ICON) 
	 * @since jPicEdt
	 * @author Sylvain Reynal
	 */
	public Icon getActionIcon(String actionName);
	
} 

