/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.graphic.toolkit;

import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;


/**
 * A factory for custom cursors.
 * @author Sylvain Reynal
 * @since jPicEdt 1.3.1
 * @version $Id: CursorFactory.java,v 1.5.2.1 2007/09/02 11:56:28 reynal Exp $
 */
public class CursorFactory {

	/* custom cursor starts from 100 */
	public static final int SELECT    = Cursor.DEFAULT_CURSOR; 	// 0
	public static final int DRAW      = Cursor.CROSSHAIR_CURSOR;// 1
	public static final int MOVE      = Cursor.MOVE_CURSOR;	// 13
	public static final int N_RESIZE  = Cursor.N_RESIZE_CURSOR; // 8
	public static final int E_RESIZE  = Cursor.E_RESIZE_CURSOR; // 11
	public static final int S_RESIZE  = Cursor.S_RESIZE_CURSOR;	// 9
	public static final int W_RESIZE  = Cursor.W_RESIZE_CURSOR;	// 10
	public static final int NE_RESIZE = Cursor.NE_RESIZE_CURSOR;// 7
	public static final int NW_RESIZE = Cursor.NW_RESIZE_CURSOR;// 6
	public static final int SE_RESIZE = Cursor.SE_RESIZE_CURSOR;// 5
	public static final int SW_RESIZE = Cursor.SW_RESIZE_CURSOR;// 4
	public static final int ZOOM_IN   = 100;
	public static final int ZOOM_OUT  = 101;
	public static final int MOVE_ENDPT= 102; // arrow + "°"
	public static final int REMOVE_ENDPT= 103; // arrow + "-"
	public static final int ADD_ENDPT= 104; // arrow + "+"
	public static final int INVALID= 105; 
	public static final int MOVE_ENDPT_FROM_CENTER = 106; // arrow + "O"

	// cached cursors :
	private Cursor zoomInCursor, zoomOutCursor, moveEndPtCursor, removeEndPtCursor, addEndPtCursor;
	private Cursor invalidCursor, moveEndPtFromCenterCursor;

	/**
	 * @return a custom cursor from the given type.
	 * @param type if type relates to a custom cursor, the cursor is cached the first time.
	 */
	public Cursor getPECursor(int type){

		//String size = "16x16"; 
		//String size = "32x32"; // [pending] use 32x32 on Windows ? They look too small...
		String size = "64x64"; 
		// [pending] We should test prefered size with ...
		// Dimension dim = Toolkit.getBestCursorSize(32,32)

		switch(type){

		case ZOOM_IN :
			if (zoomInCursor != null) return zoomInCursor;
			zoomInCursor = createCustomCursor("CursorZoomin",size,"ZoomIn",6,6);
			return zoomInCursor;
		case ZOOM_OUT :
			if (zoomOutCursor != null) return zoomOutCursor;
			zoomOutCursor = createCustomCursor("CursorZoomout",size,"ZoomOut",6,6);
			return zoomOutCursor;
		case MOVE_ENDPT :
			if (moveEndPtCursor != null) return moveEndPtCursor;
			moveEndPtCursor = createCustomCursor("CursorMoveEndPoint",size,"MoveEndPoint",3,3);
			return moveEndPtCursor;
		case MOVE_ENDPT_FROM_CENTER :
			if (moveEndPtFromCenterCursor != null) return moveEndPtFromCenterCursor;
			moveEndPtFromCenterCursor = createCustomCursor("CursorMoveEndPointFromCenter",size,"MoveEndPointFromCenter",8,8);
			return moveEndPtFromCenterCursor;
		case ADD_ENDPT :
			if (addEndPtCursor != null) return addEndPtCursor;
			addEndPtCursor = createCustomCursor("CursorAddEndPoint",size,"AddEndPoint",3,3);
			return addEndPtCursor;
		case REMOVE_ENDPT :
			if (removeEndPtCursor != null) return removeEndPtCursor;
			removeEndPtCursor = createCustomCursor("CursorRemoveEndPoint",size,"RemoveEndPoint",3,3);
			return removeEndPtCursor;
		case INVALID :
			if (invalidCursor != null) return invalidCursor;
			invalidCursor = createCustomCursor("CursorInvalid",size,"Invalid",8,8);
			return invalidCursor;
		default :
			return Cursor.getPredefinedCursor(type);
		}
	}
	
	/**
	 * @return a custom cursor, whose image is build from 
	 * <code>/jpicedt/graphic/toolkit/cursor/ + imageFileName + size + .gif</code>
	 */
	private Cursor createCustomCursor(String imageFileName, String size, String cursorName, int hotSpotX, int hotSpotY){

		// custom cursor construction :
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Image image = toolkit.getImage(getClass().getResource("/jpicedt/graphic/toolkit/cursor/" + imageFileName + size + ".gif"));
		return toolkit.createCustomCursor(image, new Point(hotSpotX,hotSpotY), cursorName);
	}


} // PECursor
