/*
 MouseTransform.java - February 24, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.toolkit;

import jpicedt.graphic.model.Element;
import jpicedt.graphic.event.PEMouseEvent;

import java.awt.*;
import java.awt.geom.*;

/**
 * an interface that specifies behaviours shared by mouse-driven object-editing actions, especially
 * when driven by <code>SelectionTool</code>. 
 * Sequence order is :
 * mouse-press : start()
 * mouse-dragged : process()
 * mouse-released : next() ? no => end()
 * mouse-moved : process()
 * mouse-pressed : next() ? no => end()
 * etc... 
 * @since jpicedt 1.3.2
 */
public interface MouseTransform {

	/**
	 * Called when the mouse is pressed. The transform should do the initialization work here.
	 * @since jpicedt 1.3.2
	 */
	public void start(PEMouseEvent e);
	
    /**
     * Called when the mouse is dragged/moved after the first mouse-pressed event. 
	 * Element geometry update should occur here.
	 * @since jpicedt 1.3.2
     */
    public void process(PEMouseEvent e);
	
	/**
	 * Called when the left mouse-button changes state.
	 * @return true if there's another task in the sequence, false if mouse-transform has completed with this mouse-event.
	 */
	public boolean next(PEMouseEvent e);
	
	/**
	 * Return the cursor for this MouseTransform
	 * @since jpicedt 1.3.2
	 */
	public Cursor getCursor();

	/**
	 * Allows the MouseTransform to do specific graphic rendering when it's operating.
	 * @since jpicedt 1.3.2
	 */
	public void paint(Graphics2D g, Rectangle2D allocation, double scale);
	
	/**
	 * Return a help-message for the UI, that makes sense with this transform.
	 */
	public String getHelpMessage();
	
	
} // MouseTransform
