/*
 MoveTool.java - February 28, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.toolkit;

import jpicedt.graphic.event.*;
import jpicedt.graphic.*;
import jpicedt.graphic.model.Element;

import java.awt.*;
import java.awt.geom.*;


/**
 * A MouseTool used to translate a target Element by a given vector. 
 * @since jPicEdt 1.3
 * @author Sylvain Reynal
 * @version $Id: MoveTool.java,v 1.8.2.1 2007/09/02 11:56:28 reynal Exp $
 */
public class MoveTool extends MouseTool {

	private EditorKit kit;
	private Line2D.Double vec;
	private CursorFactory cursorFactory = new CursorFactory();
	private Element target;
	private PicPoint lastDragPoint = new PicPoint();

	
	/**
	 * @param kit the EditorKit this mouse-tool is to registered with.
	 * @param target the target Element to be moved
	 */
	public MoveTool(EditorKit kit, Element target){
		this.kit = kit;
		this.target = target;
	}
	
	/** set the first point of the translation vector */
	public void mousePressed(PEMouseEvent e){
		super.mousePressed(e);
		if (e.isPopupTrigger())  {
			kit.setCurrentMouseTool(EditorKit.SELECT);
			return;	
		}
		e.getCanvas().beginUndoableUpdate(jpicedt.Localizer.currentLocalizer().get("action.editorkit.Translate.tooltip"));
		PicPoint pt = e.getPicPoint();
		e.getCanvas().getGrid().nearestNeighbour(pt,pt); // pt = nn(pt)
		vec = new Line2D.Double(pt.x,pt.y,pt.x,pt.y);
		lastDragPoint.x = pt.x;
		lastDragPoint.y = pt.y;
	}

	/** set the 2nd pt of the translation vector and translate the target */
	public void mouseDragged(PEMouseEvent e){
		super.mouseDragged(e);
		PicPoint pt = e.getPicPoint();
		e.getCanvas().getGrid().nearestNeighbour(pt,pt); // pt = nn(pt)
		if (vec.x2 == pt.x && vec.y2 == pt.y) return;
		vec.x2 = pt.x;
		vec.y2 = pt.y;
		//e.getCanvas().repaintFromModelRect(vec.getBounds2D());
		e.getCanvas().repaint(); // [pending]
		target.translate(vec.x2-lastDragPoint.x,vec.y2-lastDragPoint.y);
		
		lastDragPoint.x = pt.x;
		lastDragPoint.y = pt.y;
	}


	/** */	
	public void mouseReleased(PEMouseEvent e){
		super.mouseReleased(e);
		if (e.isPopupTrigger())  return;
		vec = null;
		e.getCanvas().endUndoableUpdate();
		e.getCanvas().repaint();
	}

	/** set the cursor */
	public void mouseMoved(PEMouseEvent e){
		super.mouseMoved(e);
		kit.postHelpMessage("help-message.MoveTransform");
		e.getCanvas().setCursor(cursorFactory.getPECursor(CursorFactory.MOVE));
	}

	/**
	 * Allow the tool to paint shapes that are specific to this tool. 
	 */
	public void paint(Graphics2D g, Rectangle2D allocation, double scale){
		if (vec == null) return;
		g.setPaint(Color.blue);
		float[] dash = {1.0f,1.0f};
		g.setStroke(new BasicStroke((float)(1.0/scale),BasicStroke.CAP_ROUND,
		BasicStroke.JOIN_ROUND,10.0f,dash,0.5f));
		g.draw(vec);
	}
	
} // class 

