/*
 PEToggleAction.java - January 2, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.graphic.toolkit;

import jpicedt.graphic.*;
import jpicedt.graphic.model.*;

import javax.swing.*;
import java.util.*;


/**
 * A subclass of PEAction suited for toggle-able AbstractButton's, e.g. JToggleButton,
 * JCheckBox, JCheckBoxMenuItem and JRadioButton.<br>
 * The main point is that PEMenu and PEToolBar know how to add a PEToggleAction and 
 * to create the adequate widget (either JCheckBoxMenuItem or JToggleButton).<br>
 * Concrete implementation of the <code>actionPerformed</method> should use 
 * <code>((AbstractButton)e.getSource()).isSelected()</code> to fetch the current widget state.
 * @since jPicEdt
 * @author Sylvain Reynal
 */
public abstract class PEToggleAction extends PEAction {

	/**
	 * @param actionDispatcher the ActionDispatcher that provide a target when this Action gets invoked.
	 * @param actionName the actionName of this Action that serve to build this Action's properties
	 * @param localizer (can be null) the ActionLocalizer that feeds this Action with localized properties ;
	 *        if null, the Action's NAME property is set to actionName, but other 
	 *        properties are not set (this allow a subclass to set properties manually).
	 * @since jPicEdt 1.3.2
	 */
	public PEToggleAction(ActionDispatcher actionDispatcher, String actionName, ActionLocalizer localizer){
		super(actionDispatcher,actionName,localizer);
	}
} // class

