
/*
 LAFManager.java - December 31, 2001 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.ui;

import jpicedt.JPicEdt;

import javax.swing.*;
import java.util.*;

//import com.l2fprod.gui.plaf.skin.*; // doesn't work at the moment; needs UIViewport to work properly

/**
 * LAF management : maintains a list of installed LAF, and handles LAF loading.
 */
public class LAFManager {

	/** key used to fetch the LAF name from the Properties (see init) */
	public static final String KEY_LAF = "ui.look-and-feel";
	
	/**
	 * @return a string representing current LAF name
	 */
	public static String getCurrentLafName(){
		String s = UIManager.getLookAndFeel().getName();
		if (s.startsWith("Mac OS X")) // bug fix: s="Mac OS X Aqua", while "Mac OS X" is expected since this the value returned by getInstalledLookAndFeels()
			return "Mac OS X";
		else
			return s;
	}

	/**
	 * @return a string representing the default LAF (ie "Metal") name
	 */
	public static String getDefaultLafName(){
		String defaultLafClassName = UIManager.getCrossPlatformLookAndFeelClassName();
		UIManager.LookAndFeelInfo[] lafInfo = UIManager.getInstalledLookAndFeels();
		for (int i=0; i<lafInfo.length; i++){
			if (lafInfo[i].getClassName().equals(defaultLafClassName))
				return lafInfo[i].getName();
		}
		return " ";
	}

	/**
	 * @return a list of installed LAF names (as returned by LookAndFeelInfo.getName())
	 *         suitable for a menu or JComboBox, e.g. "Metal,CDE/Motif,Windows"
	 */
	public static String[] getInstalledLookAndFeelsNames(){
		UIManager.LookAndFeelInfo[] lafInfo = UIManager.getInstalledLookAndFeels();
		String[] lafNames = new String[lafInfo.length];
		for (int i=0; i<lafNames.length; i++){
			lafNames[i] = lafInfo[i].getName();
		}
		return lafNames;
	}
	
	/**
	 * Init LAF from the given Properties using key=KEY_LAF and value = LAF name (as returned
	 * by LookAndFeelInfo.getName()).
	 */
	public static void updateLaf(){
		String lafName = JPicEdt.getPreferences().getProperty(KEY_LAF);
		if (lafName != null) updateLaf(lafName);
		// else updateLafFromClassName(UIManager.getCrossPlatformLookAndFeelClassName());
		// useless since "Metal" is the default
	}
	
	/**
	 * set current LAF from the given LAF name
	 * @param name one of the predefined LAF names, e.g. "CDE/Motif" or "Metal"
	 */
	public static void updateLaf(String lafName){
		//jpicedt.Log.debug(new LAFManager(),"updateLaf","lafName="+lafName);
		if (lafName.equals(getCurrentLafName())) return;  // unchanged
		UIManager.LookAndFeelInfo[] lafInfo = UIManager.getInstalledLookAndFeels();
		for (int i=0; i<lafInfo.length; i++){
			if (lafName.equals(lafInfo[i].getName())){
				updateLafFromClassName(lafInfo[i].getClassName());
				return;
			}
		}
		//jpicedt.Log.debug(new LAFManager(),"updateLaf","no match !");
	}

	/**
	 * Set the LAF from the given class name.
	 */
	public static void updateLafFromClassName(String className) {
		//jpicedt.Log.debug(new LAFManager(),"updateLafFromClassName","className="+className);
		try  {
			UIManager.setLookAndFeel(className);
		}
		catch ( UnsupportedLookAndFeelException ue ) {
			System.err.println("An error occured during Look'n Feel loading: this Look'n Feel is probably not supported on this platform... (this may occur due to License Agreement limitation)"
			                   + "\nIf you are convinced this isn't the case, then please send a bug report including the following stack trace to reynal@ensea.fr"
			                   + "\nStack trace dump starts here :");
			ue.printStackTrace(); // to System.err
			System.err.println("Stack trace ends here");
			JOptionPane.showMessageDialog(null,
			                              ue.getMessage(), // [todo] add a localized error description
			                              jpicedt.Localizer.currentLocalizer().get("LookAndFeel"),
			                              JOptionPane.ERROR_MESSAGE);
		}
		catch ( Exception ue ) {
			System.err.println("An error occured during Look'n Feel loading: classes for this Look'n Feel weren't found..."
			                   + "\nIf you are convinced this isn't the case, then please send a bug report including the following stack trace to reynal@ensea.fr"
			                   + "\nStack trace dump starts here :");
			ue.printStackTrace(); // to System.err
			System.err.println("Stack trace ends here");
			JOptionPane.showMessageDialog(null,
			                              ue.getMessage(), // [todo] add a localized error description
			                              jpicedt.Localizer.currentLocalizer().get("LookAndFeel"),
			                              JOptionPane.ERROR_MESSAGE);
		}
		
	}

} // LAFManager
