/*
 DirectoriesPanel.java - December 31, 2001 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.ui.dialog;

import jpicedt.JPicEdt;
import jpicedt.Localizer;
import jpicedt.graphic.toolkit.AbstractCustomizer;
import jpicedt.ui.dialog.PEFileChooser; // needed by directories browsers
import jpicedt.ui.util.BeanShell;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * A panel for editing default directories, e.g. for saving/loading pictures, fragments or running scripts.
 * @author Sylvain Reynal
 * @since jPicEdt 1.3
 * @version $Id: DirectoriesCustomizer.java,v 1.10.2.1 2007/09/02 11:56:30 reynal Exp $
 */
public class DirectoriesCustomizer extends AbstractCustomizer implements ActionListener {

	private JTextField fileDirectoryTF;
	private JButton buttonDirectoryBrowse;

	private JTextField tmpDirectoryTF;
	private JButton buttonTmpdirBrowse;

	private Properties preferences;
	
	/**
	 * Construct a new panel for editing directories.
	 */
	public DirectoriesCustomizer(Properties preferences){
		
		this.preferences  = preferences;
		
		setBorder(BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(), Localizer.currentLocalizer().get("preferences.Directories")));

		Box box = new Box(BoxLayout.Y_AXIS);
		JPanel p;
		JLabel l;

		/* Sub-Panel 1 : open-file default dir */
		p = new JPanel(); // horizontal flowlayout
		p.add(l=new JLabel(" "+Localizer.currentLocalizer().get("preferences.OpenFileDirectory.label")+" :"));
			l.setToolTipText(Localizer.currentLocalizer().get("preferences.OpenFileDirectory.tooltip"));
		p.add(buttonDirectoryBrowse= new JButton(Localizer.currentLocalizer().get("button.Browse")));
			buttonDirectoryBrowse.addActionListener(this);
		box.add(p);

		box.add(fileDirectoryTF=new JTextField(20));

		/* Sub-Panel 2 : user tmp dir */
		p = new JPanel();
		p.add(l=new JLabel(" "+Localizer.currentLocalizer().get("preferences.TmpDirectory.label")+" :"));
			l.setToolTipText(Localizer.currentLocalizer().get("preferences.TmpDirectory.tooltip"));
		p.add(buttonTmpdirBrowse= new JButton(Localizer.currentLocalizer().get("button.Browse")));
			buttonTmpdirBrowse.addActionListener(this);
		box.add(p);

		box.add(tmpDirectoryTF=new JTextField(20));

		//////////////////		
		add(box, BorderLayout.NORTH);

	}

	/**
	 * Load widgets display content with default values fetched from the PEFileChooser class. 
	 */
	public void loadDefault() {
		fileDirectoryTF.setText(PEFileChooser.DEFAULT_DIRECTORY);
		tmpDirectoryTF.setText(JPicEdt.DEFAULT_TMPDIR);
	}

	/**
	 * Load widgets value from the Options class.
	 */
	public void load() {
		fileDirectoryTF.setText(preferences.getProperty(PEFileChooser.KEY_FILE_DIRECTORY,PEFileChooser.DEFAULT_DIRECTORY));
		tmpDirectoryTF.setText(JPicEdt.getTmpDir().getPath());
	}

	/**
	 * Store current widgets value, presumably to the Properties object given in the constructor
	 */
	 public void store(){
		preferences.setProperty(PEFileChooser.KEY_FILE_DIRECTORY, fileDirectoryTF.getText());
		JPicEdt.setTmpDir(tmpDirectoryTF.getText());
	}

	/**
	 * @return the panel title, used e.g. for Border or Tabpane title. 
	 */
	public String getTitle(){ 
		return Localizer.currentLocalizer().get("preferences.Directories");	
	}
	
	
	/**
	 * @return the Icon associated with this panel, used e.g. for TabbedPane decoration 
	 */
	public Icon getIcon(){ 
		return null;	
	}
	
	
	/**
	 * @return the tooltip string associated with this panel 
	 */
	public String getTooltip(){ 
		return Localizer.currentLocalizer().get("preferences.DefaultDirectories");	
	}

	/**
	 * Open a directory browser when an action occurs on the "Browse" button.
	 */
	public void actionPerformed(ActionEvent e) {

		if (e.getSource() == buttonDirectoryBrowse) {
			String s = PEFileChooser.getFileName(PEFileChooser.BROWSEFILEDIR,null);
			if (s!= null) fileDirectoryTF.setText(s);
		}
		else if (e.getSource() == buttonTmpdirBrowse)  {
			JFileChooser fileChooser = new JFileChooser(tmpDirectoryTF.getText());
			fileChooser.setDialogTitle(Localizer.currentLocalizer().get("preferences.TmpDirectory.label"));
			fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			if (fileChooser.showDialog(null, Localizer.currentLocalizer().get("misc.SelectDirectory")) == JFileChooser.CANCEL_OPTION) return;
			String s =  fileChooser.getSelectedFile().toString();
			if (s!= null) tmpDirectoryTF.setText(s);
		}
	}

} // class


