/*
 GridZoomPanel.java - December 31, 2001 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.ui.dialog;

import jpicedt.Localizer;
import jpicedt.graphic.toolkit.AbstractCustomizer;
import jpicedt.graphic.grid.Grid;
import jpicedt.graphic.PECanvas;
import jpicedt.graphic.PEToolKit;

import java.awt.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * Grid and zoom preferences editing.<br>
 * @author Sylvain Reynal
 * @since jPicEdt 1.3
 * @version $Id: GridZoomCustomizer.java,v 1.8.2.1 2007/09/02 11:56:30 reynal Exp $
 *
 */
public class GridZoomCustomizer extends AbstractCustomizer {

	private JComboBox snapStepList, gridStyleList, gridStepList; 
	private JCheckBox newGridIsOnCB, newSnapIsOnCB;
	private JComboBox zoomFactorList; 
	private Properties preferences;
	private	NumberFormat formatPercent = NumberFormat.getPercentInstance(Locale.US); // for zoom

	//private HistoryTextField snapStepHTF;

	/**
	 * Construct a new panel for grid/zoom preferences editing.
	 */
	public GridZoomCustomizer(Properties preferences) {

		this.preferences = preferences;
		JPanel p;
		JLabel l;

		/* Sub-Panel 1 : grid step, grid on/off state and snap on/off state on startup */
		p = new JPanel(new GridLayout(6,2,5,5));
		p.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
		            Localizer.currentLocalizer().get("preferences.ParametersOnStartup")));

		// grid visible
		l = new JLabel(" "+Localizer.currentLocalizer().get("preferences.NewGridIsOn.label")+" :");
		l.setToolTipText(Localizer.currentLocalizer().get("preferences.NewGridIsOn.tooltip"));
		p.add(l);
		newGridIsOnCB = new JCheckBox();
		p.add(newGridIsOnCB);

		// grid-step
		l = new JLabel(" "+Localizer.currentLocalizer().get("preferences.GridStep.label")+"  (mm) :");
		l.setToolTipText(Localizer.currentLocalizer().get("preferences.GridStep.tooltip"));
		p.add(l);
		gridStepList = new JComboBox();
		gridStepList.setEditable(true);
 		for(int i=0; i<Grid.PREDEFINED_GRID_STEPS.length; i++){
 			gridStepList.addItem(new Double(Grid.PREDEFINED_GRID_STEPS[i])); // Double's
 		}
		p.add(gridStepList);

		// grid style (solid or dash or ...)
		l = new JLabel(" "+Localizer.currentLocalizer().get("preferences.GridStyle.label")+" :");
		l.setToolTipText(Localizer.currentLocalizer().get("preferences.GridStyle.tooltip"));
		p.add(l);
		gridStyleList = new JComboBox();
 		for(int i=0; i<Grid.PREDEFINED_STYLES.length; i++){
 			gridStyleList.addItem(Grid.PREDEFINED_STYLES[i]); 
 		}
		p.add(gridStyleList);

		// snap on
		l = new JLabel(" "+Localizer.currentLocalizer().get("preferences.NewSnapIsOn.label")+" :");
		l.setToolTipText(Localizer.currentLocalizer().get("preferences.NewSnapIsOn.tooltip"));
		p.add(l);
		newSnapIsOnCB = new JCheckBox();
		p.add(newSnapIsOnCB);

		// snap-step
		l = new JLabel(" "+Localizer.currentLocalizer().get("preferences.SnapStep.label")+"  (mm) :");
		l.setToolTipText(Localizer.currentLocalizer().get("preferences.SnapStep.tooltip"));
		p.add(l);
		snapStepList = new JComboBox();
		snapStepList.setEditable(true);
 		for(int i=0; i<Grid.PREDEFINED_SNAP_STEPS.length; i++){
 			snapStepList.addItem(new Double(Grid.PREDEFINED_SNAP_STEPS[i])); // Double's
 		}
		p.add(snapStepList);

		// zoom factor (display using the percent sign)
		l = new JLabel(" "+Localizer.currentLocalizer().get("preferences.ZoomFactor.label")+" :");
		l.setToolTipText(Localizer.currentLocalizer().get("preferences.ZoomFactor.tooltip"));
		p.add(l);
		zoomFactorList = new JComboBox();
		zoomFactorList.setEditable(true);
 		for(int i=0; i<PECanvas.PREDEFINED_ZOOMS.length; i++){
 			zoomFactorList.addItem(formatPercent.format(PECanvas.PREDEFINED_ZOOMS[i])); // Strings
 		}
		p.add(zoomFactorList);

// 		l = new JLabel("test history");
// 		p.add(l);
// 		snapStepHTF = new HistoryTextField("grid",true,true);
// 		HistoryModel hm = snapStepHTF.getModel();
// 		for(int i=0; i<Grid.PREDEFINED_SNAP_STEPS.length; i++){
// 			hm.addItem(Double.toString(Grid.PREDEFINED_SNAP_STEPS[i]));
// 		}
// 		p.add(snapStepHTF);

		add(p,BorderLayout.NORTH);
	}

	/**
	 * Returns the panel title, used e.g. for Border or Tabpane title. 
	 */
	public String getTitle(){ 
		return Localizer.currentLocalizer().get("preferences.Grid")+"/"+Localizer.currentLocalizer().get("preferences.Zoom");	
	}


	/**
	 * Returns the Icon associated with this panel, used e.g. for TabbedPane decoration 
	 */
	public Icon getIcon(){ 
		return null;	
	}


	/**
	 * Returns the tooltip string associated with this panel 
	 */
	public String getTooltip(){ 
		return Localizer.currentLocalizer().get("preferences.GridZoomParameters");	
	}

	/**
	 * Load widgets display content with a default value retrieved from the Options class. 
	 */
	public void loadDefault() {

		snapStepList.setSelectedIndex(Grid.getSnapStepIndex(Grid.snapStepDEFAULT)); // [pending] test if -1
		gridStepList.setSelectedIndex(Grid.getSnapStepIndex(Grid.gridStepDEFAULT)); // [pending] test if -1
		newGridIsOnCB.setSelected(Grid.isVisibleDEFAULT);
		newSnapIsOnCB.setSelected(Grid.snapOnDEFAULT);
		zoomFactorList.setSelectedIndex(PECanvas.getZoomIndex(PECanvas.ZOOM_DEFAULT));  // [pending] test if -1
		gridStyleList.setSelectedItem(Grid.lineStyleDEFAULT);
	}

	/**
	 * Load widgets value from the Properties object
	 */
	public void load() {
		boolean b;
		int i;
		String s;
		
		// snap step
		Double snap = new Double(jpicedt.MiscUtilities.parseProperty(preferences, Grid.KEY_SNAP_STEP,Grid.snapStepDEFAULT));
		for (i=0,b=false; i<snapStepList.getItemCount(); i++){
			if (snapStepList.getItemAt(i).equals(snap)) {
				snapStepList.setSelectedIndex(i); b=true; 
				break;
			}
		}
		if (b==false) {
			snapStepList.addItem(snap);
			snapStepList.setSelectedIndex(snapStepList.getItemCount()-1);
		}
		
		// grid visible
		b = jpicedt.MiscUtilities.parseProperty(preferences, Grid.KEY_VISIBLE, Grid.isVisibleDEFAULT);
		newGridIsOnCB.setSelected(b);
		
		// grid style
		s = preferences.getProperty(Grid.KEY_LINE_STYLE, Grid.lineStyleDEFAULT);
		gridStyleList.setSelectedItem(s);

		// grid step
		Double grid = new Double(jpicedt.MiscUtilities.parseProperty(preferences, Grid.KEY_GRID_STEP,Grid.gridStepDEFAULT));
		for (i=0,b=false; i<gridStepList.getItemCount(); i++){
			if (gridStepList.getItemAt(i).equals(grid)) {
				gridStepList.setSelectedIndex(i); b=true; 
				break;
			}
		}
		if (b==false) {
			gridStepList.addItem(grid);
			gridStepList.setSelectedIndex(gridStepList.getItemCount()-1);
		}
		
		// snap on
		b = jpicedt.MiscUtilities.parseProperty(preferences, Grid.KEY_SNAP_ON, Grid.snapOnDEFAULT);
		newSnapIsOnCB.setSelected(b);
		
		// zoom
		double zoom = jpicedt.MiscUtilities.parseProperty(preferences,PECanvas.KEY_ZOOM,PECanvas.ZOOM_DEFAULT);
		s = formatPercent.format(zoom); // display in "%" format
		for (i=0,b=false; i<zoomFactorList.getItemCount(); i++){
			if (zoomFactorList.getItemAt(i).equals(s)) { // ComboBox was built from e.g. "200%" Strings
				zoomFactorList.setSelectedIndex(i); b=true; 
				break;
			}
		}
		if (b==false) {
			zoomFactorList.addItem(s);
			zoomFactorList.setSelectedIndex(zoomFactorList.getItemCount()-1);
		}
	}
	
	/**
	 * Store current widgets value, presumably to a file or to a dedicated storage class
	 */
	 public void store(){

		preferences.setProperty(Grid.KEY_SNAP_STEP,snapStepList.getSelectedItem().toString());
		preferences.setProperty(Grid.KEY_GRID_STEP,gridStepList.getSelectedItem().toString());
		preferences.setProperty(Grid.KEY_VISIBLE, new Boolean(newGridIsOnCB.isSelected()).toString());
		preferences.setProperty(Grid.KEY_SNAP_ON, new Boolean(newSnapIsOnCB.isSelected()).toString());
		preferences.setProperty(Grid.KEY_LINE_STYLE, gridStyleList.getSelectedItem().toString());
		String s = (String)zoomFactorList.getSelectedItem();
		if (s.indexOf("%")==-1) { // if zoom doesn't contain a "%" sign, format it to percent.
			// this is how it's stored in the preferences file
			s = formatPercent.format(new Double(s));
		}
		preferences.setProperty(PECanvas.KEY_ZOOM,s);
	}
} // PanelGridZoom
