/*
 JFrameMDIManager.java - January 2, 2005 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.ui.internal;

import jpicedt.*;

import jpicedt.graphic.*;
import jpicedt.graphic.event.*;
import jpicedt.graphic.toolkit.*;

import jpicedt.ui.*;
import jpicedt.ui.util.*;
import jpicedt.ui.action.*;
import jpicedt.ui.dialog.*;

import jpicedt.widgets.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;
import javax.swing.undo.*;
import javax.swing.event.*;

/**
 * An MDI Manager implementation for JFrame's
 * @since jPicEdt 1.4
 * @author Sylvain Reynal
 * @version $Id: JFrameMDIManager.java,v 1.3.2.1 2007/09/02 11:56:30 reynal Exp $
 */
public class JFrameMDIManager extends MDIManager {

	private PopupMenuFactory popupMenuFactory;
	private JFrameEventHandler jFrameHandler; // window listener

	public String getName(){
		return MDI_STANDALONEFRAMES;
	}

	/**
	 * @param progressBar The progress bar to inform of progress in the initialization ; can be null.
	 */
	public JFrameMDIManager(PEProgressBar progressBar){

		// init LAF
		jpicedt.ui.LAFManager.updateLaf();

		// add a PropertyChangeListener to the UIManager, so that
		// every change in LAF leads to an immediate repainting of
		// the whole component tree
		UIManager.addPropertyChangeListener(new PropertyChangeListener(){
			                                    public void propertyChange(PropertyChangeEvent e) {
				                                    if (e.getPropertyName().equals("lookAndFeel")) {
														Frame[] allFrames = Frame.getFrames();
														for (int i=0; i<allFrames.length; i++){
															Frame f = allFrames[i];
															if (!(f instanceof JFrame)) continue;
															JFrame jf = (JFrame)f;
															if (jf.isVisible()){
																JComponent c = (JComponent)jf.getRootPane();
																SwingUtilities.updateComponentTreeUI(c);
																c.invalidate();	c.validate(); c.repaint();
															}
														}
				                                    }}});

		// init the application main frame
		/**
		mainFrame = new JFrame("jPicEdt "+Version.getVersion());
		mainFrame.setResizable(false);
		mainFrame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		mainFrame.getContentPane().setLayout(new BorderLayout());
		// allows JOptionPane to be instanciated with an implicit root frame
		JOptionPane.setRootFrame(mainFrame);
		*/

		jFrameHandler = new JFrameEventHandler();
		
		/*
		mainFrame.addWindowListener(new WindowAdapter(){
			                            public void windowClosing(WindowEvent e){
				                            new jpicedt.JPicEdt.ExitAction().actionPerformed(null);
			                            }
		                            });
		mainFrame.validate();
		// size and location on the screen :
		mainFrame.setLocation(0,0);
		*/

		// init colours from pref.
		update();
		if (progressBar != null) {
			progressBar.increment("GUI created successfully !");
		}

		// setVisible() and/or pack() "realizes" the main JFrame (i.e. create new Swing thread)
		/*
		mainFrame.pack();
		mainFrame.setVisible(true);
		*/
	}


	/**
	 * Factory method that creates an MDIComponent which acts as a UI delegate to the given pane.
	 * This implementation returns a properly init'd JFrame, with the given pane
	 * added to its content-pane.
	 */
	public MDIComponent createMDIComponent(JPanel pane){
		return new JFrameMDIComponent(pane);
	}

	/**
	 * Add the given component to the desktop
	 */
	public void addMDIComponent(MDIComponent c){
		//if (!(c instanceof JFrameMDIComponent)) return;
		JPanel innerPane = c.getInnerPane();
		if (innerPane instanceof PEDrawingBoard) {
			JFrame jf = (JFrame)c;
			try {
				jf.addWindowListener(jFrameHandler);
				jf.setJMenuBar(new PEMenuBar());
				jf.getContentPane().add(new PEToolBar(), BorderLayout.NORTH);
			}
			catch(MissingResourceException e){
				System.err.println("During GUI initialization, I was unable to find internationalization resource: " + e.getKey());
				System.err.println("JVM stack trace dump :");
				e.printStackTrace();
				System.err.println("In any case, you may need to check the integrity of the jpicedt.jar file you downloaded.");
				System.err.println("If it still doesn't work, please send a bug report to : syd@jpicedt.org, with this console dump included.");
				System.err.println(":-((((((((((  Exiting... sorry !");
				SystemOutUtilities.instance().redirect(SystemOutUtilities.STANDARD); // close any open File Stream
				System.exit(0);
			}
			catch (Exception e) {
				System.err.println("OK, there SURE is a bug somewhere... Thank you for sending a bug report including the following stack trace to syd@jpicedt.org");
				System.err.println("Stack trace dump starts here :");
				e.printStackTrace();
				System.err.println(":-((((((((((  Exiting...sorry !");
				SystemOutUtilities.instance().redirect(SystemOutUtilities.STANDARD);// close any open File Stream
				System.exit(0);
			}
		}
	}

	public void closeBoard(PEDrawingBoard board){
		JFrameMDIComponent mdiComp = (JFrameMDIComponent)getHostingContainer(board);
		if (mdiComp==null) return;
		// simulate a click on the "close" button, it's safe because it's gives us a unified way of closing boards
		WindowEvent we = new WindowEvent(mdiComp,WindowEvent.WINDOW_CLOSING);
		jFrameHandler.windowClosing(we);
	}

	/**
	 * Returns the dimension of the desktop that hosts the components of this MDIManager.
	 * @return the screen size
	 */
	public Dimension getDesktopSize(){
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		return screenSize;
	}

	/**
	 * Saves UI geometry to JPicEdt's preferences.
	 * @see #InternalFrameMDIManager
	 * @see #addDockablePanel
	 * @see DockablePanel#setVisible
	 */
	public void saveGeometry(){
		// save desktop geometry :
		/* (from InternalFrameMDIManager)
		Properties preferences = JPicEdt.getPreferences();
		preferences.setProperty(KEY_GEOMETRY_X,Integer.toString(mainFrame.getX()));
		preferences.setProperty(KEY_GEOMETRY_Y,Integer.toString(mainFrame.getY()));
		preferences.setProperty(KEY_GEOMETRY_WIDTH,Integer.toString(mainFrame.getWidth()));
		preferences.setProperty(KEY_GEOMETRY_HEIGHT,Integer.toString(mainFrame.getHeight()));
		*/
		super.saveGeometry(); // save DockablePanel's geometry
	}

	/**
	 * Updates local properties from JPicEdt's preferences (e.g. GUI colours, rendering-hints,...)
	 * This implementation calls the superclass's method, then update the desktop-pane colour.
	 */
	public void update(){
		//desktopPane = null; // test error.log
		// colours :
		/* (from InternalFrameMDIManager)
		Color desktopColor = JPicEdt.getProperty(KEY_DESKTOP_COLOR,desktopColorDEFAULT);
		desktopPane.setBackground(desktopColor);
		if (mainFrame.isVisible()){
			desktopPane.repaint();
		}
		*/
		super.update(); // update RenderingHints for each PEDrawingBoard
	}



	//////////////////////////////////////////////////////////////////////
	//// GUI COMPONENTS
	//////////////////////////////////////////////////////////////////////

	/**
	 * Update JMenuItem's accelerators from the key/value pairs stored in JPicEdt's preferences 
	 */
	public void updateAccelerators(){
		PEDrawingBoard[] allBoards = getAllDrawingBoards();
		for (int i=0; i<allBoards.length; i++){
			JFrame f = (JFrame)getHostingContainer(allBoards[i]);
			PEMenuBar bar = (PEMenuBar)f.getJMenuBar();
			if (bar != null) bar.updateAccelerators();
		}
	}

	/**
	 * Update the content (i.e. PEMenu's) of the menu-bar depending on the current state of the MDIManager.
	 */
	 public void updateMenuBar(){
		PEDrawingBoard[] allBoards = getAllDrawingBoards();
		for (int i=0; i<allBoards.length; i++){
			JFrame f = (JFrame)getHostingContainer(allBoards[i]);
			PEMenuBar bar = (PEMenuBar)f.getJMenuBar();
			if (bar != null) bar.updateMenuBar();
		}
	 }
	
	/** 
	 * Updates the "recent files" sub-menu items after a frame was saved or opened.
	 * @since PicEdt 1.1
	 */
	 public void updateRecentFilesSubmenu(){
		PEDrawingBoard[] allBoards = getAllDrawingBoards();
		for (int i=0; i<allBoards.length; i++){
			JFrame f = (JFrame)getHostingContainer(allBoards[i]);
			PEMenuBar bar = (PEMenuBar)f.getJMenuBar();
			if (bar != null) bar.updateRecentFilesSubmenu();
		}
	 }
	
	/** 
	 * Updates the "script" menu items 
	 * @since jpicedt 1.3.2
	 */
	 public void updateScriptsMenu(){
		PEDrawingBoard[] allBoards = getAllDrawingBoards();
		for (int i=0; i<allBoards.length; i++){
			JFrame f = (JFrame)getHostingContainer(allBoards[i]);
			PEMenuBar bar = (PEMenuBar)f.getJMenuBar();
			if (bar != null) bar.updateScriptsMenu();
		}
	 }

	/** 
	 * Updates the "fragments" menu items 
	 * @since jpicedt 1.3.2
	 */
	 public void updateFragmentsMenu(){
		PEDrawingBoard[] allBoards = getAllDrawingBoards();
		for (int i=0; i<allBoards.length; i++){
			JFrame f = (JFrame)getHostingContainer(allBoards[i]);
			PEMenuBar bar = (PEMenuBar)f.getJMenuBar();
			if (bar != null) bar.updateFragmentsMenu();
		}
	 }

	/**
	 * update "undo" and "redo" text
	 * @since jpicedt 1.3.2
	 */
	 public void updateUndoRedoMenus(String undoName,String redoName){
		PEDrawingBoard[] allBoards = getAllDrawingBoards();
		for (int i=0; i<allBoards.length; i++){
			JFrame f = (JFrame)getHostingContainer(allBoards[i]);
			PEMenuBar bar = (PEMenuBar)f.getJMenuBar();
			if (bar != null) bar.updateUndoRedoMenus(undoName, redoName);
		}
	 }

	/** 
	 * Updates the Window menu as soon as an internal frame is opened or closed 
	 * (this menu keeps tracks of every open internal frames so that the user can activate 
	 * them quickly by use of a shortcut)
	 * 
	 * @since PicEdt 1.1
	 */
	 public void updateWindowMenu() throws MissingResourceException {
		PEDrawingBoard[] allBoards = getAllDrawingBoards();
		for (int i=0; i<allBoards.length; i++){
			JFrame f = (JFrame)getHostingContainer(allBoards[i]);
			PEMenuBar bar = (PEMenuBar)f.getJMenuBar();
			if (bar != null) bar.updateWindowMenu();
		}
	 }

	/**
	 * update currently "selected" menuitem in Toolkit menu
	 */
	 public void updateToolkitMenu(PropertyChangeEvent e){
		PEDrawingBoard[] allBoards = getAllDrawingBoards();
		for (int i=0; i<allBoards.length; i++){
			JFrame f = (JFrame)getHostingContainer(allBoards[i]);
			PEMenuBar bar = (PEMenuBar)f.getJMenuBar();
			if (bar != null) bar.propertyChange(e);
		}
	 }
		 
	/**
	 * Return an array containing all the menu-items components attached to menubars
	 */
	 public JMenuItem[] getMenuItems(){
		PEDrawingBoard[] allBoards = getAllDrawingBoards();
		for (int i=0; i<allBoards.length; i++){
			JFrame f = (JFrame)getHostingContainer(allBoards[i]);
			PEMenuBar bar = (PEMenuBar)f.getJMenuBar();
			if (bar != null) return bar.getMenuItems();
		}
		return null;
	 }


	//////////////////////////////////////////////////////////////////////
	//// BOARDS MANAGEMENT
	//////////////////////////////////////////////////////////////////////

	/**
	 * Returns the currently active drawing board ;  
	 * null if there's no frame hosting a PEDrawingBoard in the desktop.<br>
	 */
	public PEDrawingBoard getActiveDrawingBoard(){
		return this.activeBoard;
	}

	/**
	 * selects the given drawing board by bringing it to front.
	 */
	public void selectDrawingBoard(PEDrawingBoard board){

		JFrame iFrame = (JFrame)getHostingContainer(board);
		if (iFrame == null) return;
		iFrame.toFront();
	}

	///////////////////////////////////////////////////////////////////////
	/////// MODAL DIALOG BOXES
	///////////////////////////////////////////////////////////////////////
	/**
	 * [pending] experimental ! 1.3.2-beta9 ; not used yet.
	 */
	public void showModalDialogBox(String title, JComponent innerPane){
		JDialog dlg = new JDialog((JFrame)null, title, true);
		dlg.getContentPane().add(innerPane);
		dlg.setVisible(true); // block I/O
	}

	//////////////////////////////////////////////////////////////////////
	//// GEOMETRY
	//////////////////////////////////////////////////////////////////////

	/**
	 * Cascade all open internal frames
	 */
	public void cascadeDrawingBoards(){

		PEDrawingBoard[] allBoards = getAllDrawingBoards();
		ArrayList nonIconFrames = new ArrayList();

		int counter=0;
		for(int i = 0; i < allBoards.length; i++){
			MDIComponent container = getHostingContainer(allBoards[i]);
			if (!(container instanceof JFrame)) continue;
			JFrame ifr = (JFrame)container;
			if (ifr.getExtendedState()==Frame.ICONIFIED) continue;
			ifr.setExtendedState(Frame.NORMAL); // in case this one was Maximized (may fire a PropertyVetoException)
			ifr.pack();
			//ifr.show(); // already visible !
			//int dist = ifr.getHeight() - ifr.getContentPane().getHeight(); // title bar height
			int dist=30;
			int x = counter * dist;
			if (x + ifr.getWidth() > getDesktopSize().width) x = 0; // wrap around at desktop edge
			int y = counter * dist;
			if (y + ifr.getHeight() > getDesktopSize().height) y = 0; // wrap around at desktop edge
			ifr.setLocation(x,y);
			counter++;
		}
	}

	/**
	 * Tiles all open internal frames horizontally
	 */
	public void tileDrawingBoardsHorizontally(){

		PEDrawingBoard[] allBoards = getAllDrawingBoards();
		ArrayList nonIconFrames = new ArrayList();
		for(int i = 0; i < allBoards.length; i++){
			MDIComponent container = getHostingContainer(allBoards[i]);
			if (!(container instanceof JFrame)) continue;
			JFrame jif = (JFrame)container;
			if (jif.getExtendedState()==Frame.ICONIFIED) {
				jif.setExtendedState(Frame.NORMAL); 
				nonIconFrames.add(jif);
			}
		}
		int nonIconNb = nonIconFrames.size();
		if (nonIconNb == 0) return;

		// number of columns = int(sqrt(N))
		int colNb = (int)Math.floor(Math.sqrt(nonIconNb));
		int colWidth = getDesktopSize().width / colNb;

		int remaining = nonIconNb;

		for(int col = 0; col < colNb; col++){

			int numberOfFrameInThisColumn = (int)Math.floor(remaining / (colNb - col));
			int rowHeight = getDesktopSize().height / numberOfFrameInThisColumn;

			for (int i = 0; i < numberOfFrameInThisColumn; i++){
				JFrame ifr = (JFrame)nonIconFrames.get(nonIconNb-remaining);
				ifr.reshape(col*colWidth, i * rowHeight , colWidth, rowHeight);
				remaining--;
			}
		}
	}

	/**
	 * Tiles all open internal frames vertically.
	 */
	public void tileDrawingBoardsVertically(){

		PEDrawingBoard[] allBoards = getAllDrawingBoards();
		ArrayList nonIconFrames = new ArrayList();
		for(int i = 0; i < allBoards.length; i++){
			MDIComponent container = getHostingContainer(allBoards[i]);
			if (!(container instanceof JFrame)) continue;
			JFrame jif = (JFrame)container;
			if (jif.getExtendedState()==Frame.ICONIFIED) {
				jif.setExtendedState(Frame.NORMAL); 
				nonIconFrames.add(jif);
			}
		}
		int nonIconNb = nonIconFrames.size();
		if (nonIconNb == 0) return;

		// number of row = int(sqrt(N)) where N = number of non-iconified frame
		int rowNb = (int)Math.floor(Math.sqrt(nonIconNb));
		int rowHeight = getDesktopSize().height / rowNb;
		int remaining = nonIconNb;

		for(int rowIndex = 0; rowIndex < rowNb; rowIndex++){

			int numberOfFrameInThisRow = (int)Math.floor(remaining / (rowNb - rowIndex));
			int colWidth = getDesktopSize().width / numberOfFrameInThisRow;

			for (int i = 0; i < numberOfFrameInThisRow; i++){
				JFrame ifr = (JFrame)nonIconFrames.get(nonIconNb-remaining);
				ifr.reshape(i*colWidth, rowIndex * rowHeight , colWidth, rowHeight);
				remaining--;
			}
		}
	}


	////////////////////////////////////////////////////
	/// MDIComponent -> STAND-ALONE FRAME
	/////////////////////////////////////////////////////

	/** simple adapter for JFrame so that they implement the MDIComponent interface */
	private class JFrameMDIComponent extends JFrame implements MDIComponent {

		JPanel pane; // the inner pane, see getInnerPane()

		JFrameMDIComponent(JPanel pane){
			super(""); // no title
			this.getContentPane().add(pane);
			this.pane = pane;
			this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		}
		
		public void setTitle(String title){
			super.setTitle(title);
			//revalidate(); // was revalidate [pending] does it work so for JFrame's ?
			//repaint();
		}

		public JPanel getInnerPane(){
			return pane;
		}

		// all other methods of the MDIComponent interface are already implemented by JFrame...

	}

	////////////////////////////////////////////////////
	/// DESKTOP MANAGER
	/////////////////////////////////////////////////////

	/**
	 * Handler for JFrame window-events. 
	 * @since jPicEdt 1.4
	 */
	private class JFrameEventHandler extends WindowAdapter {
		
		/**
		 * Updates "activeBoard".
		 */
		public void windowActivated(WindowEvent e){
			Window w = e.getWindow();
			if (w instanceof JFrameMDIComponent){
				JFrameMDIComponent f = (JFrameMDIComponent)w;
				//Log.debug(f.getTitle() + " got activated...");
				
				if (f.getInnerPane() instanceof PEDrawingBoard){ 
					PEDrawingBoard board = (PEDrawingBoard)f.getInnerPane();  
					activeBoard = board; 				    
					//actionRegistry.updateActionState();

					PropertyChangeEvent propevt = new PropertyChangeEvent(this,ACTIVE_BOARD_CHANGE, null, board);
					boardEventHandler.propertyChange(propevt);
			
					/* 
					EditorKit kit = board.getCanvas().getEditorKit();
					propevt = new PropertyChangeEvent(kit,EditorKit.EDIT_MODE_CHANGE, null, kit.getCurrentMouseTool());
					boardEventHandler.propertyChange(propevt);
					*/
				}
			}
			
		}
		
		/**
		 * Try to save the content of the board if it's dirty before it gets closed.
		 * Also updates the widget states (menubar and toolbars).
		 * (note that this method is invoked BEFORE JDesktopManager.closeFrame)
		 */
		public void windowClosing(WindowEvent e){
			Window w = e.getWindow();
			if (w instanceof JFrameMDIComponent){
				JFrameMDIComponent f = (JFrameMDIComponent)w;
				//Log.debug(f.getTitle() + " is closing...");
				if (!(f.getInnerPane() instanceof PEDrawingBoard)) return; 
				if (removeDrawingBoard((PEDrawingBoard)f.getInnerPane())){ // try to save content (returns false if cancelled)
					// test if no more open board in which case we create a new empty board
					PEDrawingBoard[] allBoards = getAllDrawingBoards();
					if (allBoards.length==0) JPicEdt.newBoard();

					f.dispose(); 

					if (f.getInnerPane() instanceof PEDrawingBoard){
						activeBoard = activateNextBoard(); // ensure we activate a frame hosting a PEDrawingBoard!
						//if (activeBoard!=null)Log.debugAppendLn("New active board="+activeBoard.getTitle());
						//else Log.debugAppendLn("No active board anymore");
						PropertyChangeEvent propevt = new PropertyChangeEvent(this,ACTIVE_BOARD_CHANGE, null, activeBoard); 
						boardEventHandler.propertyChange(propevt);
					}

				
					// update GUI
					updateWindowMenu(); // it's fine because board was just removed from the hash

					// update Action's :
					JPicEdt.getActionRegistry().updateActionState(); 
					updateMenuBar();
					
				}
			}
		}
		
		private PEDrawingBoard activateNextBoard() { // copied from DefaultDesktopManager with minor changes...
			int i;
			PEDrawingBoard[] allBoards = getAllDrawingBoards();
			for (i = 0; i < allBoards.length; i++) {
				PEDrawingBoard brd = allBoards[i];
				JFrameMDIComponent comp = (JFrameMDIComponent)getHostingContainer(brd);
				comp.toFront();
				return brd;
			}
			return null;
		}
		
		/**
		 * used for debugging purpose
		 */
		private String printAllFrames(){

			Frame[] jif = Frame.getFrames();
			String s = "          Desktop contains : ";
			for (int i = 0; i < jif.length; i++){
				Frame f = jif[i];
				if (f instanceof JFrame){
					s += "["+Integer.toString(i)+"]"+((JFrame)f).getTitle() + "; ";
				}
			}
			return s;
		}

		
	}
	
} // class JFrameMDIManager

