/*
 BeanShell.java - July 30, 2002 - jPicEdt, a picture editor for LaTeX.
 Copyright (C) 1999-2007 Sylvain Reynal

 Département de Physique
 Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
 6, avenue du Ponceau
 F-95014 CERGY CEDEX

 Tel : +33 130 736 245
 Fax : +33 130 736 667
 e-mail : reynal@ensea.fr
 jPicEdt web page : http://www.jpicedt.org/
  
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or any later version.
  
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
  
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jpicedt.ui.util;

import jpicedt.JPicEdt;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import bsh.*;
import bsh.util.*;

/**
 * Wrapper for the BSH interpreter console + some useful static methods for running scripts from inside JPicEdt.
 */
public class BeanShell extends JFrame implements Runnable {

	private Interpreter interpreter; // for use by BeanShell() (i.e. BSH Console)
	
	public static void _main(String[] args){
		JFrame f = new JFrame();
		JButton b = new JButton("console");
		f.getContentPane().add(b);
		b.addActionListener(new ActionListener(){public void actionPerformed(ActionEvent ae){
				BeanShell bsh = new BeanShell();
				//Thread t = new Thread(bsh);t.start();
		}});
		f.addWindowListener(new WindowAdapter(){public void windowClosing(WindowEvent e){System.out.println("Exiting...");System.exit(0);}});
		f.pack();
		f.setVisible(true);
	}
	
	/**
	 * Open a new BSH console, and start the BSH interpreter interactively in it.
	 */
	public BeanShell(){
		
		super("BeanShell Console");
		setSize(400,400);
		
		//System.out.println("Creating console...");
		JConsole console = new JConsole();
		getContentPane().add(console);
		//System.out.println("Creating interpreter...");
		interpreter = new Interpreter(console);	
		//System.out.println("Setting predefined variables and methods...");
		try {
			//interpreter.eval("import jpicedt.*;");
			interpreter.eval(
				"preferences=jpicedt.JPicEdt.getPreferences();" +
				"clipboard=jpicedt.JPicEdt.getClipboard();" +
				"mdimgr=jpicedt.JPicEdt.getMDIManager();" +
				"board(){return jpicedt.JPicEdt.getActiveDrawingBoard();}" +
				"canvas(){return jpicedt.JPicEdt.getActiveCanvas();}" +
				"drawing(){return jpicedt.JPicEdt.getActiveDrawing();}" +
				"editorkit(){return jpicedt.JPicEdt.getActiveEditorKit();}");
		}
		catch (Exception ex){ex.printStackTrace();}
		//System.out.println("Show !");
		setVisible(true);
		//System.out.println("Done !");
		
		//SwingUtilities.invokeLater(this); // [pending] But why the hell does it block ????
		Thread t = new Thread(this);t.start();
	}
	
	/**
	 * Run the BSH interpreter in interactive mode. 
	 */
	public void run(){
		//System.out.println("Starting interpreter...");
		interpreter.run(); // run interactively
	}

	/////////////////////// STATIC ///////////////////////////
	/**
	 * Run a BSH script with the given path.
	 * @param scriptPath absolute script path
	 * @param scriptName script name, for instance "Repeat copy" ; used mainly for undo/redo menus.
	 */
	public static void runScript(String scriptPath, String scriptName){
		if (scriptPath == null) return;
		//System.out.println("Running script : " + scriptPath);
		Interpreter interpreter = new Interpreter(); // no console
		try {
			interpreter.eval(
				"preferences=jpicedt.JPicEdt.getPreferences();" +
				"clipboard=jpicedt.JPicEdt.getClipboard();" +
				"mdimgr=jpicedt.JPicEdt.getMDIManager();" +
				"board(){return jpicedt.JPicEdt.getActiveDrawingBoard();}" +
				"canvas(){return jpicedt.JPicEdt.getActiveCanvas();}" +
				"drawing(){return jpicedt.JPicEdt.getActiveDrawing();}" +
				"editorkit(){return jpicedt.JPicEdt.getActiveEditorKit();}");
			// start new UndoableEdit :
			jpicedt.graphic.PECanvas canvas = jpicedt.JPicEdt.getActiveCanvas();
			if (canvas != null) canvas.beginUndoableUpdate("Script: " + scriptName);
			interpreter.source(scriptPath);
			if (canvas != null) canvas.endUndoableUpdate();
		}
		catch ( TargetError te ) { // The script threw an exception
			JOptionPane.showMessageDialog(null,
				"BeanShell script threw exception :"
				+ "\n\"" + te.getTarget() + "\""
				+ "\nat line " + te.getErrorLineNumber() 
				+ " in file [" + te.getErrorSourceFile() + "]",
				"BeanShell script",
				JOptionPane.ERROR_MESSAGE);
		} 
		catch ( EvalError ee ) { // General Error evaluating script
			JOptionPane.showMessageDialog(null,
				"BeanShell script evaluation error :"
				+ "\n\"" + ee.getErrorText() + "\""
				+ "\nat line " + ee.getErrorLineNumber() 
				+ " in file [" + ee.getErrorSourceFile() + "]",
				"BeanShell script",
				JOptionPane.ERROR_MESSAGE);
		}
		catch(Exception ex){ // Other exception
			JOptionPane.showMessageDialog(null,
				"BeanShell script :\n" + ex.getMessage(),
				"BeanShell script",
				JOptionPane.ERROR_MESSAGE);
		}
	}
	
	public static void main(String[] args){
		JMenu menu = createMenu();
		for (int i=0; i<menu.getMenuComponentCount(); i++){
			System.out.println("item " + i + "=" + menu.getMenuComponent(i));
			System.out.println();
		}
	}
	
	/**
	 * Return a JMenu containing a hierarchy of JMenu's and JMenuItem's built from
	 * the BSH scripts contained, first in the installation script directory, then
	 * in the user script directory.
	 * @return an empty JMenu if no BSH scripts were found in either directory.
	 */
	public static JMenu createMenu(){
		// 1°) scan install script directory :
		String jpicedtHome = jpicedt.MiscUtilities.getJPicEdtHome();
		JMenu installScriptsMenu = null;
		if (jpicedtHome!=null) installScriptsMenu=createMenu(new File(jpicedtHome,"macros"));
		// 2°) scan user script directory :
		JMenu userScriptsMenu = null; 
		if (JPicEdt.getUserSettingsDirectory() != null) 
			userScriptsMenu = createMenu(new File(JPicEdt.getUserSettingsDirectory(),"macros"));
		// build array :
		JMenu menu = new JMenu("Macros");
		if (installScriptsMenu != null) {
			Component[] cc = installScriptsMenu.getMenuComponents();
			for(int i=0; i<cc.length; i++){
				menu.add(cc[i]);
			}
		}
		menu.add(new JSeparator());
		if (userScriptsMenu != null) {
			Component[] cc = userScriptsMenu.getMenuComponents();
			for(int i=0; i<cc.length; i++){
				menu.add(cc[i]);
			}
		}
		return menu;
	}

	/**
	 * Return a JMenu containing a hierarchy of JMenu's and JMenuItem's built from
	 * the BSH scripts contained in the given directory and its children.
	 * @param directory path where to look for .bsh scripts, or subdirectories.
	 * @return null if directory (or one of its children) doesn't exist, or contains no BSH script.
	 */
	private static JMenu createMenu(File directory){
		if (!directory.exists()) return null;
		String[] dirContent = directory.list();
		if (dirContent==null) return null; // directory is empty
		Arrays.sort(dirContent);
		// [pending] sort dir content
		JMenu menu = null; // so that we return null if nothing gets added...
		for (int i=0; i<dirContent.length; i++){
			String fileName = dirContent[i];
			File file = new File(directory, fileName); // create full path name
			// 1°) file is a BSH script :
			if (fileName.toLowerCase().endsWith(".bsh")){
				String label = fileName.substring(0,fileName.indexOf(".bsh")).replace('_',' ');
				JMenuItem menuItem = new JMenuItem(label);
				menuItem.addActionListener(new MenuScriptsAA(file.getPath(), label));
				// [pending] add CTRL+? shortcut
				if (menu==null) menu = new JMenu(directory.getName());
				menu.add(menuItem);
			}
			// 2°) file is a subdirectory :
			else if (file.isDirectory()){
				JMenu subMenu = createMenu(file);
				if (subMenu != null) {
					if (menu==null) menu = new JMenu(directory.getName());
					menu.add(subMenu);
				}
			}
		}
		return menu;
	}
	
	/**
	 * ActionAdapter for Scripts menuitems.
	 */
	private static class MenuScriptsAA implements ActionListener{

		String scriptPath;
		String scriptName;
		MenuScriptsAA(String scriptPath, String scriptName) {
			this.scriptPath = scriptPath;
			this.scriptName = scriptName;
		}
		public void actionPerformed(ActionEvent e) {
			BeanShell.runScript(scriptPath, scriptName);
		}
	}
	
}


