/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.jpicedt.org
*/

package jpicedt.ui.util;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * A RepaintManager that may used for debugging purpose<p>
 * Print out information about adding and/or painting dirty regions<br>
 * How to use it :<br> 
 * - Set DEBUG_PAINTING to TRUE in JPicEdt.java<br>
 * - launch jpicedt with "-redir=standard"<br>
 */
public class DebugRepaintManager extends RepaintManager {

	// set to true to display information concerning jpicedt.* components only.
	private static final boolean ONLY_JPICEDT_COMPONENTS = true;

	/**
 	 * creates a new DebugRepaintManager with double-buffering turned off
	 */
	public DebugRepaintManager(){
		super();
		setDoubleBufferingEnabled(false);
	}
	
	/**
	 * Add a component in the list of components that should be refreshed. 
	 * If c already has a dirty region, the rectangle (x,y,w,h) will be unioned with the region that should be redrawn.
	 */
	public synchronized void addDirtyRegion(JComponent c, int x, int y, int w, int h) {

		if (ONLY_JPICEDT_COMPONENTS){

			if (c.getClass().getName().startsWith("jpicedt.graphic.PECanvas"))
				jpicedt.Log.debug(x+","+y+" "+w+"x"+h);
		}
		else {
			jpicedt.Log.debug(x+","+y+" "+w+"x"+h);
		}
		super.addDirtyRegion(c,x,y,w,h);

	}

	/**
	 * Paint all of the components that have been marked dirty.
	 */
	public void paintDirtyRegions() {

		// Unfortunately most of the RepaintManager state is package
		// private and not accessible from the subclass at the moment,
		// so we can't print more info about what's being painted.
		jpicedt.Log.debug("painting DirtyRegions");
		super.paintDirtyRegions();
	}
}


