/*  jPicEdt, a picture editor for LaTeX.
    Copyright (C) 1999-2007  Sylvain Reynal

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Sylvain Reynal
    Département de Physique
    Ecole Nationale Supérieure de l'Electronique et de ses Applications (ENSEA)
    6, avenue du Ponceau
    95014 CERGY CEDEX
    FRANCE

    Tel : 00 +33 130 736 245
    Fax : 00 +33 130 736 667
    e-mail : reynal@ensea.fr
    jPicEdt web page : http://www.ensea.fr/pub/picedt.html
*/


package jpicedt.widgets;

import javax.swing.*; 
import javax.swing.text.*; 

import java.awt.Toolkit;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;

/**
 * A JtextField that accept only DecimalNumbers
 * @author Sylvain Reynal
 * @since jpicedt 1.3.2
 * @version $Id: DecimalNumberField.java,v 1.5.2.1 2007/09/02 11:56:30 reynal Exp $
 * <p>
 *
 */
public class DecimalNumberField extends JTextField {

    private Toolkit toolkit;
    private NumberFormat doubleFormatter;
    private boolean positiveOnly=false; // >=0 numbers

    /**
     * contructs a new Field with "columns" as the initial number of columns
     */
    public DecimalNumberField(int columns){this(0, columns);}

    /**
     * contructs a new Field with an initial value and an initial number of columns, and
     * the given "positiveOnly" flag
     */
    public DecimalNumberField(double value, int columns, boolean positiveOnly){

		this(value, columns);
		this.positiveOnly = positiveOnly;
    }

    /**
     * constructs a new Field with an initial value and an initial number of columns
     */
    public DecimalNumberField(double value, int columns) {
        super(columns);
        toolkit = Toolkit.getDefaultToolkit();
        doubleFormatter = NumberFormat.getNumberInstance(Locale.US);
		doubleFormatter.setGroupingUsed(false); // no 1,000,000 !
        //doubleFormatter.setParseIntegerOnly(true);
        setValue(value);
    }

    /**
     * @return the field's text parsed as a double
     */
    public double getValue() {
        double retVal = 0;
        try {
            retVal = doubleFormatter.parse(getText()).doubleValue();
        } catch (ParseException e) {
            toolkit.beep();
        }
        return retVal;
    }

    /**
     * set the field's text from the given double value
     */
    public void setValue(double value) {
        setText(doubleFormatter.format(value));
    }

    protected Document createDefaultModel() {
        return new DecimalNumberDocument();
    }

    protected class DecimalNumberDocument extends PlainDocument {

        public void insertString(int offs, String str, AttributeSet a) 
            throws BadLocationException {

            char[] source = str.toCharArray();
            char[] result = new char[source.length];
            int j = 0;

            for (int i = 0; i < result.length; i++) {
                if (Character.isDigit(source[i]) || source[i]=='.' || (!positiveOnly & source[i]=='-'))
                    result[j++] = source[i];
                else {
                    toolkit.beep();
                    //System.err.println("insertString: " + source[i] + " from " + str);
                }
            }
            super.insertString(offs, new String(result, 0, j), a);
        }
    }

}
